@@ -60,12 +60,12 @@ namespace Opm {
           These constructors will make a copy of the src grid, with
           zcorn and or actnum have been adjustments.
         */
-        EclipseGrid(const EclipseGrid& src) = default;
         EclipseGrid(const EclipseGrid& src, const std::vector<int>& actnum);
         EclipseGrid(const EclipseGrid& src, const double* zcorn, const std::vector<int>& actnum);
 
         EclipseGrid(size_t nx, size_t ny, size_t nz,
                     double dx = 1.0, double dy = 1.0, double dz = 1.0);
+        explicit EclipseGrid(const GridDims& gd);
 
         EclipseGrid(const std::array<int, 3>& dims ,
                     const std::vector<double>& coord ,
@@ -208,6 +208,17 @@ namespace Opm {
         void resetACTNUM( const std::vector<int>& actnum);
 
         bool equal(const EclipseGrid& other) const;
+        static bool hasDVDEPTHZKeywords(const Deck&);
+        
+        /*
+          For ALugrid we can *only* use the keyword <DXV, DXYV, DZV, DEPTHZ> so to  
+          initialize a Regular Cartesian Grid; further we need equidistant mesh
+          spacing in each direction to initialize ALuGrid (mandatory for  
+          mesh refinement!).
+        */
+        
+        static bool hasEqualDVDEPTHZ(const Deck&);
+        static bool allEqual(const std::vector<double> &v);
 
     private:
         std::vector<double> m_minpvVector;
@@ -260,11 +271,10 @@ namespace Opm {
         void initCartesianGrid(const Deck&);
         void initDTOPSGrid(const Deck&);
         void initDVDEPTHZGrid(const Deck&);
-        void initGrid(const Deck&);
+        void initGrid(const Deck&, const int* actnum);
         void initCornerPointGrid(const Deck&);
         void assertCornerPointKeywords(const Deck&);
 
-        static bool hasDVDEPTHZKeywords(const Deck&);
         static bool hasDTOPSKeywords(const Deck&);
         static void assertVectorSize(const std::vector<double>& vector, size_t expectedSize, const std::string& msg);
 
