@@ -43,6 +43,8 @@ public:
     // The default constructed fieldProps object is **NOT** usable
     FieldPropsManager() = default;
     FieldPropsManager(const Deck& deck, const Phases& ph, const EclipseGrid& grid, const TableManager& tables);
+    virtual ~FieldPropsManager() = default;
+
     virtual void reset_actnum(const std::vector<int>& actnum);
     const std::string& default_region() const;
     virtual std::vector<int> actnum() const;
@@ -228,16 +230,8 @@ public:
 
     void apply_numerical_aquifers(const NumericalAquifers& aquifers);
 
+    const Fieldprops::TranMap& getTran() const;
 
-    /*
-      When using MPI the FieldPropsManager is typically only assembled on the
-      root node and then distributed to the other nodes afterwards. These
-      methods are support methods for that, the real data used by the
-      transmissibility calculators is in the form of custom 3D fields, they are
-      distributed the same way the rest of the 3D fields are distributed.
-    */
-    virtual std::vector<char> serialize_tran() const;
-    virtual void deserialize_tran(const std::vector<char>& buffer);
 private:
     /*
       Return the keyword values as a std::vector<>. All elements in the return
@@ -274,14 +268,9 @@ private:
     template <typename T>
     std::vector<T> get_global(const std::string& keyword) const;
 
-
     std::shared_ptr<FieldProps> fp;
 };
 
-
-void deserialize_tran(std::unordered_map<std::string, Fieldprops::TranCalculator>& tran,
-                      const std::vector<char>& buffer);
-
 template<class MapType>
 void apply_tran(const std::unordered_map<std::string, Fieldprops::TranCalculator>& tran,
                 const MapType& double_data,
