@@ -27,7 +27,7 @@ class DeckItem;
 
 class SaltsolTable : public SimpleTable {
 public:
-    SaltsolTable( const DeckItem& item );
+    SaltsolTable( const DeckItem& item, const int tableID );
 
     const TableColumn& getSaltsolColumn() const;
 };
