@@ -26,6 +26,7 @@
 #include <set>
 
 #include <opm/input/eclipse/EclipseState/Tables/DenT.hpp>
+#include <opm/input/eclipse/EclipseState/Tables/JouleThomson.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/PvtgTable.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/PvtgwTable.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/PvtgwoTable.hpp>
@@ -66,7 +67,7 @@ namespace Opm {
         explicit TableManager( const Deck& deck );
         TableManager() = default;
 
-        static TableManager serializeObject();
+        static TableManager serializationTestObject();
 
         const TableContainer& getTables( const std::string& tableName ) const;
         const TableContainer& operator[](const std::string& tableName) const;
@@ -93,6 +94,7 @@ namespace Opm {
         const TableContainer& getSsfnTables() const;
         const TableContainer& getRsvdTables() const;
         const TableContainer& getRvvdTables() const;
+        const TableContainer& getRvwvdTables() const;
         const TableContainer& getPbvdTables() const;
         const TableContainer& getPdvdTables() const;
         const TableContainer& getSaltvdTables() const;
@@ -145,6 +147,9 @@ namespace Opm {
         const DenT& WatDenT() const;
         const DenT& GasDenT() const;
         const DenT& OilDenT() const;
+        const JouleThomson& WatJT() const;
+        const JouleThomson& GasJT() const;
+        const JouleThomson& OilJT() const;
         const StandardCond& stCond() const;
         std::size_t gas_comp_index() const;
         const PvtwTable& getPvtwTable() const;
@@ -163,6 +168,8 @@ namespace Opm {
         const ShrateTable& getShrateTable() const;
         const Stone1exTable& getStone1exTable() const;
         const WatdentTable& getWatdentTable() const;
+        const SgofletTable& getSgofletTable() const;
+        const SwofletTable& getSwofletTable() const;
         const std::map<int, PlymwinjTable>& getPlymwinjTables() const;
         const std::map<int, SkprwatTable>& getSkprwatTables() const;
         const std::map<int, SkprpolyTable>& getSkprpolyTables() const;
@@ -194,53 +201,58 @@ namespace Opm {
         {
             auto simpleTables = m_simpleTables;
             auto split = splitSimpleTable(simpleTables);
-            serializer.map(simpleTables);
+            serializer(simpleTables);
             serializer(split.plyshMax);
-            serializer.map(split.plyshMap);
+            serializer(split.plyshMap);
             serializer(split.rockMax);
-            serializer.map(split.rockMap);
-            serializer.vector(m_pvtgTables);
-            serializer.vector(m_pvtgwTables);
-            serializer.vector(m_pvtgwoTables);
-            serializer.vector(m_pvtoTables);
-            serializer.vector(m_pvtsolTables);
-            serializer.vector(m_rock2dTables);
-            serializer.vector(m_rock2dtrTables);
-            m_pvtwTable.serializeOp(serializer);
-            m_pvcdoTable.serializeOp(serializer);
-            m_densityTable.serializeOp(serializer);
-            m_diffCoeffTable.serializeOp(serializer);
-            m_plyvmhTable.serializeOp(serializer);
-            m_rockTable.serializeOp(serializer);
-            m_plmixparTable.serializeOp(serializer);
-            m_shrateTable.serializeOp(serializer);
-            m_stone1exTable.serializeOp(serializer);
-            m_viscrefTable.serializeOp(serializer);
-            m_watdentTable.serializeOp(serializer);
-            serializer.vector(m_pvtwsaltTables);
-            serializer.vector(m_rwgsaltTables);
-            serializer.vector(m_bdensityTables);
-            serializer.vector(m_sdensityTables);
-            serializer.map(m_plymwinjTables);
-            serializer.map(m_skprwatTables);
-            serializer.map(m_skprpolyTables);
-            m_tabdims.serializeOp(serializer);
-            m_regdims.serializeOp(serializer);
-            m_eqldims.serializeOp(serializer);
-            m_aqudims.serializeOp(serializer);
+            serializer(split.rockMap);
+            serializer(m_pvtgTables);
+            serializer(m_pvtgwTables);
+            serializer(m_pvtgwoTables);
+            serializer(m_pvtoTables);
+            serializer(m_pvtsolTables);
+            serializer(m_rock2dTables);
+            serializer(m_rock2dtrTables);
+            serializer(m_pvtwTable);
+            serializer(m_pvcdoTable);
+            serializer(m_densityTable);
+            serializer(m_diffCoeffTable);
+            serializer(m_plyvmhTable);
+            serializer(m_rockTable);
+            serializer(m_plmixparTable);
+            serializer(m_shrateTable);
+            serializer(m_stone1exTable);
+            serializer(m_viscrefTable);
+            serializer(m_watdentTable);
+            serializer(m_sgofletTable);
+            serializer(m_swofletTable);
+            serializer(m_pvtwsaltTables);
+            serializer(m_rwgsaltTables);
+            serializer(m_bdensityTables);
+            serializer(m_sdensityTables);
+            serializer(m_plymwinjTables);
+            serializer(m_skprwatTables);
+            serializer(m_skprpolyTables);
+            serializer(m_tabdims);
+            serializer(m_regdims);
+            serializer(m_eqldims);
+            serializer(m_aqudims);
             serializer(hasImptvd);
             serializer(hasEnptvd);
             serializer(hasEqlnum);
             serializer(hasShrate);
             serializer(jfunc);
-            oilDenT.serializeOp(serializer);
-            gasDenT.serializeOp(serializer);
-            watDenT.serializeOp(serializer);
-            stcond.serializeOp(serializer);
+            serializer(oilDenT);
+            serializer(gasDenT);
+            serializer(watDenT);
+            serializer(oilJT);
+            serializer(gasJT);
+            serializer(watJT);
+            serializer(stcond);
             serializer(m_gas_comp_index);
             serializer(m_rtemp);
             serializer(m_salinity);
-            m_tlmixpar.serializeOp(serializer);
+            serializer(m_tlmixpar);
             if (!serializer.isSerializing()) {
                 m_simpleTables = simpleTables;
                 if (split.plyshMax > 0) {
@@ -356,6 +368,8 @@ namespace Opm {
         Stone1exTable m_stone1exTable;
         ViscrefTable m_viscrefTable;
         WatdentTable m_watdentTable;
+        SgofletTable m_sgofletTable;
+        SwofletTable m_swofletTable;
         std::vector<PvtwsaltTable> m_pvtwsaltTables;
         std::vector<RwgsaltTable> m_rwgsaltTables;
         std::vector<BrineDensityTable> m_bdensityTables;
@@ -379,6 +393,9 @@ namespace Opm {
         DenT oilDenT;
         DenT gasDenT;
         DenT watDenT;
+        JouleThomson oilJT;
+        JouleThomson gasJT;
+        JouleThomson watJT;
         StandardCond stcond;
         std::size_t m_gas_comp_index = 77;
         double m_rtemp {288.7056}; // 60 Fahrenheit in Kelvin
