@@ -20,22 +20,21 @@
 #define SCHEDULE_HPP
 
 #include <cstddef>
+#include <ctime>
 #include <map>
 #include <memory>
 #include <optional>
-#include <ostream>
+#include <iosfwd>
 #include <string>
 #include <unordered_map>
-#include <unordered_set>
 #include <utility>
 #include <vector>
 
-#include <time.h>
-
 #include <opm/input/eclipse/EclipseState/Runspec.hpp>
+#include <opm/input/eclipse/Parser/InputErrorAction.hpp>
+#include <opm/input/eclipse/Python/Python.hpp>
 #include <opm/input/eclipse/Schedule/GasLiftOpt.hpp>
 #include <opm/input/eclipse/Schedule/Group/Group.hpp>
-#include <opm/input/eclipse/Schedule/Group/GTNode.hpp>
 #include <opm/input/eclipse/Schedule/Group/GuideRateConfig.hpp>
 #include <opm/input/eclipse/Schedule/MessageLimits.hpp>
 #include <opm/input/eclipse/Schedule/Network/ExtNetwork.hpp>
@@ -45,18 +44,10 @@
 #include <opm/input/eclipse/Schedule/Well/PAvg.hpp>
 #include <opm/input/eclipse/Schedule/Well/Well.hpp>
 #include <opm/input/eclipse/Schedule/Well/WellTestConfig.hpp>
-#include <opm/input/eclipse/Schedule/Well/WellMatcher.hpp>
 #include <opm/input/eclipse/Schedule/WriteRestartFileEvents.hpp>
 #include <opm/input/eclipse/Schedule/CompletedCells.hpp>
-#include <opm/input/eclipse/Schedule/ScheduleDeck.hpp>
-#include <opm/input/eclipse/Schedule/ScheduleState.hpp>
-#include <opm/input/eclipse/Schedule/RPTConfig.hpp>
 #include <opm/input/eclipse/Schedule/Action/SimulatorUpdate.hpp>
 #include <opm/input/eclipse/Schedule/Action/WGNames.hpp>
-#include <opm/input/eclipse/Parser/ParseContext.hpp>
-
-#include <opm/input/eclipse/Python/Python.hpp>
-
 #include <opm/input/eclipse/Units/UnitSystem.hpp>
 
 namespace Opm
@@ -67,11 +58,13 @@ namespace Opm
     class DeckRecord;
     class EclipseState;
     class FieldPropsManager;
+    class GTNode;
     class ParseContext;
     class SCHEDULESection;
     class SummaryState;
     class ErrorGuard;
     class UDQConfig;
+    class WellMatcher;
 
     namespace RestartIO { struct RstState; }
 
@@ -106,27 +99,27 @@ namespace Opm
         template<class Serializer>
         void serializeOp(Serializer& serializer)
         {
-            m_deck_message_limits.serializeOp(serializer);
-            this->rst_info.serializeOp(serializer);
-            m_runspec.serializeOp(serializer);
-            m_unit_system.serializeOp(serializer);
+            serializer(m_deck_message_limits);
+            serializer(this->rst_info);
+            serializer(m_runspec);
+            serializer(m_unit_system);
             serializer(this->m_input_path);
-            rst_info.serializeOp(serializer);
-            rst_config.serializeOp(serializer);
+            serializer(rst_info);
+            serializer(rst_config);
             serializer(this->output_interval);
             serializer(this->gaslift_opt_active);
         }
 
 
-        static ScheduleStatic serializeObject() {
+        static ScheduleStatic serializationTestObject() {
             auto python = std::make_shared<Python>(Python::Enable::OFF);
             ScheduleStatic st(python);
-            st.m_deck_message_limits = MessageLimits::serializeObject();
-            st.m_runspec = Runspec::serializeObject();
+            st.m_deck_message_limits = MessageLimits::serializationTestObject();
+            st.m_runspec = Runspec::serializationTestObject();
             st.m_unit_system = UnitSystem::newFIELD();
             st.m_input_path = "Some/funny/path";
-            st.rst_config = RSTConfig::serializeObject();
-            st.rst_info = ScheduleRestartInfo::serializeObject();
+            st.rst_config = RSTConfig::serializationTestObject();
+            st.rst_info = ScheduleRestartInfo::serializationTestObject();
             return st;
         }
 
@@ -207,16 +200,16 @@ namespace Opm
                  const std::optional<int>& output_interval = {},
                  const RestartIO::RstState* rst = nullptr);
 
-        static Schedule serializeObject();
+        static Schedule serializationTestObject();
 
         /*
          * If the input deck does not specify a start time, Eclipse's 1. Jan
          * 1983 is defaulted
          */
-        time_t getStartTime() const;
-        time_t posixStartTime() const;
-        time_t posixEndTime() const;
-        time_t simTime(std::size_t timeStep) const;
+        std::time_t getStartTime() const;
+        std::time_t posixStartTime() const;
+        std::time_t posixEndTime() const;
+        std::time_t simTime(std::size_t timeStep) const;
         double seconds(std::size_t timeStep) const;
         double stepLength(std::size_t timeStep) const;
         std::optional<int> exitStatus() const;
@@ -328,35 +321,36 @@ namespace Opm
         template<class Serializer>
         void serializeOp(Serializer& serializer)
         {
-            m_sched_deck.serializeOp(serializer);
-            serializer.vector(snapshots);
-            m_static.serializeOp(serializer);
-            restart_output.serializeOp(serializer);
-            this->completed_cells.serializeOp(serializer);
-            this->action_wgnames.serializeOp(serializer);
+            serializer(this->m_static);
+            serializer(this->m_sched_deck);
+            serializer(this->action_wgnames);
+            serializer(this->exit_status);
+            serializer(this->snapshots);
+            serializer(this->restart_output);
+            serializer(this->completed_cells);
 
-            pack_unpack<PAvg, Serializer>(serializer);
-            pack_unpack<WellTestConfig, Serializer>(serializer);
-            pack_unpack<GConSale, Serializer>(serializer);
-            pack_unpack<GConSump, Serializer>(serializer);
-            pack_unpack<WListManager, Serializer>(serializer);
-            pack_unpack<Network::ExtNetwork, Serializer>(serializer);
-            pack_unpack<Network::Balance, Serializer>(serializer);
-            pack_unpack<RPTConfig, Serializer>(serializer);
-            pack_unpack<Action::Actions, Serializer>(serializer);
-            pack_unpack<UDQActive, Serializer>(serializer);
-            pack_unpack<UDQConfig, Serializer>(serializer);
-            pack_unpack<NameOrder, Serializer>(serializer);
-            pack_unpack<GroupOrder, Serializer>(serializer);
-            pack_unpack<GuideRateConfig, Serializer>(serializer);
-            pack_unpack<GasLiftOpt, Serializer>(serializer);
-            pack_unpack<RFTConfig, Serializer>(serializer);
-            pack_unpack<RSTConfig, Serializer>(serializer);
+            this->template pack_unpack<PAvg>(serializer);
+            this->template pack_unpack<WellTestConfig>(serializer);
+            this->template pack_unpack<GConSale>(serializer);
+            this->template pack_unpack<GConSump>(serializer);
+            this->template pack_unpack<WListManager>(serializer);
+            this->template pack_unpack<Network::ExtNetwork>(serializer);
+            this->template pack_unpack<Network::Balance>(serializer);
+            this->template pack_unpack<RPTConfig>(serializer);
+            this->template pack_unpack<Action::Actions>(serializer);
+            this->template pack_unpack<UDQActive>(serializer);
+            this->template pack_unpack<UDQConfig>(serializer);
+            this->template pack_unpack<NameOrder>(serializer);
+            this->template pack_unpack<GroupOrder>(serializer);
+            this->template pack_unpack<GuideRateConfig>(serializer);
+            this->template pack_unpack<GasLiftOpt>(serializer);
+            this->template pack_unpack<RFTConfig>(serializer);
+            this->template pack_unpack<RSTConfig>(serializer);
 
-            pack_unpack_map<int, VFPProdTable, Serializer>(serializer);
-            pack_unpack_map<int, VFPInjTable, Serializer>(serializer);
-            pack_unpack_map<std::string, Group, Serializer>(serializer);
-            pack_unpack_map<std::string, Well, Serializer>(serializer);
+            this->template pack_unpack_map<int, VFPProdTable>(serializer);
+            this->template pack_unpack_map<int, VFPInjTable>(serializer);
+            this->template pack_unpack_map<std::string, Group>(serializer);
+            this->template pack_unpack_map<std::string, Well>(serializer);
         }
 
         template <typename T, class Serializer>
@@ -365,13 +359,13 @@ namespace Opm
             std::vector<std::size_t> index_list;
 
             if (serializer.isSerializing())
-                pack_state<T>(value_list, index_list);
+                this->template pack_state<T>(value_list, index_list);
 
-            serializer.vector(value_list);
-            serializer.template vector<std::size_t, false>(index_list);
+            serializer(value_list);
+            serializer(index_list);
 
             if (!serializer.isSerializing())
-                unpack_state<T>(value_list, index_list);
+                this->template unpack_state<T>(value_list, index_list);
         }
 
         template <typename T>
@@ -389,7 +383,7 @@ namespace Opm
 
         template <typename T>
         void pack_state(std::vector<T>& value_list, std::vector<std::size_t>& index_list) const {
-            auto unique_values = this->unique<T>();
+            auto unique_values = this->template unique<T>();
             for (auto& [index, value] : unique_values) {
                 value_list.push_back( std::move(value) );
                 index_list.push_back( index );
@@ -425,7 +419,7 @@ namespace Opm
             if (serializer.isSerializing())
                 pack_map<K,T>(value_list, index_list);
 
-            serializer.vector(value_list);
+            serializer(value_list);
             serializer(index_list);
 
             if (!serializer.isSerializing())
@@ -504,6 +498,7 @@ namespace Opm
             ErrorGuard& errors;
             SimulatorUpdate * sim_update;
             const std::unordered_map<std::string, double> * target_wellpi;
+            std::unordered_map<std::string, double>* wpimult_global_factor;
             const ScheduleGrid& grid;
 
             HandlerContext(const ScheduleBlock& block_,
@@ -515,7 +510,8 @@ namespace Opm
                            const ParseContext& parseContext_,
                            ErrorGuard& errors_,
                            SimulatorUpdate * sim_update_,
-                           const std::unordered_map<std::string, double> * target_wellpi_)
+                           const std::unordered_map<std::string, double> * target_wellpi_,
+                           std::unordered_map<std::string, double>* wpimult_global_factor_)
             : block(block_)
             , keyword(keyword_)
             , currentStep(currentStep_)
@@ -525,6 +521,7 @@ namespace Opm
             , errors(errors_)
             , sim_update(sim_update_)
             , target_wellpi(target_wellpi_)
+            , wpimult_global_factor(wpimult_global_factor_)
             , grid(grid_)
             {}
 
@@ -535,6 +532,11 @@ namespace Opm
 
         };
 
+        // Please update the member functions
+        //   - operator==(const Schedule&) const
+        //   - serializationTestObject()
+        //   - serializeOp(Serializer&)
+        // when you update/change this list of data members.
         ScheduleStatic m_static;
         ScheduleDeck m_sched_deck;
         Action::WGNames action_wgnames;
@@ -574,7 +576,8 @@ namespace Opm
                                     ErrorGuard& errors,
                                     const ScheduleGrid& grid,
                                     const std::unordered_map<std::string, double> * target_wellpi,
-                                    const std::string& prefix);
+                                    const std::string& prefix,
+                                    const bool log_to_debug = false);
         void addACTIONX(const Action::ActionX& action);
         void addGroupToGroup( const std::string& parent_group, const std::string& child_group);
         void addGroup(const std::string& groupName , std::size_t timeStep);
@@ -586,12 +589,14 @@ namespace Opm
         void handleKeyword(std::size_t currentStep,
                            const ScheduleBlock& block,
                            const DeckKeyword& keyword,
-                           const ParseContext& parseContext, ErrorGuard& errors,
+                           const ParseContext& parseContext,
+                           ErrorGuard& errors,
                            const ScheduleGrid& grid,
                            const std::vector<std::string>& matching_wells,
-                           bool runtime,
-                           SimulatorUpdate * sim_update,
-                           const std::unordered_map<std::string, double> * target_wellpi);
+                           bool actionx_mode,
+                           SimulatorUpdate* sim_update,
+                           const std::unordered_map<std::string, double>* target_wellpi,
+                           std::unordered_map<std::string, double>* wpimult_global_factor = nullptr);
 
         void prefetch_cell_properties(const ScheduleGrid& grid, const DeckKeyword& keyword);
         void store_wgnames(const DeckKeyword& keyword);
@@ -600,6 +605,7 @@ namespace Opm
         void invalidNamePattern( const std::string& namePattern, const HandlerContext& context) const;
         static std::string formatDate(std::time_t t);
         std::string simulationDays(std::size_t currentStep) const;
+        void applyGlobalWPIMULT( const std::unordered_map<std::string, double>& wpimult_global_factor);
 
         bool must_write_rst_file(std::size_t report_step) const;
 
