@@ -256,9 +256,9 @@ namespace Opm {
             }
 
 
-            static map_member<K,T> serializeObject() {
+            static map_member<K,T> serializationTestObject() {
                 map_member<K,T> map_object;
-                T value_object = T::serializeObject();
+                T value_object = T::serializationTestObject();
                 K key = value_object.name();
                 map_object.m_data.emplace( key, std::make_shared<T>( std::move(value_object) ));
                 return map_object;
@@ -295,7 +295,7 @@ namespace Opm {
         bool first_in_year() const;
 
         bool operator==(const ScheduleState& other) const;
-        static ScheduleState serializeObject();
+        static ScheduleState serializationTestObject();
 
         void update_tuning(Tuning tuning);
         Tuning& tuning();
@@ -486,15 +486,15 @@ namespace Opm {
             serializer(m_sumthin);
             serializer(this->m_rptonly);
             serializer(this->next_tstep);
-            m_tuning.serializeOp(serializer);
-            m_nupcol.serializeOp(serializer);
-            m_oilvap.serializeOp(serializer);
-            m_events.serializeOp(serializer);
-            m_wellgroup_events.serializeOp(serializer);
-            serializer.vector(m_geo_keywords);
-            m_message_limits.serializeOp(serializer);
+            serializer(m_tuning);
+            serializer(m_nupcol);
+            serializer(m_oilvap);
+            serializer(m_events);
+            serializer(m_wellgroup_events);
+            serializer(m_geo_keywords);
+            serializer(m_message_limits);
             serializer(m_whistctl_mode);
-            serializer.template map<WellPIMapType, false>(target_wellpi);
+            serializer(target_wellpi);
         }
 
 
