@@ -185,7 +185,7 @@ public:
         GuideRateTarget guide_phase;
         double scale_factor;
 
-        static WellGuideRate serializeObject()
+        static WellGuideRate serializationTestObject()
         {
             WellGuideRate result;
             result.available = true;
@@ -231,6 +231,7 @@ public:
         double temperature;
         int    vfp_table_number;
         bool   prediction_mode;
+        double rs_rv_inj;
 
         bool hasControl(InjectorCMode cmode_arg) const {
             return (this->controls & static_cast<int>(cmode_arg)) != 0;
@@ -261,13 +262,15 @@ public:
         InjectorType injectorType;
         InjectorCMode controlMode;
 
+        double rsRvInj;
+
         bool operator==(const WellInjectionProperties& other) const;
         bool operator!=(const WellInjectionProperties& other) const;
 
         WellInjectionProperties();
         WellInjectionProperties(const UnitSystem& units, const std::string& wname);
 
-        static WellInjectionProperties serializeObject();
+        static WellInjectionProperties serializationTestObject();
 
         void handleWELTARG(WELTARGCMode cmode, const UDAValue& new_arg, double SIFactorP);
         void handleWCONINJE(const DeckRecord& record, bool availableForGroupControl, const std::string& well_name);
@@ -298,6 +301,7 @@ public:
         void setBHPLimit(const double limit);
         InjectionControls controls(const UnitSystem& unit_system, const SummaryState& st, double udq_default) const;
         bool updateUDQActive(const UDQConfig& udq_config, UDQActive& active) const;
+        bool updateUDQActive(const UDQConfig& udq_config, const WELTARGCMode cmode, UDQActive& active) const;
         void update_uda(const UDQConfig& udq_config, UDQActive& udq_active, UDAControl control, const UDAValue& value);
         void handleWTMULT(Well::WELTARGCMode cmode, double factor);
 
@@ -305,10 +309,10 @@ public:
         void serializeOp(Serializer& serializer)
         {
             serializer(name);
-            surfaceInjectionRate.serializeOp(serializer);
-            reservoirInjectionRate.serializeOp(serializer);
-            BHPTarget.serializeOp(serializer);
-            THPTarget.serializeOp(serializer);
+            serializer(surfaceInjectionRate);
+            serializer(reservoirInjectionRate);
+            serializer(BHPTarget);
+            serializer(THPTarget);
             serializer(bhp_hist_limit);
             serializer(thp_hist_limit);
             serializer(temperature);
@@ -319,6 +323,7 @@ public:
             serializer(injectionControls);
             serializer(injectorType);
             serializer(controlMode);
+            serializer(rsRvInj);
         }
     };
 
@@ -401,7 +406,7 @@ public:
         WellProductionProperties();
         WellProductionProperties(const UnitSystem& units, const std::string& name_arg);
 
-        static WellProductionProperties serializeObject();
+        static WellProductionProperties serializationTestObject();
 
         bool hasProductionControl(ProducerCMode controlModeArg) const {
             return (m_productionControls & static_cast<int>(controlModeArg)) != 0;
@@ -426,6 +431,7 @@ public:
         void clearControls();
         ProductionControls controls(const SummaryState& st, double udq_default) const;
         bool updateUDQActive(const UDQConfig& udq_config, UDQActive& active) const;
+        bool updateUDQActive(const UDQConfig& udq_config, const WELTARGCMode cmode, UDQActive& active) const;
         void update_uda(const UDQConfig& udq_config, UDQActive& udq_active, UDAControl control, const UDAValue& value);
 
         void setBHPLimit(const double limit);
@@ -436,14 +442,14 @@ public:
         void serializeOp(Serializer& serializer)
         {
             serializer(name);
-            OilRate.serializeOp(serializer);
-            WaterRate.serializeOp(serializer);
-            GasRate.serializeOp(serializer);
-            LiquidRate.serializeOp(serializer);
-            ResVRate.serializeOp(serializer);
-            BHPTarget.serializeOp(serializer);
-            THPTarget.serializeOp(serializer);
-            ALQValue.serializeOp(serializer);
+            serializer(OilRate);
+            serializer(WaterRate);
+            serializer(GasRate);
+            serializer(LiquidRate);
+            serializer(ResVRate);
+            serializer(BHPTarget);
+            serializer(THPTarget);
+            serializer(ALQValue);
             serializer(bhp_hist_limit);
             serializer(thp_hist_limit);
             serializer(BHPH);
@@ -501,12 +507,13 @@ public:
          const UnitSystem& unit_system,
          double udq_undefined);
 
-    static Well serializeObject();
+    static Well serializationTestObject();
 
     bool isMultiSegment() const;
     bool isAvailableForGroupControl() const;
     double getGuideRate() const;
     GuideRateTarget getGuideRatePhase() const;
+    GuideRateTarget getRawGuideRatePhase() const;
     double getGuideRateScalingFactor() const;
 
     bool hasBeenDefined(size_t timeStep) const;
@@ -619,6 +626,7 @@ public:
     bool handleWELOPENConnections(const DeckRecord& record, Connection::State status);
     bool handleCOMPLUMP(const DeckRecord& record);
     bool handleWPIMULT(const DeckRecord& record);
+    bool applyGlobalWPIMULT(double scale_factor);
 
     void filterConnections(const ActiveGridCells& grid);
     ProductionControls productionControls(const SummaryState& st) const;
@@ -655,7 +663,7 @@ public:
         serializer(headJ);
         serializer(ref_depth);
         serializer(wpave_ref_depth);
-        unit_system.serializeOp(serializer);
+        serializer(unit_system);
         serializer(udq_undefined);
         serializer(status);
         serializer(drainage_radius);
@@ -663,8 +671,8 @@ public:
         serializer(automatic_shutin);
         serializer(pvt_table);
         serializer(gas_inflow);
-        wtype.serializeOp(serializer);
-        guide_rate.serializeOp(serializer);
+        serializer(wtype);
+        serializer(guide_rate);
         serializer(efficiency_factor);
         serializer(solvent_fraction);
         serializer(has_produced);
@@ -681,13 +689,15 @@ public:
         serializer(injection);
         serializer(segments);
         serializer(wvfpexp);
-        m_pavg.serializeOp(serializer);
+        serializer(m_pavg);
     }
 
 private:
     void switchToInjector();
     void switchToProducer();
 
+    GuideRateTarget preferredPhaseAsGuideRatePhase() const;
+
     std::string wname;
     std::string group_name;
     std::size_t init_step;
