@@ -28,7 +28,6 @@
 #include <opm/input/eclipse/Units/Dimension.hpp>
 #include <opm/input/eclipse/Schedule/UDQ/UDQEnums.hpp>
 
-
 namespace Opm {
 
     class UnitSystem {
@@ -60,6 +59,7 @@ namespace Opm {
             gas_surface_rate,
             rate,
             geometric_volume_rate,
+            pipeflow_velocity,
             transmissibility,
             effective_Kh,
             mass,
@@ -81,6 +81,7 @@ namespace Opm {
             aicd_strength,
             polymer_density,
             salinity,
+            gas_oil_ratio_rate,
             _count // New entries must be added *before* this
         };
 
@@ -88,7 +89,7 @@ namespace Opm {
         explicit UnitSystem(UnitType unit = UnitType::UNIT_TYPE_METRIC);
         explicit UnitSystem(const std::string& deck_name);
 
-        static UnitSystem serializeObject();
+        static UnitSystem serializationTestObject();
 
         const std::string& getName() const;
         UnitType getType() const;
@@ -132,7 +133,7 @@ namespace Opm {
         {
             serializer(m_name);
             serializer(m_unittype);
-            serializer.map(m_dimensions);
+            serializer(m_dimensions);
             serializer(m_use_count);
             if (!serializer.isSerializing())
                 init();
@@ -170,8 +171,7 @@ namespace Opm {
         */
         mutable std::size_t m_use_count = 0;
     };
-}
 
+} // namespace Opm
 
-#endif
-
+#endif  // UNITSYSTEM_H
