@@ -71,7 +71,8 @@ namespace Opm {
                 brine            = (1 << 17),
                 alq              = (1 << 18),
                 tracer           = (1 << 19),
-                micp           = (1 << 20)
+                micp             = (1 << 20),
+                vaporized_water     = (1 << 21)
             };
 
             using enum_size = std::underlying_type< opt >::type;
@@ -102,7 +103,65 @@ namespace Opm {
 
             bool operator==(const Rates& rat2) const;
 
-        inline void init_json(Json::JsonObject& json_data) const;
+            inline void init_json(Json::JsonObject& json_data) const;
+
+            template<class Serializer>
+            void serializeOp(Serializer& serializer)
+            {
+                serializer(mask);
+                serializer(wat);
+                serializer(oil);
+                serializer(gas);
+                serializer(polymer);
+                serializer(solvent);
+                serializer(energy);
+                serializer(dissolved_gas);
+                serializer(vaporized_oil);
+                serializer(reservoir_water);
+                serializer(reservoir_oil);
+                serializer(reservoir_gas);
+                serializer(productivity_index_water);
+                serializer(productivity_index_oil);
+                serializer(productivity_index_gas);
+                serializer(well_potential_water);
+                serializer(well_potential_oil);
+                serializer(well_potential_gas);
+                serializer(brine);
+                serializer(alq);
+                serializer(tracer);
+                serializer(micp);
+                serializer(vaporized_water);
+            }
+
+            static Rates serializationTestObject()
+            {
+                Rates rat1;
+                rat1.set(opt::wat, 1.0);
+                rat1.set(opt::oil, 2.0);
+                rat1.set(opt::gas, 3.0);
+                rat1.set(opt::polymer, 4.0);
+                rat1.set(opt::solvent, 5.0);
+                rat1.set(opt::energy, 6.0);
+                rat1.set(opt::dissolved_gas, 7.0);
+                rat1.set(opt::vaporized_oil, 8.0);
+                rat1.set(opt::reservoir_water, 9.0);
+                rat1.set(opt::reservoir_oil, 10.0);
+                rat1.set(opt::reservoir_gas, 11.0);
+                rat1.set(opt::productivity_index_water, 12.0);
+                rat1.set(opt::productivity_index_oil, 13.0);
+                rat1.set(opt::productivity_index_gas, 14.0);
+                rat1.set(opt::well_potential_water, 15.0);
+                rat1.set(opt::well_potential_oil, 16.0);
+                rat1.set(opt::well_potential_gas, 17.0);
+                rat1.set(opt::brine, 18.0);
+                rat1.set(opt::alq, 19.0);
+                rat1.set(opt::micp, 21.0);
+                rat1.set(opt::vaporized_water, 22.0);
+                rat1.tracer.insert({"test_tracer", 1.0});
+
+                return rat1;
+            }
+
         private:
             double& get_ref( opt );
             double& get_ref( opt, const std::string& tracer_name );
@@ -132,6 +191,7 @@ namespace Opm {
             double alq = 0.0;
             std::map<std::string, double> tracer;
             double micp = 0.0;
+            double vaporized_water = 0.0;
     };
 
     struct Connection {
@@ -157,7 +217,8 @@ namespace Opm {
                    cell_pressure == conn2.cell_pressure &&
                    cell_saturation_water == conn2.cell_saturation_water &&
                    cell_saturation_gas == conn2.cell_saturation_gas &&
-                   effective_Kh == conn2.effective_Kh;
+                   effective_Kh == conn2.effective_Kh &&
+                   trans_factor == conn2.trans_factor;
         }
 
         template <class MessageBufferType>
@@ -166,6 +227,27 @@ namespace Opm {
         void read(MessageBufferType& buffer);
 
         inline void init_json(Json::JsonObject& json_data) const;
+
+        template<class Serializer>
+        void serializeOp(Serializer& serializer)
+        {
+            serializer(index);
+            serializer(rates);
+            serializer(pressure);
+            serializer(reservoir_rate);
+            serializer(cell_pressure);
+            serializer(cell_saturation_water);
+            serializer(cell_saturation_gas);
+            serializer(effective_Kh);
+            serializer(trans_factor);
+        }
+
+        static Connection serializationTestObject()
+        {
+            return Connection{1, Rates::serializationTestObject(),
+                              2.0, 3.0, 4.0, 5.0,
+                              6.0, 7.0, 8.0};
+        }
     };
 
     class SegmentPressures {
@@ -205,6 +287,24 @@ namespace Opm {
             }
         }
 
+        template<class Serializer>
+        void serializeOp(Serializer& serializer)
+        {
+            serializer(values_);
+        }
+
+        static SegmentPressures serializationTestObject()
+        {
+            SegmentPressures spres;
+            spres[Value::Pressure] = 1.0;
+            spres[Value::PDrop] = 2.0;
+            spres[Value::PDropHydrostatic] = 3.0;
+            spres[Value::PDropAccel] = 4.0;
+            spres[Value::PDropFriction] = 5.0;
+
+            return spres;
+        }
+
     private:
         constexpr static std::size_t numvals = 5;
 
@@ -233,6 +333,22 @@ namespace Opm {
 
         template <class MessageBufferType>
         void read(MessageBufferType& buffer);
+
+        template<class Serializer>
+        void serializeOp(Serializer& serializer)
+        {
+            serializer(rates);
+            serializer(pressures);
+            serializer(segNumber);
+        }
+
+        static Segment serializationTestObject()
+        {
+            return Segment{Rates::serializationTestObject(),
+                           SegmentPressures::serializationTestObject(),
+                           10
+                   };
+        }
     };
 
     struct CurrentControl {
@@ -271,6 +387,22 @@ namespace Opm {
 
         template <class MessageBufferType>
         void read(MessageBufferType& buffer);
+
+        template<class Serializer>
+        void serializeOp(Serializer& serializer)
+        {
+            serializer(isProducer);
+            serializer(prod);
+            serializer(inj);
+        }
+
+        static CurrentControl serializationTestObject()
+        {
+          return CurrentControl{false,
+                                ::Opm::Well::ProducerCMode::BHP,
+                                ::Opm::Well::InjectorCMode::GRUP
+                 };
+        }
     };
 
     struct Well {
@@ -321,15 +453,46 @@ namespace Opm {
 
         bool operator==(const Well& well2) const
         {
-          return rates == well2.rates &&
-                 bhp == well2.bhp &&
-                 thp == well2.thp &&
-                 temperature == well2.temperature &&
-                 control == well2.control &&
-                 connections == well2.connections &&
-                 segments == well2.segments &&
-                 current_control == well2.current_control &&
-                 guide_rates == well2.guide_rates;
+            return rates == well2.rates &&
+                   bhp == well2.bhp &&
+                   thp == well2.thp &&
+                   temperature == well2.temperature &&
+                   control == well2.control &&
+                   dynamicStatus == well2.dynamicStatus &&
+                   connections == well2.connections &&
+                   segments == well2.segments &&
+                   current_control == well2.current_control &&
+                   guide_rates == well2.guide_rates;
+        }
+
+        template<class Serializer>
+        void serializeOp(Serializer& serializer)
+        {
+            serializer(rates);
+            serializer(bhp);
+            serializer(thp);
+            serializer(temperature);
+            serializer(control);
+            serializer(dynamicStatus);
+            serializer(connections);
+            serializer(segments);
+            serializer(current_control);
+            serializer(guide_rates);
+        }
+
+        static Well serializationTestObject()
+        {
+            return Well{Rates::serializationTestObject(),
+                        1.0,
+                        2.0,
+                        3.0,
+                        4,
+                        ::Opm::Well::Status::SHUT,
+                        {Connection::serializationTestObject()},
+                        {{0, Segment::serializationTestObject()}},
+                        CurrentControl::serializationTestObject(),
+                        GuideRateValue::serializationTestObject()
+                   };
         }
     };
 
@@ -406,6 +569,19 @@ namespace Opm {
             return json_data;
         }
 
+        template<class Serializer>
+        void serializeOp(Serializer& serializer)
+        {
+            serializer(static_cast<std::map<std::string,Well>&>(*this));
+        }
+
+        static Wells serializationTestObject()
+        {
+            Wells w;
+            w.insert({"test_well", Well::serializationTestObject()});
+
+            return w;
+        }
     };
 
 
@@ -486,7 +662,8 @@ namespace Opm {
              brine == rate.brine &&
              alq == rate.alq &&
              tracer == rate.tracer &&
-             micp == rate.micp;
+             micp == rate.micp &&
+             vaporized_water == rate.vaporized_water;
     }
 
 
@@ -523,6 +700,7 @@ namespace Opm {
             case opt::tracer: /* Should _not_ be called with tracer argument */
                 break;
             case opt::micp: return this->micp;
+            case opt::vaporized_water: return this->vaporized_water;
         }
 
         throw std::invalid_argument(
@@ -603,6 +781,7 @@ namespace Opm {
                 buffer.write(rate);
             }
             buffer.write(this->micp);
+            buffer.write(this->vaporized_water);
     }
 
     template <class MessageBufferType>
@@ -717,6 +896,7 @@ namespace Opm {
                 this->tracer.emplace(tracer_name, tracer_rate);
             }
             buffer.read(this->micp);
+            buffer.read(this->vaporized_water);
     }
 
    template <class MessageBufferType>
