@@ -1,5 +1,6 @@
 #include <ctime>
 #include <chrono>
+#include <map>
 
 #include <opm/input/eclipse/Parser/Parser.hpp>
 #include <opm/input/eclipse/Deck/Deck.hpp>
@@ -9,6 +10,7 @@
 
 #include <opm/input/eclipse/Schedule/Schedule.hpp>
 
+#include <fmt/format.h>
 #include <pybind11/stl.h>
 #include <pybind11/chrono.h>
 #include "export.hpp"
@@ -56,6 +58,57 @@ namespace {
         throw py::key_error( name );
     }
 
+    std::map<std::string, double> get_production_properties(
+        const Schedule& sch, const std::string& well_name, const size_t& report_step)
+    {
+        const Well* well = nullptr;
+        try{
+            well = &(sch.getWell( well_name, report_step ));
+        } catch( const std::out_of_range& e ) {
+            throw py::index_error( fmt::format("well {} is not defined", well_name ));
+        }
+        if (well->isProducer()) {
+            auto& prod_prop = well->getProductionProperties();
+            return {
+                { "oil_rate", prod_prop.OilRate.get<double>() },
+                { "gas_rate", prod_prop.GasRate.get<double>() },
+                { "water_rate", prod_prop.WaterRate.get<double>() },
+                { "liquid_rate", prod_prop.LiquidRate.get<double>() },
+                { "resv_rate", prod_prop.ResVRate.get<double>() },
+                { "bhp_target", prod_prop.BHPTarget.get<double>() },
+                { "thp_target", prod_prop.THPTarget.get<double>() },
+                { "alq_value", prod_prop.ALQValue.get<double>() },
+            };
+        }
+        else {
+            throw py::key_error( fmt::format("well {} is not a producer", well_name) );
+        }
+    }
+
+    std::map<std::string, double> get_injection_properties(
+        const Schedule& sch, const std::string& well_name, const size_t& report_step)
+    {
+        const Well* well = nullptr;
+        try{
+            well = &(sch.getWell( well_name, report_step ));
+        } catch( const std::out_of_range& e ) {
+            throw py::index_error( fmt::format("well {}: invalid well name", well_name ));
+        }
+
+        if (well->isInjector()) {
+            auto& inj_prop = well->getInjectionProperties();
+            return {
+                { "surf_inj_rate", inj_prop.surfaceInjectionRate.get<double>() },
+                { "resv_inj_rate", inj_prop.reservoirInjectionRate.get<double>() },
+                { "bhp_target", inj_prop.BHPTarget.get<double>() },
+                { "thp_target", inj_prop.THPTarget.get<double>() },
+            };
+        }
+        else {
+            throw py::key_error( fmt::format("well {} is not an injector", well_name) );
+        }
+    }
+
     system_clock::time_point get_start_time( const Schedule& s ) {
         return datetime(s.posixStartTime());
     }
@@ -157,6 +210,8 @@ void python::common::export_Schedule(py::module& module) {
     .def( "open_well", &Schedule::open_well)
     .def( "stop_well", &Schedule::stop_well)
     .def( "get_wells", &Schedule::getWells)
+    .def( "get_injection_properties", &get_injection_properties, py::arg("well_name"), py::arg("report_step"))
+    .def( "get_production_properties", &get_production_properties, py::arg("well_name"), py::arg("report_step"))
     .def("well_names", py::overload_cast<const std::string&>(&Schedule::wellNames, py::const_))
     .def( "get_well", &get_well)
     .def( "insert_keywords",
