@@ -21,6 +21,7 @@
 #define _USE_MATH_DEFINES
 
 #include <opm/input/eclipse/EclipseState/Grid/EclipseGrid.hpp>
+#include <opm/input/eclipse/EclipseState/Grid/FieldProps.hpp>
 
 #include <opm/common/ErrorMacros.hpp>
 #include <opm/common/OpmLog/OpmLog.hpp>
@@ -124,6 +125,25 @@ EclipseGrid::EclipseGrid(const std::string& fileName )
 }
 
 
+EclipseGrid::EclipseGrid(const GridDims& gd)
+    : GridDims(gd),
+      m_minpvMode(MinpvMode::ModeEnum::Inactive),
+      m_pinchoutMode(PinchMode::ModeEnum::TOPBOT),
+      m_multzMode(PinchMode::ModeEnum::TOP),
+      m_pinchGapMode(PinchMode::ModeEnum::GAP)
+{
+    this->m_nactive = this->getCartesianSize();
+    this->active_volume = std::nullopt;
+    // Nothing else initialized. Leaving in particular as empty:
+    // m_actnum,
+    // m_global_to_active,
+    // m_active_to_global.
+    //
+    // EclipseGrid will only be usable for constructing Box objects with
+    // all cells active.
+}
+
+
 EclipseGrid::EclipseGrid(size_t nx, size_t ny , size_t nz,
                          double dx, double dy, double dz)
     : GridDims(nx, ny, nz),
@@ -254,39 +274,11 @@ EclipseGrid::EclipseGrid(const Deck& deck, const int * actnum)
 
     updateNumericalAquiferCells(deck);
 
-    initGrid(deck);
+    initGrid(deck, actnum);
 
     if (deck.hasKeyword<ParserKeywords::MAPAXES>())
         this->m_mapaxes = std::make_optional<MapAxes>( deck );
 
-    if (actnum != nullptr) {
-        resetACTNUM(actnum);
-    } else {
-        if (m_useActnumFromGdfile){
-            // actnum already reset in initBinaryGrid
-        } else {
-            if (deck.hasKeyword<ParserKeywords::ACTNUM>()) {
-                const auto& actnumData = deck.get<ParserKeywords::ACTNUM>().back().getIntData();
-                /*
-                  Would have liked to fail hard in the case where the size of the
-                  ACTNUM array disagrees with nx*ny*nz; but it is possible to embed
-                  the ACTNUM keyword in a BOX / ENDBOX pair and in that case it is
-                  legitimate with size(ACTNUM) < nx*ny*nz.
-
-                  If size(actnum) != nx*ny*nz it is ignored here, however it is
-                  taken into account when creating an final actnum in the field
-                  property manager.
-                */
-                if (actnumData.size() == getCartesianSize())
-                    resetACTNUM( actnumData);
-                else
-                    resetACTNUM();
-            } else {
-                resetACTNUM();
-            }
-        }
-    }
-
     /*
       The GRIDUNIT handling is simplified compared to the full specification:
 
@@ -320,8 +312,8 @@ EclipseGrid::EclipseGrid(const Deck& deck, const int * actnum)
         return this->m_circle;
     }
 
-    void EclipseGrid::initGrid(const Deck& deck) {
-
+    void EclipseGrid::initGrid(const Deck& deck, const int* actnum)
+    {
         if (deck.hasKeyword<ParserKeywords::RADIAL>()) {
             initCylindricalGrid(deck );
         } else if (deck.hasKeyword<ParserKeywords::SPIDER>()) {
@@ -353,15 +345,22 @@ EclipseGrid::EclipseGrid(const Deck& deck, const int * actnum)
         }
 
         if (deck.hasKeyword<ParserKeywords::MINPV>() && deck.hasKeyword<ParserKeywords::MINPVFIL>()) {
-            throw std::invalid_argument("Can not have both MINPV and MINPVFIL in deck.");
+            throw std::invalid_argument("Can not have both MINPV and MINPVFIL in the deck.");
+        } else if(deck.hasKeyword<ParserKeywords::MINPORV>() && deck.hasKeyword<ParserKeywords::MINPVFIL>()) {   
+            throw std::invalid_argument("Can not have both MINPORV and MINPVFIL in the deck.");
         }
-
+        
         m_minpvVector.resize(getCartesianSize(), 0.0);
         if (deck.hasKeyword<ParserKeywords::MINPV>()) {
             const auto& record = deck.get<ParserKeywords::MINPV>( ).back().getRecord(0);
             const auto& item = record.getItem<ParserKeywords::MINPV::VALUE>( );
             std::fill(m_minpvVector.begin(), m_minpvVector.end(), item.getSIDouble(0));
             m_minpvMode = MinpvMode::ModeEnum::EclSTD;
+        } else if (deck.hasKeyword<ParserKeywords::MINPORV>()) {
+            const auto& record = deck.get<ParserKeywords::MINPORV>( ).back().getRecord(0);
+            const auto& item = record.getItem<ParserKeywords::MINPORV::VALUE>( );
+            std::fill(m_minpvVector.begin(), m_minpvVector.end(), item.getSIDouble(0));
+            m_minpvMode = MinpvMode::ModeEnum::EclSTD;
         } else if(deck.hasKeyword<ParserKeywords::MINPVV>()) {
             // We should use the grid properties to support BOX, but then we need the eclipseState
             const auto& record = deck.get<ParserKeywords::MINPVV>( ).back().getRecord(0);
@@ -369,6 +368,16 @@ EclipseGrid::EclipseGrid(const Deck& deck, const int * actnum)
             m_minpvMode = MinpvMode::ModeEnum::EclSTD;
         }
 
+        if (actnum != nullptr) {
+            this->resetACTNUM(actnum);
+        }
+        else if (! this->m_useActnumFromGdfile) {
+            const auto fp = FieldProps {
+                deck, EclipseGrid { static_cast<GridDims&>(*this) }
+            };
+
+            this->resetACTNUM(fp.actnumRaw());
+        }
     }
 
     void EclipseGrid::initGridFromEGridFile(Opm::EclIO::EclFile& egridfile, std::string fileName){
@@ -478,6 +487,9 @@ EclipseGrid::EclipseGrid(const Deck& deck, const int * actnum)
     }
 
     size_t EclipseGrid::activeIndex(size_t globalIndex) const {
+        if (m_global_to_active.empty()) {
+            return globalIndex;
+        }
 
         if (m_global_to_active[ globalIndex] == -1) {
             throw std::invalid_argument("Input argument does not correspond to an active cell");
@@ -1136,33 +1148,19 @@ EclipseGrid::EclipseGrid(const Deck& deck, const int * actnum)
         this->resetACTNUM(actnum);
     }
 
-    void EclipseGrid::initCornerPointGrid(const Deck& deck) {
-        assertCornerPointKeywords(deck);
-        {
-            const auto& ZCORNKeyWord = deck.get<ParserKeywords::ZCORN>().back();
-            const auto& COORDKeyWord = deck.get<ParserKeywords::COORD>().back();
+    void EclipseGrid::initCornerPointGrid(const Deck& deck)
+    {
+        this->assertCornerPointKeywords(deck);
 
-            const std::vector<double>& zcorn = ZCORNKeyWord.getSIDoubleData();
-            const std::vector<double>& coord = COORDKeyWord.getSIDoubleData();
+        OpmLog::info(fmt::format("\nCreating corner-point grid from "
+                                 "keywords COORD, ZCORN and others"));
 
-            int * actnum = nullptr;
-            std::vector<int> actnumVector;
+        const auto& coord = deck.get<ParserKeywords::COORD>().back();
+        const auto& zcorn = deck.get<ParserKeywords::ZCORN>().back();
 
-            if (deck.hasKeyword<ParserKeywords::ACTNUM>()) {
-                 const auto& actnumKeyword = deck.get<ParserKeywords::ACTNUM>().back();
-                actnumVector = actnumKeyword.getIntData();
-
-                if (actnumVector.size() != this->getCartesianSize())
-                    throw std::invalid_argument("ACTNUM vector has wrong size");
-
-                actnum = actnumVector.data();
-                OpmLog::info(fmt::format("\nCreating cornerpoint grid from keywords ZCORN, COORD and ACTNUM"));
-            } else
-                OpmLog::info(fmt::format("\nCreating cornerpoint grid from keywords ZCORN and COORD"));
-
-
-            initCornerPointGrid( coord , zcorn, actnum);
-        }
+        this->initCornerPointGrid(coord.getSIDoubleData(),
+                                  zcorn.getSIDoubleData(),
+                                  nullptr);
     }
 
     bool EclipseGrid::hasCornerPointKeywords(const Deck& deck) {
@@ -1237,7 +1235,21 @@ EclipseGrid::EclipseGrid(const Deck& deck, const int * actnum)
         else
             return false;
     }
-
+    
+    bool EclipseGrid::hasEqualDVDEPTHZ(const Deck& deck) {
+        const std::vector<double>& DXV = deck.get<ParserKeywords::DXV>().back().getSIDoubleData();
+        const std::vector<double>& DYV = deck.get<ParserKeywords::DYV>().back().getSIDoubleData();
+        const std::vector<double>& DZV = deck.get<ParserKeywords::DZV>().back().getSIDoubleData();
+        const std::vector<double>& DEPTHZ = deck.get<ParserKeywords::DEPTHZ>().back().getSIDoubleData();
+        if (EclipseGrid::allEqual(DXV) &&
+            EclipseGrid::allEqual(DYV)&&
+            EclipseGrid::allEqual(DZV)&& 
+            EclipseGrid::allEqual(DEPTHZ))
+            return true;
+        else
+            return false;
+    }
+    
     bool EclipseGrid::hasDTOPSKeywords(const Deck& deck) {
         if ((deck.hasKeyword<ParserKeywords::DX>() || deck.hasKeyword<ParserKeywords::DXV>()) &&
             (deck.hasKeyword<ParserKeywords::DY>() || deck.hasKeyword<ParserKeywords::DYV>()) &&
@@ -1368,7 +1380,12 @@ std::vector<double> EclipseGrid::createDVector(const std::array<int,3>& dims, st
             }
         }
     }
-
+    
+    bool EclipseGrid::allEqual(const std::vector<double> &v) {
+        auto comp = [](double x, double y) { return std::fabs(x - y) < 1e-12; };
+        return std::equal(v.begin() + 1, v.end(), v.begin(), comp);
+    }
+    
     bool EclipseGrid::equal(const EclipseGrid& other) const {
 
         //double reltol = 1.0e-6;
@@ -1410,8 +1427,11 @@ std::vector<double> EclipseGrid::createDVector(const std::array<int,3>& dims, st
 
     bool EclipseGrid::cellActive( size_t globalIndex ) const {
         assertGlobalIndex( globalIndex );
-
-        return m_actnum[globalIndex]>0;
+        if (m_actnum.empty()) {
+            return true;
+        } else {
+            return m_actnum[globalIndex]>0;
+        }
     }
 
     bool EclipseGrid::cellActive( size_t i , size_t j , size_t k ) const {
