@@ -15,6 +15,9 @@
   You should have received a copy of the GNU General Public License along with
   OPM.  If not, see <http://www.gnu.org/licenses/>.
 */
+
+#include <opm/input/eclipse/EclipseState/Grid/FieldProps.hpp>
+
 #include <functional>
 #include <algorithm>
 #include <unordered_map>
@@ -23,6 +26,7 @@
 #include <set>
 #include <unordered_set>
 
+#include <fmt/format.h>
 #include <opm/common/utility/OpmInputError.hpp>
 
 #include <opm/input/eclipse/Parser/ParserKeywords/A.hpp>
@@ -42,14 +46,29 @@
 #include <opm/input/eclipse/EclipseState/Grid/FieldPropsManager.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/SatfuncPropertyInitializers.hpp>
 #include <opm/input/eclipse/EclipseState/Runspec.hpp>
-#include <opm/common/utility/Serializer.hpp>
 #include <opm/input/eclipse/EclipseState/Aquifer/NumericalAquifer/NumericalAquifers.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/TableManager.hpp>
 
-#include "FieldProps.hpp"
 #include "Operate.hpp"
 
 
+namespace {
+    Opm::Box makeGlobalGridBox(const Opm::EclipseGrid* gridPtr)
+    {
+        return Opm::Box {
+            *gridPtr,
+            [gridPtr](const std::size_t global_index)
+            {
+                return gridPtr->cellActive(global_index);
+            },
+            [gridPtr](const std::size_t global_index)
+            {
+                return gridPtr->activeIndex(global_index);
+            }
+        };
+    }
+}
+
 namespace Opm {
 
 namespace Fieldprops
@@ -308,7 +327,7 @@ Fieldprops::ScalarOperation fromString(const std::string& keyword) {
     if (keyword == ParserKeywords::MAXVALUE::keywordName)
         return Fieldprops::ScalarOperation::MAX;
 
-    throw std::invalid_argument("Keyword operation not recognized");
+    throw std::invalid_argument(fmt::format("Keyword operation ({}) not recognized", keyword));
 }
 
 
@@ -490,6 +509,32 @@ FieldProps::FieldProps(const Deck& deck, const Phases& phases, const EclipseGrid
 }
 
 
+// Special constructor ONLY used to get the correct ACTNUM.
+// The grid argument should have all active cells.
+FieldProps::FieldProps(const Deck& deck, const EclipseGrid& grid) :
+    active_size(grid.getNumActive()),
+    global_size(grid.getCartesianSize()),
+    unit_system(deck.getActiveUnitSystem()),
+    nx(grid.getNX()),
+    ny(grid.getNY()),
+    nz(grid.getNZ()),
+    m_phases(),
+    m_satfuncctrl(deck),
+    m_actnum(global_size, 1),  // NB! activates all at start!
+    cell_volume(),             // NB! empty for this purpose.
+    cell_depth(),              // NB! empty for this purpose.
+    m_default_region(default_region_keyword(deck)),
+    grid_ptr(&grid),
+    tables()                   // NB! empty for this purpose.
+{
+    if (this->active_size != this->global_size) {
+        throw std::logic_error("Programmer error: FieldProps special case processing for ACTNUM called with grid object that already had deactivated cells.");
+    }
+    if (DeckSection::hasGRID(deck))
+        this->scanGRIDSectionOnlyACTNUM(GRIDSection(deck));
+}
+
+
 
 void FieldProps::reset_actnum(const std::vector<int>& new_actnum) {
     if (this->global_size != new_actnum.size())
@@ -905,7 +950,14 @@ void FieldProps::handle_region_operation(const DeckKeyword& keyword) {
                   with global storage.
                 */
                 if (field_data.global_data)
-                    throw std::logic_error("Region operations on 3D fields with global storage is not implemented");
+                {
+                    const auto& location = keyword.location();
+                    using namespace std::string_literals;
+                    throw OpmInputError(fmt::format("region operation on 3D field {} with "s +
+                                                    "global storage is not implemented!"s,
+                                                    target_kw),
+                                        location);
+                }
 
                 FieldProps::apply(fromString(keyword.name()), field_data.data, field_data.value_status, scalar_value, index_list);
             }
@@ -1148,8 +1200,13 @@ std::vector<int> FieldProps::actnum() {
 }
 
 
+const std::vector<int>& FieldProps::actnumRaw() const {
+    return m_actnum;
+}
+
+
 void FieldProps::scanGRIDSection(const GRIDSection& grid_section) {
-    Box box(*this->grid_ptr);
+    auto box = makeGlobalGridBox(this->grid_ptr);
 
     for (const auto& keyword : grid_section) {
         const std::string& name = keyword.name();
@@ -1168,8 +1225,27 @@ void FieldProps::scanGRIDSection(const GRIDSection& grid_section) {
     }
 }
 
+void FieldProps::scanGRIDSectionOnlyACTNUM(const GRIDSection& grid_section) {
+    Box box(*this->grid_ptr, [](const std::size_t) { return true; }, [](const std::size_t i) { return i; });
+
+    for (const auto& keyword : grid_section) {
+        const std::string& name = keyword.name();
+        if (name == "ACTNUM") {
+            this->handle_int_keyword(Fieldprops::keywords::GRID::int_keywords.at(name), keyword, box);
+        } else if (name == "EQUALS" || (Fieldprops::keywords::box_keywords.count(name) == 1)) {
+            this->handle_keyword(keyword, box);
+        }
+    }
+    const auto iter = this->int_data.find("ACTNUM");
+    if (iter == this->int_data.end()) {
+        m_actnum.assign(this->grid_ptr->getCartesianSize(), 1);
+    } else {
+        m_actnum = iter->second.data;
+    }
+}
+
 void FieldProps::scanEDITSection(const EDITSection& edit_section) {
-    Box box(*this->grid_ptr);
+    auto box = makeGlobalGridBox(this->grid_ptr);
     for (const auto& keyword : edit_section) {
         const std::string& name = keyword.name();
 
@@ -1212,7 +1288,7 @@ void FieldProps::init_satfunc(const std::string& keyword, Fieldprops::FieldData<
 
 
 void FieldProps::scanPROPSSection(const PROPSSection& props_section) {
-    Box box(*this->grid_ptr);
+    auto box = makeGlobalGridBox(this->grid_ptr);
 
     for (const auto& keyword : props_section) {
         const std::string& name = keyword.name();
@@ -1238,7 +1314,7 @@ void FieldProps::scanPROPSSection(const PROPSSection& props_section) {
 
 
 void FieldProps::scanREGIONSSection(const REGIONSSection& regions_section) {
-    Box box(*this->grid_ptr);
+    auto box = makeGlobalGridBox(this->grid_ptr);
 
     for (const auto& keyword : regions_section) {
         const std::string& name = keyword.name();
@@ -1260,7 +1336,7 @@ void FieldProps::scanREGIONSSection(const REGIONSSection& regions_section) {
 
 
 void FieldProps::scanSOLUTIONSection(const SOLUTIONSection& solution_section) {
-    Box box(*this->grid_ptr);
+    auto box = makeGlobalGridBox(this->grid_ptr);
     for (const auto& keyword : solution_section) {
         const std::string& name = keyword.name();
         if (Fieldprops::keywords::SOLUTION::double_keywords.count(name) == 1) {
@@ -1273,7 +1349,7 @@ void FieldProps::scanSOLUTIONSection(const SOLUTIONSection& solution_section) {
 }
 
 void FieldProps::handle_schedule_keywords(const std::vector<DeckKeyword>& keywords) {
-    Box box(*this->grid_ptr);
+    auto box = makeGlobalGridBox(this->grid_ptr);
 
     // When called in the SCHEDULE section the context is that the scaling factors
     // have already been applied.
@@ -1308,26 +1384,6 @@ void FieldProps::apply_tran(const std::string& keyword, std::vector<double>& dat
     Opm::apply_tran(this->tran, this->double_data, this->active_size, keyword, data);
 }
 
-
-std::vector<char> FieldProps::serialize_tran() const {
-    Serializer ser;
-    ser.put(this->tran.size());
-    for (const auto& tran_pair : this->tran) {
-        const auto& calc = tran_pair.second;
-        ser.put(calc.name());
-        ser.put(calc.size());
-        for (const auto& action : calc) {
-            ser.put(static_cast<int>(action.op));
-            ser.put(action.field);
-        }
-    }
-    return std::move(ser.buffer);
-}
-
-void FieldProps::deserialize_tran(const std::vector<char>& buffer) {
-    Opm::deserialize_tran(this->tran, buffer);
-}
-
 bool FieldProps::tran_active(const std::string& keyword) const {
     auto calculator = this->tran.find(keyword);
     return calculator != this->tran.end() && calculator->second.size() > 0;
