@@ -19,8 +19,6 @@
 
 #include <opm/input/eclipse/EclipseState/Grid/SatfuncPropertyInitializers.hpp>
 
-#include <opm/input/eclipse/EclipseState/EclipseState.hpp>
-#include <opm/input/eclipse/EclipseState/Grid/EclipseGrid.hpp>
 #include <opm/input/eclipse/EclipseState/Runspec.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/SgfnTable.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/SgofTable.hpp>
@@ -93,8 +91,8 @@ namespace {
         }
 
         const auto family1 =       // SGOF/SLGOF and/or SWOF
-            (gas && (tm.hasTables("SGOF") || tm.hasTables("SLGOF"))) ||
-            (wat && tm.hasTables("SWOF"));
+            (gas && ((tm.hasTables("SGOF") || !tm.getSgofletTable().empty()) || tm.hasTables("SLGOF"))) ||
+            (wat && (tm.hasTables("SWOF") || !tm.getSwofletTable().empty()));
         // note: we allow for SOF2 to be part of family1 for threeP + solvent simulations.
 
         const auto family2 =      // SGFN, SOF{2,3}, SWFN
@@ -135,18 +133,30 @@ namespace {
             return std::vector<double>(num_tables, 0.0);
 
         const auto& swofTables = tm.getSwofTables();
+        const auto& swofLetTables  = tm.getSwofletTable();
         const auto& swfnTables = tm.getSwfnTables();
 
         const auto famI = [&swofTables]( int i ) {
             return swofTables.getTable<Opm::SwofTable>( i ).getSwColumn().front();
         };
 
+        const auto famI_let = [&swofLetTables]( int i ) {
+            return swofLetTables[i].s1_residual;
+        };
+
         const auto famII = [&swfnTables]( int i ) {
             return swfnTables.getTable<Opm::SwfnTable>( i ).getSwColumn().front();
         };
 
         switch( getSaturationFunctionFamily( tm, ph ) ) {
-            case SatfuncFamily::I: return map( famI, Opm::fun::iota( num_tables ) );
+            case SatfuncFamily::I:
+                if( !swofTables.empty() )
+                    return Opm::fun::map( famI, Opm::fun::iota( num_tables ) );
+                else if ( !swofLetTables.empty() )
+                    return Opm::fun::map( famI_let, Opm::fun::iota( num_tables ) );
+                else
+                    throw std::domain_error("Either SWOF or SWOFLET tables must be provided");
+
             case SatfuncFamily::II: return map( famII, Opm::fun::iota( num_tables ) );
             default:
                 throw std::domain_error("No valid saturation keyword family specified");
@@ -163,18 +173,30 @@ namespace {
             return std::vector<double>(num_tables, 0.0);
 
         const auto& swofTables = tm.getSwofTables();
+        const auto& swofLetTables  = tm.getSwofletTable();
         const auto& swfnTables = tm.getSwfnTables();
 
         const auto famI = [&swofTables]( int i ) {
             return swofTables.getTable<Opm::SwofTable>( i ).getSwColumn().back();
         };
 
+        const auto famI_let = [&swofLetTables]( int i ) {
+            return 1.0 - swofLetTables[i].s2_residual;
+        };
+
         const auto famII = [&swfnTables]( int i ) {
             return swfnTables.getTable<Opm::SwfnTable>( i ).getSwColumn().back();
         };
 
         switch( getSaturationFunctionFamily( tm, ph ) ) {
-            case SatfuncFamily::I: return map( famI, Opm::fun::iota( num_tables ) );
+            case SatfuncFamily::I:
+                if( !swofTables.empty() )
+                    return map( famI, Opm::fun::iota( num_tables ) );
+                else if ( !swofLetTables.empty() )
+                    return Opm::fun::map( famI_let, Opm::fun::iota( num_tables ) );
+                else
+                    throw std::domain_error("Either SWOF or SWOFLET tables must be provided");
+
             case SatfuncFamily::II: return map( famII, Opm::fun::iota( num_tables ) );
             default:
                 throw std::domain_error("No valid saturation keyword family specified");
@@ -191,6 +213,7 @@ namespace {
             return std::vector<double>(num_tables, 0.0);
 
         const auto& sgofTables  = tm.getSgofTables();
+        const auto& sgofLetTables  = tm.getSgofletTable();
         const auto& slgofTables = tm.getSlgofTables();
         const auto& sgfnTables = tm.getSgfnTables();
 
@@ -198,6 +221,10 @@ namespace {
             return sgofTables.getTable<Opm::SgofTable>( i ).getSgColumn().front();
         };
 
+        const auto famI_sgof_let = [&sgofLetTables]( int i ) {
+            return sgofLetTables[i].s1_residual;
+        };
+
         const auto famI_slgof = [&slgofTables]( int i ) {
             return 1.0 - slgofTables.getTable<Opm::SlgofTable>( i ).getSlColumn().back();
         };
@@ -208,11 +235,13 @@ namespace {
 
         switch( getSaturationFunctionFamily( tm, ph ) ) {
             case SatfuncFamily::I:
-                if( sgofTables.empty() && slgofTables.empty() )
+                if( sgofTables.empty() && sgofLetTables.empty() && slgofTables.empty() )
                     throw std::runtime_error( "Saturation keyword family I requires either sgof or slgof non-empty" );
 
                 if( !sgofTables.empty() )
                     return Opm::fun::map( famI_sgof, Opm::fun::iota( num_tables ) );
+                else if( !sgofLetTables.empty() )
+                    return Opm::fun::map( famI_sgof_let, Opm::fun::iota( num_tables ) );
                 else
                     return Opm::fun::map( famI_slgof, Opm::fun::iota( num_tables ) );
 
@@ -234,6 +263,8 @@ namespace {
             return std::vector<double>(num_tables, 0.0);
 
         const auto& sgofTables  = tm.getSgofTables();
+        const auto& sgofLetTables  = tm.getSgofletTable();
+        const auto& swofLetTables  = tm.getSwofletTable();
         const auto& slgofTables = tm.getSlgofTables();
         const auto& sgfnTables = tm.getSgfnTables();
 
@@ -241,6 +272,11 @@ namespace {
             return sgofTables.getTable<Opm::SgofTable>( i ).getSgColumn().back();
         };
 
+        const auto famI_sgof_let = [&swofLetTables]( int i ) {
+            // Assume this to be 1-swco
+            return 1.0 - swofLetTables[i].s1_residual;
+        };
+
         const auto famI_slgof = [&slgofTables]( int i ) {
             return 1.0 - slgofTables.getTable<Opm::SlgofTable>( i ).getSlColumn().front();
         };
@@ -251,11 +287,13 @@ namespace {
 
         switch( getSaturationFunctionFamily( tm, ph ) ) {
             case SatfuncFamily::I:
-                if( sgofTables.empty() && slgofTables.empty() )
+                if( sgofTables.empty() && sgofLetTables.empty() && slgofTables.empty() )
                     throw std::runtime_error( "Saturation keyword family I requires either sgof or slgof non-empty" );
 
                 if( !sgofTables.empty() )
                     return Opm::fun::map( famI_sgof, Opm::fun::iota( num_tables ) );
+                else if( !sgofLetTables.empty() )
+                    return Opm::fun::map( famI_sgof_let, Opm::fun::iota( num_tables ) );
                 else
                     return Opm::fun::map( famI_slgof, Opm::fun::iota( num_tables ) );
 
@@ -335,6 +373,7 @@ namespace {
             return std::vector<double>(num_tables, 0.0);
 
         const auto& swofTables = tm.getSwofTables();
+        const auto& swofLetTables  = tm.getSwofletTable();
         const auto& swfnTables = tm.getSwfnTables();
 
         const auto famI = [&swofTables, tolcrit](const int i) -> double
@@ -342,13 +381,24 @@ namespace {
             return critical_water(swofTables.getTable<Opm::SwofTable>(i), tolcrit);
         };
 
+        const auto famI_let = [&swofLetTables]( int i ) {
+            return swofLetTables[i].s1_critical;
+        };
+
         const auto famII = [&swfnTables, tolcrit](const int i) -> double
         {
             return critical_water(swfnTables.getTable<Opm::SwfnTable>(i), tolcrit);
         };
 
         switch( getSaturationFunctionFamily( tm, ph ) ) {
-            case SatfuncFamily::I: return Opm::fun::map( famI, Opm::fun::iota( num_tables ) );
+            case SatfuncFamily::I:
+                if( !swofTables.empty() )
+                    return Opm::fun::map( famI, Opm::fun::iota( num_tables ) );
+                else if ( !swofLetTables.empty() )
+                    return Opm::fun::map( famI_let, Opm::fun::iota( num_tables ) );
+                else
+                    throw std::domain_error("Either SWOF or SWOFLET tables must be provided");
+
             case SatfuncFamily::II: return Opm::fun::map( famII, Opm::fun::iota( num_tables ) );
             default: throw std::domain_error("No valid saturation keyword family specified");
         }
@@ -396,6 +446,7 @@ namespace {
 
         const auto& sgfnTables = tm.getSgfnTables();
         const auto& sgofTables = tm.getSgofTables();
+        const auto& sgofLetTables  = tm.getSgofletTable();
         const auto& slgofTables = tm.getSlgofTables();
 
         const auto famI_sgof = [&sgofTables, tolcrit](const int i) -> double
@@ -403,6 +454,10 @@ namespace {
             return critical_gas(sgofTables.getTable<Opm::SgofTable>(i), tolcrit);
         };
 
+        const auto famI_sgof_let = [&sgofLetTables]( int i ) {
+            return sgofLetTables[i].s1_critical;
+        };
+
         const auto famI_slgof = [&slgofTables, tolcrit](const int i) -> double
         {
             return critical_gas(slgofTables.getTable<Opm::SlgofTable>(i), tolcrit);
@@ -415,11 +470,13 @@ namespace {
 
         switch( getSaturationFunctionFamily( tm, ph ) ) {
             case SatfuncFamily::I:
-                if( sgofTables.empty() && slgofTables.empty() )
+                if( sgofTables.empty() && sgofLetTables.empty() && slgofTables.empty() )
                     throw std::runtime_error( "Saturation keyword family I requires either sgof or slgof non-empty" );
 
                 if( !sgofTables.empty() )
                     return Opm::fun::map( famI_sgof, Opm::fun::iota( num_tables ) );
+                else if( !sgofLetTables.empty() )
+                    return Opm::fun::map( famI_sgof_let, Opm::fun::iota( num_tables ) );
                 else
                     return Opm::fun::map( famI_slgof, Opm::fun::iota( num_tables ) );
 
@@ -486,6 +543,7 @@ namespace {
             return std::vector<double>(num_tables, 0.0);
 
         const auto& swofTables = tm.getSwofTables();
+        const auto& swofLetTables  = tm.getSwofletTable();
         const auto& sof2Tables = tm.getSof2Tables();
         const auto& sof3Tables = tm.getSof3Tables();
 
@@ -494,6 +552,10 @@ namespace {
             return critical_oil_water(swofTables.getTable<Opm::SwofTable>(i), tolcrit);
         };
 
+        const auto famI_let = [&swofLetTables]( int i ) {
+            return swofLetTables[i].s2_critical;
+        };
+
         const auto famII_2p = [&sof2Tables, tolcrit](const int i) -> double
         {
             return critical_oil(sof2Tables.getTable<Opm::Sof2Table>(i), tolcrit);
@@ -506,7 +568,14 @@ namespace {
         };
 
         switch( getSaturationFunctionFamily( tm, ph ) ) {
-            case SatfuncFamily::I: return Opm::fun::map( famI, Opm::fun::iota( num_tables ) );
+            case SatfuncFamily::I:
+                if( !swofTables.empty() )
+                    return Opm::fun::map( famI, Opm::fun::iota( num_tables ) );
+                else if ( !swofLetTables.empty() )
+                    return Opm::fun::map( famI_let, Opm::fun::iota( num_tables ) );
+                else
+                    throw std::domain_error("Either SWOF or SWOFLET tables must be provided");
+
             case SatfuncFamily::II:
                 return ph.active(::Opm::Phase::GAS)
                     ? Opm::fun::map( famII_3p, Opm::fun::iota( num_tables ) )
@@ -559,6 +628,7 @@ namespace {
             return std::vector<double>(num_tables, 0.0);
 
         const auto& sgofTables = tm.getSgofTables();
+        const auto& sgofLetTables  = tm.getSgofletTable();
         const auto& slgofTables = tm.getSlgofTables();
         const auto& sof2Tables = tm.getSof2Tables();
         const auto& sof3Tables = tm.getSof3Tables();
@@ -568,6 +638,10 @@ namespace {
             return critical_oil_gas(sgofTables.getTable<Opm::SgofTable>(i), tolcrit) - swco[i];
         };
 
+        const auto famI_sgof_let = [&sgofLetTables]( int i ) {
+            return sgofLetTables[i].s2_critical;
+        };
+
         const auto famI_slgof = [&slgofTables, &swco, tolcrit](const int i) -> double
         {
             return critical_oil_gas(slgofTables.getTable<Opm::SlgofTable>(i), tolcrit) - swco[i];
@@ -586,11 +660,13 @@ namespace {
 
         switch( getSaturationFunctionFamily( tm, ph ) ) {
             case SatfuncFamily::I:
-                if( sgofTables.empty() && slgofTables.empty() )
+                if( sgofTables.empty() && sgofLetTables.empty() && slgofTables.empty() )
                     throw std::runtime_error( "Saturation keyword family I requires either sgof or slgof non-empty" );
 
                 if( !sgofTables.empty() )
                     return Opm::fun::map( famI_sgof, Opm::fun::iota( num_tables ) );
+                else if( !sgofLetTables.empty() )
+                    return Opm::fun::map( famI_sgof_let, Opm::fun::iota( num_tables ) );
                 else
                     return Opm::fun::map( famI_slgof, Opm::fun::iota( num_tables ) );
 
@@ -614,6 +690,7 @@ namespace {
             return std::vector<double>(num_tables, 0.0);
 
         const auto& sgofTables = tm.getSgofTables();
+        const auto& sgofLetTables  = tm.getSgofletTable();
         const auto& slgofTables = tm.getSlgofTables();
         const auto& sgfnTables = tm.getSgfnTables();
 
@@ -621,6 +698,10 @@ namespace {
             return sgofTables.getTable<Opm::SgofTable>( i ).getKrgColumn().back();
         };
 
+        const auto& famI_sgof_let = [&sgofLetTables]( int i ) {
+            return sgofLetTables[i].krt1_relperm;
+        };
+
         const auto& famI_slgof = [&slgofTables]( int i ) {
             return slgofTables.getTable<Opm::SlgofTable>( i ).getKrgColumn().front();
         };
@@ -631,10 +712,12 @@ namespace {
 
         switch( getSaturationFunctionFamily( tm, ph ) ) {
             case SatfuncFamily::I:
-                if( sgofTables.empty() && slgofTables.empty() )
+                if( sgofTables.empty() && sgofLetTables.empty() && slgofTables.empty() )
                     throw std::runtime_error( "Saturation keyword family I requires either sgof or slgof non-empty" );
                 if( !sgofTables.empty() )
                     return Opm::fun::map( famI_sgof, Opm::fun::iota( num_tables ) );
+                else if( !sgofLetTables.empty() )
+                    return Opm::fun::map( famI_sgof_let, Opm::fun::iota( num_tables ) );
                 else
                     return Opm::fun::map( famI_slgof, Opm::fun::iota( num_tables ) );
             case SatfuncFamily::II:
@@ -655,6 +738,7 @@ namespace {
             return std::vector<double>(num_tables, 0.0);
 
         const auto& sgofTables = tm.getSgofTables();
+        const auto& sgofLetTables  = tm.getSgofletTable();
         const auto& slgofTables = tm.getSlgofTables();
         const auto& sgfnTables = tm.getSgfnTables();
 
@@ -681,6 +765,11 @@ namespace {
             return sgof.getKrgColumn().eval(ix);
         };
 
+        const auto famI_sgof_let = [&sgofLetTables](const int i) -> double
+        {
+            return sgofLetTables[i].krt1_relperm;
+        };
+
         const auto famI_slgof = [&slgofTables, &sr](const int i) -> double
         {
             const auto& slgof = slgofTables.getTable<Opm::SlgofTable>(i);
@@ -699,10 +788,12 @@ namespace {
 
         switch( getSaturationFunctionFamily( tm, ph ) ) {
             case SatfuncFamily::I:
-                if( sgofTables.empty() && slgofTables.empty() )
+                if( sgofTables.empty() && sgofLetTables.empty() && slgofTables.empty() )
                     throw std::runtime_error( "Saturation keyword family I requires either sgof or slgof non-empty" );
                 if( !sgofTables.empty() )
                     return Opm::fun::map( famI_sgof, Opm::fun::iota( num_tables ) );
+                else if( !sgofLetTables.empty() )
+                    return Opm::fun::map( famI_sgof_let, Opm::fun::iota( num_tables ) );
                 else
                     return Opm::fun::map( famI_slgof, Opm::fun::iota( num_tables ) );
             case SatfuncFamily::II:
@@ -723,6 +814,7 @@ namespace {
             return std::vector<double>(num_tables, 0.0);
 
         const auto& swofTables = tm.getSwofTables();
+        const auto& swofLetTables  = tm.getSwofletTable();
         const auto& swfnTables = tm.getSwfnTables();
 
         auto sr = std::vector<double>(num_tables, 0.0);
@@ -748,6 +840,11 @@ namespace {
             return swof.getKrwColumn().eval(ix);
         };
 
+        const auto& famI_let = [&swofLetTables](const int i) -> double
+        {
+            return swofLetTables[i].krt1_relperm;
+        };
+
         const auto& famII = [&swfnTables, &sr](const int i) -> double
         {
             const auto& swfn = swfnTables.getTable<Opm::SwfnTable>(i);
@@ -758,7 +855,13 @@ namespace {
 
         switch( getSaturationFunctionFamily( tm, ph ) ) {
             case SatfuncFamily::I:
-                return Opm::fun::map( famI, Opm::fun::iota( num_tables ) );
+                if( !swofTables.empty() )
+                    return Opm::fun::map( famI, Opm::fun::iota( num_tables ) );
+                else if( !swofLetTables.empty() )
+                    return Opm::fun::map( famI_let, Opm::fun::iota( num_tables ) );
+                else
+                    throw std::domain_error("Either SWOF or SWOFLET tables must be provided");
+
             case SatfuncFamily::II:
                 return Opm::fun::map( famII, Opm::fun::iota( num_tables ) );
             default:
@@ -778,6 +881,7 @@ namespace {
             return std::vector<double>(num_tables, 0.0);
 
         const auto& swofTables = tm.getSwofTables();
+        const auto& swofLetTables  = tm.getSwofletTable();
         const auto& sof2Tables = tm.getSof2Tables();
         const auto& sof3Tables = tm.getSof3Tables();
 
@@ -790,6 +894,11 @@ namespace {
             return swof.getKrowColumn().eval(ix);
         };
 
+        const auto famI_let = [&swofLetTables](const int i) -> double
+        {
+            return swofLetTables[i].krt2_relperm;
+        };
+
         const auto famII_3p = [&sof3Tables, &ep](const int i) -> double
         {
             const auto& sof3 = sof3Tables.getTable<Opm::Sof3Table>(i);
@@ -810,7 +919,13 @@ namespace {
 
         switch( getSaturationFunctionFamily( tm, ph ) ) {
             case SatfuncFamily::I:
-                return Opm::fun::map( famI, Opm::fun::iota( num_tables ) );
+                if ( !swofTables.empty() )
+                    return Opm::fun::map( famI, Opm::fun::iota( num_tables ) );
+                else if ( !swofLetTables.empty() )
+                    return Opm::fun::map( famI_let, Opm::fun::iota( num_tables ) );
+                else
+                    throw std::domain_error("Either SWOF or SWOFLET tables must be provided");
+
             case SatfuncFamily::II:
                 return ph.active(::Opm::Phase::GAS)
                     ? Opm::fun::map( famII_3p, Opm::fun::iota( num_tables ) )
@@ -832,6 +947,7 @@ namespace {
             return std::vector<double>(num_tables, 0.0);
 
         const auto& sgofTables = tm.getSgofTables();
+        const auto& sgofLetTables = tm.getSgofletTable();
         const auto& slgofTables = tm.getSlgofTables();
         const auto& sof2Tables = tm.getSof2Tables();
         const auto& sof3Tables = tm.getSof3Tables();
@@ -845,6 +961,11 @@ namespace {
             return sgof.getKrogColumn().eval(ix);
         };
 
+        const auto famI_sgof_let = [&sgofLetTables](const int i) -> double
+        {
+            return sgofLetTables[i].krt2_relperm;
+        };
+
         const auto famI_slgof = [&slgofTables, &ep](const int i) -> double
         {
             const auto& slgof = slgofTables.getTable<Opm::SlgofTable>(i);
@@ -873,10 +994,12 @@ namespace {
 
         switch( getSaturationFunctionFamily( tm, ph ) ) {
             case SatfuncFamily::I:
-                if( sgofTables.empty() && slgofTables.empty() )
+                if( sgofTables.empty() && sgofLetTables.empty() && slgofTables.empty() )
                     throw std::runtime_error( "Saturation keyword family I requires either sgof or slgof non-empty" );
                 if( !sgofTables.empty() )
                     return Opm::fun::map( famI_sgof, Opm::fun::iota( num_tables ) );
+                else if( !sgofLetTables.empty() )
+                    return Opm::fun::map( famI_sgof_let, Opm::fun::iota( num_tables ) );
                 else
                     return Opm::fun::map( famI_slgof, Opm::fun::iota( num_tables ) );
             case SatfuncFamily::II:
@@ -908,6 +1031,7 @@ namespace {
             return std::vector<double>(num_tables, 0.0);
 
         const auto& sgofTables = tm.getSgofTables();
+        const auto& sgofLetTables = tm.getSgofletTable();
         const auto& slgofTables = tm.getSlgofTables();
         const auto& sgfnTables = tm.getSgfnTables();
 
@@ -915,6 +1039,10 @@ namespace {
             return sgofTables.getTable<Opm::SgofTable>( i ).getPcogColumn().back();
         };
 
+        const auto& famI_sgof_let = [&sgofLetTables]( int i ) {
+            return sgofLetTables[i].pct_pc;
+        };
+
         const auto& famI_slgof = [&slgofTables]( int i ) {
             return slgofTables.getTable<Opm::SlgofTable>( i ).getPcogColumn().front();
         };
@@ -925,10 +1053,12 @@ namespace {
 
         switch( getSaturationFunctionFamily( tm, ph ) ) {
             case SatfuncFamily::I:
-                if( sgofTables.empty() && slgofTables.empty() )
+                if( sgofTables.empty() && sgofLetTables.empty() && slgofTables.empty() )
                     throw std::runtime_error( "Saturation keyword family I requires either sgof or slgof non-empty" );
                 if( !sgofTables.empty() )
                     return Opm::fun::map( famI_sgof, Opm::fun::iota( num_tables ) );
+                else if( !sgofLetTables.empty() )
+                    return Opm::fun::map( famI_sgof_let, Opm::fun::iota( num_tables ) );
                 else
                     return Opm::fun::map( famI_slgof, Opm::fun::iota( num_tables ) );
             case SatfuncFamily::II:
@@ -949,19 +1079,30 @@ namespace {
             return std::vector<double>(num_tables, 0.0);
 
         const auto& swofTables = tm.getSwofTables();
+        const auto& swofLetTables = tm.getSwofletTable();
         const auto& swfnTables = tm.getSwfnTables();
 
         const auto& famI = [&swofTables]( int i ) {
             return swofTables.getTable<Opm::SwofTable>( i ).getPcowColumn().front();
         };
 
+        const auto& famI_let = [&swofLetTables]( int i ) {
+            return swofLetTables[i].pct_pc;
+        };
+
         const auto& famII = [&swfnTables]( int i ) {
             return swfnTables.getTable<Opm::SwfnTable>( i ).getPcowColumn().front();
         };
 
         switch( getSaturationFunctionFamily( tm, ph ) ) {
             case SatfuncFamily::I:
-                return Opm::fun::map( famI, Opm::fun::iota( num_tables ) );
+                if (!swofTables.empty())
+                    return Opm::fun::map( famI, Opm::fun::iota( num_tables ) );
+                else if (!swofLetTables.empty())
+                    return Opm::fun::map( famI_let, Opm::fun::iota( num_tables ) );
+                else
+                    throw std::domain_error("Either SWOF or SWOFLET tables must be provided");
+
             case SatfuncFamily::II:
                 return Opm::fun::map( famII, Opm::fun::iota( num_tables ) );
             default:
@@ -981,6 +1122,8 @@ namespace {
         const auto wat = ph.active(::Opm::Phase::WATER);
 
         const auto& other_f1   = wat ? tm.getSwofTables() : tm.getSgofTables();
+        const auto& wat_f1_let   = tm.getSwofletTable();
+        const auto& gas_f1_let   = tm.getSgofletTable();
         const auto& sof2Tables = tm.getSof2Tables();
         const auto& sof3Tables = tm.getSof3Tables();
 
@@ -993,6 +1136,18 @@ namespace {
                 : other_f1.getTable<Opm::SgofTable>( i ).getKrogColumn().front();
         };
 
+        const auto& famI_let_wat = [&wat_f1_let]( int i ) {
+            // In O/W/G runs this relies on Krog(Sg=0) == Krow(Sw=Swco),
+            // meaning that in each region krt2_relperm is equal for SGOFLET and SWOFLET.
+            return wat_f1_let[i].krt2_relperm;
+        };
+
+        const auto& famI_let_gas = [&gas_f1_let]( int i ) {
+            // In O/W/G runs this relies on Krog(Sg=0) == Krow(Sw=Swco),
+            // meaning that in each region krt2_relperm is equal for SGOFLET and SWOFLET.
+            return gas_f1_let[i].krt2_relperm;
+        };
+
         const auto& famII_2p = [&sof2Tables]( int i ) {
             return sof2Tables.getTable<Opm::Sof2Table>( i ).getKroColumn().back();
         };
@@ -1003,7 +1158,15 @@ namespace {
 
         switch( getSaturationFunctionFamily( tm, ph ) ) {
             case SatfuncFamily::I:
-                return Opm::fun::map( famI, Opm::fun::iota( num_tables ) );
+                if ( !other_f1.empty() )
+                    return Opm::fun::map( famI, Opm::fun::iota( num_tables ) );
+                else if ( !wat_f1_let.empty() )
+                    return Opm::fun::map( famI_let_wat, Opm::fun::iota( num_tables ) );
+                else if ( !gas_f1_let.empty() )
+                    return Opm::fun::map( famI_let_gas, Opm::fun::iota( num_tables ) );
+                else
+                    throw std::domain_error("Valid family I tables must be provided");
+
             case SatfuncFamily::II:
                 return ph.active(::Opm::Phase::GAS) && ph.active(::Opm::Phase::WATER)
                     ? Opm::fun::map( famII_3p, Opm::fun::iota( num_tables ) )
@@ -1023,19 +1186,31 @@ namespace {
             return std::vector<double>(num_tables, 0.0);
 
         const auto& swofTables = tm.getSwofTables();
+        const auto& swofLetTables  = tm.getSwofletTable();
         const auto& swfnTables = tm.getSwfnTables();
 
         const auto& famI = [&swofTables]( int i ) {
             return swofTables.getTable<Opm::SwofTable>( i ).getKrwColumn().back();
         };
 
+        const auto& famI_let = [&swofLetTables]( int i ) {
+            return swofLetTables[i].krt1_relperm;
+        };
+
         const auto& famII = [&swfnTables]( int i ) {
             return swfnTables.getTable<Opm::SwfnTable>( i ).getKrwColumn().back();
         };
 
         switch( getSaturationFunctionFamily( tm, ph ) ) {
             case SatfuncFamily::I:
-                return Opm::fun::map( famI, Opm::fun::iota( num_tables ) );
+                if( !swofTables.empty() )
+                    return Opm::fun::map( famI, Opm::fun::iota( num_tables ) );
+                else if( !swofLetTables.empty() )
+                    return Opm::fun::map( famI_let, Opm::fun::iota( num_tables ) );
+                else
+                    throw std::domain_error("Either SWOF or SWOFLET tables must be provided");
+
+
             case SatfuncFamily::II:
                 return Opm::fun::map( famII, Opm::fun::iota( num_tables ) );
             default:
