@@ -37,9 +37,11 @@
 
 #include <opm/common/OpmLog/OpmLog.hpp>
 
+#include <algorithm>
 #include <ostream>
 #include <stdexcept>
 #include <type_traits>
+
 #include <fmt/format.h>
 
 namespace {
@@ -51,7 +53,8 @@ namespace {
             deck.hasKeyword<Opm::ParserKeywords::WATER>(),
             deck.hasKeyword<Opm::ParserKeywords::SOLVENT>(),
             deck.hasKeyword<Opm::ParserKeywords::POLYMER>(),
-            deck.hasKeyword<Opm::ParserKeywords::THERMAL>(),
+            deck.hasKeyword<Opm::ParserKeywords::THERMAL>() ||
+            deck.hasKeyword<Opm::ParserKeywords::TEMP>(),
             deck.hasKeyword<Opm::ParserKeywords::POLYMW>(),
             deck.hasKeyword<Opm::ParserKeywords::FOAM>(),
             deck.hasKeyword<Opm::ParserKeywords::BRINE>(),
@@ -88,7 +91,8 @@ namespace {
         const auto family1 =       // SGOF/SLGOF and/or SWOF
             (gas && (deck.hasKeyword<Opm::ParserKeywords::SGOF>() ||
                      deck.hasKeyword<Opm::ParserKeywords::SLGOF>())) ||
-            (wat && deck.hasKeyword<Opm::ParserKeywords::SWOF>());
+            (wat && deck.hasKeyword<Opm::ParserKeywords::SWOF>()) ||
+            (wat && deck.hasKeyword<Opm::ParserKeywords::SWOFLET>());
 
         // note: we allow for SOF2 to be part of family1 for threeP +
         // solvent simulations.
@@ -175,7 +179,7 @@ Phases::Phases( bool oil, bool gas, bool wat, bool sol, bool pol, bool energy, b
 
 {}
 
-Phases Phases::serializeObject()
+Phases Phases::serializationTestObject()
 {
     return Phases(true, true, true, false, true, false, true, false);
 }
@@ -202,7 +206,7 @@ Welldims::Welldims(const Deck& deck)
         this->nCWMax = wd.getItem<WD::MAXCONN>().get<int>(0);
         this->nWGMax = wd.getItem<WD::MAX_GROUPSIZE>().get<int>(0);
 
-        // This is the E100 definition.  E300 instead uses
+        // Note: nGMax uses the E100 definition.  E300 instead uses
         //
         //   Max{ "MAXGROUPS", "MAXWELLS" }
         //
@@ -210,17 +214,19 @@ Welldims::Welldims(const Deck& deck)
         this->nGMax = wd.getItem<WD::MAXGROUPS>().get<int>(0);
         this->nWMax = wd.getItem<WD::MAXWELLS>().get<int>(0);
 
-        // maximum number of well lists pr well
-        this->nWlistPrWellMax = wd.getItem<WD::MAX_WELLIST_PR_WELL>().get<int>(0);
-        //maximum number of dynamic well lists
-        this->nDynWlistMax = wd.getItem<WD::MAX_DYNAMIC_WELLIST>().get<int>(0);
+        // Maximum number of well lists pr well.  Always at least 1.
+        this->nWlistPrWellMax =
+            std::max(WD::MAX_WELLIST_PR_WELL::defaultValue,
+                     wd.getItem<WD::MAX_WELLIST_PR_WELL>().get<int>(0));
 
+        // Maximum number of dynamic well lists
+        this->nDynWlistMax = wd.getItem<WD::MAX_DYNAMIC_WELLIST>().get<int>(0);
 
         this->m_location = keyword.location();
     }
 }
 
-Welldims Welldims::serializeObject()
+Welldims Welldims::serializationTestObject()
 {
     Welldims result;
     result.nWMax = 1;
@@ -229,7 +235,7 @@ Welldims Welldims::serializeObject()
     result.nGMax = 4;
     result.nWlistPrWellMax = 5;
     result.nDynWlistMax = 6;
-    result.m_location = KeywordLocation::serializeObject();
+    result.m_location = KeywordLocation::serializationTestObject();
     return result;
 }
 
@@ -250,7 +256,7 @@ WellSegmentDims::WellSegmentDims(const Deck& deck) : WellSegmentDims()
     }
 }
 
-WellSegmentDims WellSegmentDims::serializeObject()
+WellSegmentDims WellSegmentDims::serializationTestObject()
 {
     WellSegmentDims result;
     result.nSegWellMax = 1;
@@ -289,7 +295,7 @@ bool NetworkDims::active() const {
 }
 
 
-NetworkDims NetworkDims::serializeObject()
+NetworkDims NetworkDims::serializationTestObject()
 {
     NetworkDims result;
     result.nMaxNoNodes = 1;
@@ -325,7 +331,7 @@ AquiferDimensions::AquiferDimensions(const Deck& deck)
     }
 }
 
-AquiferDimensions AquiferDimensions::serializeObject()
+AquiferDimensions AquiferDimensions::serializationTestObject()
 {
     auto dim = AquiferDimensions{};
 
@@ -379,15 +385,17 @@ EclHysterConfig::EclHysterConfig(const Opm::Deck& deck)
 	*    phase and the imbibition curve for the relperm of the wetting phase
 	*/
         const auto& ehystrKeyword = deck["EHYSTR"].back();
-        if (deck.hasKeyword("NOHYKR"))
+        std::string whereFlag = ehystrKeyword.getRecord(0).getItem("limiting_hyst_flag").getTrimmedString(0);
+
+        if (deck.hasKeyword("NOHYKR") || whereFlag == "PC")
             krHystMod = -1;
         else {
             krHystMod = ehystrKeyword.getRecord(0).getItem("relative_perm_hyst").get<int>(0);
 
-            if (krHystMod != 0 && krHystMod != 1)
+            if (krHystMod != 0 && krHystMod != 1 && krHystMod != 2 && krHystMod != 3)
                 throw std::runtime_error(
-                    "Only the Carlson relative permeability hysteresis models (indicated by '0' or "
-                    "'1' for the second item of the 'EHYSTR' keyword) are supported");
+                    "Only the Carlson or Killough relative permeability hysteresis models (indicated by '0', "
+                    "'1', '2', or '3' for the second item of the 'EHYSTR' keyword) are supported");
         }
 
         // this is slightly screwed: it is possible to specify contradicting hysteresis
@@ -399,25 +407,32 @@ EclHysterConfig::EclHysterConfig(const Opm::Deck& deck)
 	* -1: capillary pressure hysteresis is disabled
 	* 0: use the Killough model for capillary pressure hysteresis
 	*/
-        std::string whereFlag =
-            ehystrKeyword.getRecord(0).getItem("limiting_hyst_flag").getTrimmedString(0);
         if (deck.hasKeyword("NOHYPC") || whereFlag == "KR")
             pcHystMod = -1;
         else {
             // if capillary pressure hysteresis is enabled, Eclipse always uses the
             // Killough model
             pcHystMod = 0;
-
-            throw std::runtime_error("Capillary pressure hysteresis is not supported yet");
+            curvatureCapPrsValue = ehystrKeyword.getRecord(0).getItem("curvature_capillary_pressure_hyst").get<double>(0);
+            if (curvatureCapPrsValue <= 0.0)
+                throw std::runtime_error(
+                    "Only positive values allowed for the 'capillary pressure curvature parameter' "
+                    "(the first item of the 'EHYSTR' keyword).");
         }
+
+        // Killough model: Regularisation for trapped critical saturation calculations
+        if (pcHystMod == 0 || krHystMod == 2 || krHystMod == 3)
+            modParamTrappedValue = ehystrKeyword.getRecord(0).getItem("mod_param_trapped").get<double>(0);
     }
 
-EclHysterConfig EclHysterConfig::serializeObject()
+EclHysterConfig EclHysterConfig::serializationTestObject()
 {
     EclHysterConfig result;
     result.activeHyst = true;
     result.pcHystMod = 1;
     result.krHystMod = 2;
+    result.modParamTrappedValue = 3;
+    result.curvatureCapPrsValue = 4;
 
     return result;
 }
@@ -431,10 +446,18 @@ int EclHysterConfig::pcHysteresisModel() const
 int EclHysterConfig::krHysteresisModel() const
     { return krHystMod; }
 
+double EclHysterConfig::modParamTrapped() const
+    { return modParamTrappedValue; }
+
+double EclHysterConfig::curvatureCapPrs() const
+    { return curvatureCapPrsValue; }
+
 bool EclHysterConfig::operator==(const EclHysterConfig& data) const {
     return this->active() == data.active() &&
            this->pcHysteresisModel() == data.pcHysteresisModel() &&
-           this->krHysteresisModel() == data.krHysteresisModel();
+           this->krHysteresisModel() == data.krHysteresisModel() &&
+           this->modParamTrapped() == data.modParamTrapped() &&
+           this->curvatureCapPrs() == data.curvatureCapPrs();
 }
 
 SatFuncControls::SatFuncControls()
@@ -465,7 +488,7 @@ SatFuncControls::SatFuncControls(const double tolcritArg,
     , satfunc_family(family)
 {}
 
-SatFuncControls SatFuncControls::serializeObject()
+SatFuncControls SatFuncControls::serializationTestObject()
 {
     return SatFuncControls {
         1.0, ThreePhaseOilKrModel::Stone2, KeywordFamily::Family_I
@@ -496,7 +519,7 @@ void Nupcol::update(int value) {
     this->nupcol_value = std::max(value, this->min_nupcol);
 }
 
-Nupcol Nupcol::serializeObject() {
+Nupcol Nupcol::serializationTestObject() {
     Nupcol nc;
     nc.update(123);
     return nc;
@@ -523,7 +546,7 @@ bool Tracers::operator==(const Tracers& other) const {
         this->min_iter == other.min_iter;
 }
 
-Tracers Tracers::serializeObject() {
+Tracers Tracers::serializationTestObject() {
     Tracers tracers;
     tracers.m_oil_tracers = 123;
     tracers.m_gas_tracers = 77;
@@ -622,21 +645,21 @@ Runspec::Runspec( const Deck& deck )
     }
 }
 
-Runspec Runspec::serializeObject()
+Runspec Runspec::serializationTestObject()
 {
     Runspec result;
-    result.active_phases = Phases::serializeObject();
-    result.m_tabdims = Tabdims::serializeObject();
-    result.m_regdims = Regdims::serializeObject();
-    result.endscale = EndpointScaling::serializeObject();
-    result.welldims = Welldims::serializeObject();
-    result.wsegdims = WellSegmentDims::serializeObject();
-    result.aquiferdims = AquiferDimensions::serializeObject();
-    result.udq_params = UDQParams::serializeObject();
-    result.hystpar = EclHysterConfig::serializeObject();
-    result.m_actdims = Actdims::serializeObject();
-    result.m_sfuncctrl = SatFuncControls::serializeObject();
-    result.m_nupcol = Nupcol::serializeObject();
+    result.active_phases = Phases::serializationTestObject();
+    result.m_tabdims = Tabdims::serializationTestObject();
+    result.m_regdims = Regdims::serializationTestObject();
+    result.endscale = EndpointScaling::serializationTestObject();
+    result.welldims = Welldims::serializationTestObject();
+    result.wsegdims = WellSegmentDims::serializationTestObject();
+    result.aquiferdims = AquiferDimensions::serializationTestObject();
+    result.udq_params = UDQParams::serializationTestObject();
+    result.hystpar = EclHysterConfig::serializationTestObject();
+    result.m_actdims = Actdims::serializationTestObject();
+    result.m_sfuncctrl = SatFuncControls::serializationTestObject();
+    result.m_nupcol = Nupcol::serializationTestObject();
     result.m_co2storage = true;
     result.m_micp = true;
 
