@@ -82,6 +82,7 @@
 #include <opm/input/eclipse/EclipseState/Tables/PdvdTable.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/RtempvdTable.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/RvvdTable.hpp>
+#include <opm/input/eclipse/EclipseState/Tables/RvwvdTable.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/PermfactTable.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/SaltvdTable.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/SaltpvdTable.hpp>
@@ -123,36 +124,6 @@ std::optional<JFunc> make_jfunc(const Deck& deck) {
     return JFunc(deck);
 }
 
-DensityTable make_density_table(const GravityTable& gravity) {
-    auto rho = DensityTable{};
-    rho.reserve(gravity.size());
-
-    constexpr auto default_air_density =
-        1.22 * unit::kilogram / unit::cubic(unit::meter);
-
-    constexpr auto default_water_density =
-        1000.0 * unit::kilogram / unit::cubic(unit::meter);
-
-    // Degrees API defined as
-    //
-    //   API = (141.5 / SG) - 131.5
-    //
-    // with SG being the specific gravity of oil relative to pure water.
-
-    std::transform(gravity.begin(), gravity.end(),
-                   std::back_inserter(rho),
-                   [](const GRAVITYRecord& record)
-    {
-        return DENSITYRecord {
-            (141.5 / (record.oil_api + 131.5)) * default_water_density,
-            record.water_sg * default_water_density,
-            record.gas_sg * default_air_density
-        };
-    });
-
-    return rho;
-}
-
 }
 
 
@@ -197,7 +168,7 @@ DensityTable make_density_table(const GravityTable& gravity) {
             this->m_densityTable = DensityTable( deck["DENSITY"].back() );
 
         else if( deck.hasKeyword( "GRAVITY" ) )
-            this->m_densityTable = make_density_table( GravityTable ( deck["GRAVITY"].back() ) );
+            this->m_densityTable = DensityTable( GravityTable ( deck["GRAVITY"].back() ) );
 
         if( deck.hasKeyword( "DIFFC" ) )
             this->m_diffCoeffTable = DiffCoeffTable( deck["DIFFC"].back() );
@@ -211,6 +182,12 @@ DensityTable make_density_table(const GravityTable& gravity) {
         if( deck.hasKeyword( "WATDENT" ) )
             this->m_watdentTable = WatdentTable( deck["WATDENT"].back() );
 
+        if( deck.hasKeyword( "SGOFLET" ) )
+            this->m_sgofletTable = SgofletTable( deck["SGOFLET"].back() );
+
+        if( deck.hasKeyword( "SWOFLET" ) )
+            this->m_swofletTable = SwofletTable( deck["SWOFLET"].back() );
+
         if( deck.hasKeyword( "RTEMP" ) )
             m_rtemp = deck["RTEMP"].back().getRecord(0).getItem("TEMP").getSIDouble( 0 );
         else if (deck.hasKeyword( "RTEMPA" ) )
@@ -245,6 +222,16 @@ DensityTable make_density_table(const GravityTable& gravity) {
 
         if (deck.hasKeyword<ParserKeywords::WATDENT>())
             this->watDenT = DenT( deck.get<ParserKeywords::WATDENT>().back());
+        
+        if (deck.hasKeyword<ParserKeywords::GASJT>())
+            this->gasJT = JouleThomson( deck.get<ParserKeywords::GASJT>().back());
+
+        if (deck.hasKeyword<ParserKeywords::OILJT>())
+            this->oilJT = JouleThomson( deck.get<ParserKeywords::OILJT>().back());
+
+        if (deck.hasKeyword<ParserKeywords::WATJT>())
+            this->watJT = JouleThomson( deck.get<ParserKeywords::WATJT>().back());
+
 
         if (deck.hasKeyword<ParserKeywords::STCOND>()) {
             auto stcondKeyword = deck["STCOND"].back();
@@ -276,50 +263,55 @@ DensityTable make_density_table(const GravityTable& gravity) {
     }
 
 
-    TableManager TableManager::serializeObject()
+    TableManager TableManager::serializationTestObject()
     {
         TableManager result;
-        result.m_simpleTables = {{"test", TableContainer::serializeObject()}};
-        result.m_pvtgTables = {PvtgTable::serializeObject()};
-        result.m_pvtgwTables = {PvtgwTable::serializeObject()};
-        result.m_pvtgwoTables = {PvtgwoTable::serializeObject()};
-        result.m_pvtoTables = {PvtoTable::serializeObject()};
-        result.m_rock2dTables = {Rock2dTable::serializeObject()};
-        result.m_rock2dtrTables = {Rock2dtrTable::serializeObject()};
-        result.m_pvtwTable = PvtwTable::serializeObject();
-        result.m_pvcdoTable = PvcdoTable::serializeObject();
-        result.m_densityTable = DensityTable::serializeObject();
-        result.m_diffCoeffTable = DiffCoeffTable::serializeObject();
-        result.m_plyvmhTable = PlyvmhTable::serializeObject();
-        result.m_rockTable = RockTable::serializeObject();
-        result.m_plmixparTable = PlmixparTable::serializeObject();
-        result.m_shrateTable = ShrateTable::serializeObject();
-        result.m_stone1exTable = Stone1exTable::serializeObject();
-        result.m_viscrefTable = ViscrefTable::serializeObject();
-        result.m_watdentTable = WatdentTable::serializeObject();
-        result.m_pvtwsaltTables = {PvtwsaltTable::serializeObject()};
-        result.m_rwgsaltTables = {RwgsaltTable::serializeObject()};
-        result.m_bdensityTables = {BrineDensityTable::serializeObject()};
-        result.m_sdensityTables = {SolventDensityTable::serializeObject()};
-        result.m_plymwinjTables = {{1, Opm::PlymwinjTable::serializeObject()}};
-        result.m_skprwatTables =  {{2, Opm::SkprwatTable::serializeObject()}};
-        result.m_skprpolyTables = {{3, Opm::SkprpolyTable::serializeObject()}};
-        result.m_tabdims = Tabdims::serializeObject();
-        result.m_regdims = Regdims::serializeObject();
-        result.m_eqldims = Eqldims::serializeObject();
+        result.m_simpleTables = {{"test", TableContainer::serializationTestObject()}};
+        result.m_pvtgTables = {PvtgTable::serializationTestObject()};
+        result.m_pvtgwTables = {PvtgwTable::serializationTestObject()};
+        result.m_pvtgwoTables = {PvtgwoTable::serializationTestObject()};
+        result.m_pvtoTables = {PvtoTable::serializationTestObject()};
+        result.m_rock2dTables = {Rock2dTable::serializationTestObject()};
+        result.m_rock2dtrTables = {Rock2dtrTable::serializationTestObject()};
+        result.m_pvtwTable = PvtwTable::serializationTestObject();
+        result.m_pvcdoTable = PvcdoTable::serializationTestObject();
+        result.m_densityTable = DensityTable::serializationTestObject();
+        result.m_diffCoeffTable = DiffCoeffTable::serializationTestObject();
+        result.m_plyvmhTable = PlyvmhTable::serializationTestObject();
+        result.m_rockTable = RockTable::serializationTestObject();
+        result.m_plmixparTable = PlmixparTable::serializationTestObject();
+        result.m_shrateTable = ShrateTable::serializationTestObject();
+        result.m_stone1exTable = Stone1exTable::serializationTestObject();
+        result.m_viscrefTable = ViscrefTable::serializationTestObject();
+        result.m_watdentTable = WatdentTable::serializationTestObject();
+        result.m_sgofletTable = SgofletTable::serializationTestObject();
+        result.m_swofletTable = SwofletTable::serializationTestObject();
+        result.m_pvtwsaltTables = {PvtwsaltTable::serializationTestObject()};
+        result.m_rwgsaltTables = {RwgsaltTable::serializationTestObject()};
+        result.m_bdensityTables = {BrineDensityTable::serializationTestObject()};
+        result.m_sdensityTables = {SolventDensityTable::serializationTestObject()};
+        result.m_plymwinjTables = {{1, Opm::PlymwinjTable::serializationTestObject()}};
+        result.m_skprwatTables =  {{2, Opm::SkprwatTable::serializationTestObject()}};
+        result.m_skprpolyTables = {{3, Opm::SkprpolyTable::serializationTestObject()}};
+        result.m_tabdims = Tabdims::serializationTestObject();
+        result.m_regdims = Regdims::serializationTestObject();
+        result.m_eqldims = Eqldims::serializationTestObject();
         result.hasImptvd = true;
         result.hasEnptvd = true;
         result.hasEqlnum = true;
         result.hasShrate = true;
-        result.jfunc = Opm::JFunc::serializeObject();
-        result.oilDenT = DenT::serializeObject();
-        result.gasDenT = DenT::serializeObject();
-        result.watDenT = DenT::serializeObject();
-        result.stcond = StandardCond::serializeObject();
+        result.jfunc = Opm::JFunc::serializationTestObject();
+        result.oilDenT = DenT::serializationTestObject();
+        result.gasDenT = DenT::serializationTestObject();
+        result.watDenT = DenT::serializationTestObject();
+        result.oilJT = JouleThomson::serializationTestObject();
+        result.gasJT = JouleThomson::serializationTestObject();
+        result.watJT = JouleThomson::serializationTestObject();
+        result.stcond = StandardCond::serializationTestObject();
         result.m_gas_comp_index = 77;
         result.m_rtemp = 1.0;
         result.m_salinity = 1.0;
-        result.m_tlmixpar = TLMixpar::serializeObject();
+        result.m_tlmixpar = TLMixpar::serializationTestObject();
         return result;
     }
 
@@ -396,6 +388,18 @@ DensityTable make_density_table(const GravityTable& gravity) {
         return this->oilDenT;
     }
 
+    const JouleThomson& TableManager::WatJT() const {
+        return this->watJT;
+    }
+
+    const JouleThomson& TableManager::GasJT() const {
+        return this->gasJT;
+    }
+
+    const JouleThomson& TableManager::OilJT() const {
+        return this->oilJT;
+    }
+
     const StandardCond& TableManager::stCond() const {
         return this->stcond;
     }
@@ -439,6 +443,7 @@ DensityTable make_density_table(const GravityTable& gravity) {
         addTables( "PLYMAX", m_regdims.getNPLMIX());
         addTables( "RSVD", m_eqldims.getNumEquilRegions());
         addTables( "RVVD", m_eqldims.getNumEquilRegions());
+        addTables( "RVWVD", m_eqldims.getNumEquilRegions());
         addTables( "PBVD", m_eqldims.getNumEquilRegions());
         addTables( "PDVD", m_eqldims.getNumEquilRegions());
         addTables( "SALTVD", m_eqldims.getNumEquilRegions());
@@ -505,6 +510,7 @@ DensityTable make_density_table(const GravityTable& gravity) {
 
         initSimpleTableContainer<RsvdTable>(deck, "RSVD" , m_eqldims.getNumEquilRegions());
         initSimpleTableContainer<RvvdTable>(deck, "RVVD" , m_eqldims.getNumEquilRegions());
+        initSimpleTableContainer<RvwvdTable>(deck, "RVWVD" , m_eqldims.getNumEquilRegions());
         initSimpleTableContainer<PbvdTable>(deck, "PBVD" , m_eqldims.getNumEquilRegions());
         initSimpleTableContainer<PdvdTable>(deck, "PDVD" , m_eqldims.getNumEquilRegions());
         initSimpleTableContainer<SaltpvdTable>(deck, "SALTPVD" , m_eqldims.getNumEquilRegions());
@@ -838,7 +844,7 @@ DensityTable make_density_table(const GravityTable& gravity) {
             const auto& tableRecord = rocktabKeyword.getRecord( tableIdx );
             const auto& dataItem = tableRecord.getItem( 0 );
             if (dataItem.data_size() > 0) {
-                std::shared_ptr<RocktabTable> table = std::make_shared<RocktabTable>( dataItem , isDirectional, useStressOption );
+                std::shared_ptr<RocktabTable> table = std::make_shared<RocktabTable>( dataItem , isDirectional, useStressOption, tableIdx );
                 container.addTable( tableIdx , table );
             }
         }
@@ -919,6 +925,9 @@ DensityTable make_density_table(const GravityTable& gravity) {
         return getTables("RVVD");
     }
 
+    const TableContainer& TableManager::getRvwvdTables() const {
+        return getTables("RVWVD");
+    }
     const TableContainer& TableManager::getPbvdTables() const {
         return getTables("PBVD");
     }
@@ -1117,6 +1126,14 @@ DensityTable make_density_table(const GravityTable& gravity) {
         return this->m_watdentTable;
     }
 
+    const SgofletTable& TableManager::getSgofletTable() const {
+        return this->m_sgofletTable;
+    }
+
+    const SwofletTable& TableManager::getSwofletTable() const {
+        return this->m_swofletTable;
+    }
+
     const TableContainer& TableManager::getMsfnTables() const {
         return getTables("MSFN");
     }
@@ -1241,6 +1258,8 @@ DensityTable make_density_table(const GravityTable& gravity) {
                m_stone1exTable == data.m_stone1exTable &&
                m_viscrefTable == data.m_viscrefTable &&
                m_watdentTable == data.m_watdentTable &&
+               m_sgofletTable == data.m_sgofletTable &&
+               m_swofletTable == data.m_swofletTable &&
                m_pvtwsaltTables == data.m_pvtwsaltTables &&
                m_rwgsaltTables == data.m_rwgsaltTables &&
                m_bdensityTables == data.m_bdensityTables &&
@@ -1260,6 +1279,9 @@ DensityTable make_density_table(const GravityTable& gravity) {
                gasDenT == data.gasDenT &&
                oilDenT == data.oilDenT &&
                watDenT == data.watDenT &&
+               gasJT == data.gasJT &&
+               oilJT == data.oilJT &&
+               watJT == data.watJT &&
                stcond == data.stcond &&
                jfunc == data.jfunc &&
                m_rtemp == data.m_rtemp &&
@@ -1380,10 +1402,12 @@ DensityTable make_density_table(const GravityTable& gravity) {
             }
         }
 
-        if (auto prtLog = OpmLog::getBackend<EclipsePRTLog>("ECLIPSEPRTLOG")) {
+        if (OpmLog::hasBackend("ECLIPSEPRTLOG")) {
+            auto prtLog = OpmLog::getBackend<EclipsePRTLog>("ECLIPSEPRTLOG");
             prtLog->addMessage(Log::MessageType::Warning, prt.str());
         }
-        if (auto consoleLog = OpmLog::getBackend<StreamLog>("STDOUT_LOGGER")) {
+        if (OpmLog::hasBackend("STDOUT_LOGGER")) {
+            auto consoleLog = OpmLog::getBackend<StreamLog>("STDOUT_LOGGER");
             consoleLog->addMessage(Log::MessageType::Warning, console.str());
         }
     }
@@ -1516,8 +1540,12 @@ DensityTable make_density_table(const GravityTable& gravity) {
         for (size_t tableIdx = 0; tableIdx < tableKeyword.size(); ++tableIdx) {
             const auto& dataItem = tableKeyword.getRecord( tableIdx ).getItem("DATA");
             if (dataItem.data_size() > 0) {
-                std::shared_ptr<TableType> table = std::make_shared<TableType>( dataItem, useJFunc() );
-                container.addTable( tableIdx , table );
+                try {
+                    std::shared_ptr<TableType> table = std::make_shared<TableType>( dataItem, useJFunc(), tableIdx );
+                    container.addTable( tableIdx , table );
+                } catch (const std::runtime_error& err) {
+                    throw OpmInputError(err, tableKeyword.location());
+                }
             }
         }
     }
@@ -1537,12 +1565,24 @@ DensityTable make_density_table(const GravityTable& gravity) {
             return;
         }
 
+        auto lastComplete = 0 * numTables;
         const auto& tableKeyword = deck[keywordName].back();
         for (size_t tableIdx = 0; tableIdx < tableKeyword.size(); ++tableIdx) {
             const auto& dataItem = tableKeyword.getRecord( tableIdx ).getItem("DATA");
             if (dataItem.data_size() > 0) {
-                std::shared_ptr<TableType> table = std::make_shared<TableType>( dataItem );
+                std::shared_ptr<TableType> table = std::make_shared<TableType>( dataItem, tableIdx );
                 container.addTable( tableIdx , table );
+                lastComplete = tableIdx;
+            }
+            else if (tableIdx > static_cast<size_t>(0)) {
+                const auto& item = tableKeyword.getRecord(lastComplete).getItem("DATA");
+                container.addTable(tableIdx, std::make_shared<TableType>(item, tableIdx));
+            }
+            else {
+                throw OpmInputError {
+                    fmt::format("Cannot default region {}'s table data", tableIdx + 1),
+                    tableKeyword.location()
+                };
             }
         }
     }
