@@ -15,7 +15,75 @@
 
   You should have received a copy of the GNU General Public License
   along with OPM.  If not, see <http://www.gnu.org/licenses/>.
- */
+*/
+
+#include <opm/input/eclipse/Schedule/Schedule.hpp>
+
+#include <opm/io/eclipse/rst/state.hpp>
+
+#include <opm/common/OpmLog/LogUtil.hpp>
+#include <opm/common/OpmLog/OpmLog.hpp>
+#include <opm/common/utility/OpmInputError.hpp>
+#include <opm/common/utility/String.hpp>
+#include <opm/common/utility/numeric/cmp.hpp>
+#include <opm/common/utility/shmatch.hpp>
+
+#include <opm/input/eclipse/EclipseState/EclipseState.hpp>
+#include <opm/input/eclipse/EclipseState/TracerConfig.hpp>
+
+#include <opm/input/eclipse/Python/Python.hpp>
+
+#include <opm/input/eclipse/Schedule/Action/ActionResult.hpp>
+#include <opm/input/eclipse/Schedule/Action/ActionX.hpp>
+#include <opm/input/eclipse/Schedule/Action/State.hpp>
+#include <opm/input/eclipse/Schedule/Group/GConSale.hpp>
+#include <opm/input/eclipse/Schedule/Group/GConSump.hpp>
+#include <opm/input/eclipse/Schedule/Group/GTNode.hpp>
+#include <opm/input/eclipse/Schedule/MSW/SICD.hpp>
+#include <opm/input/eclipse/Schedule/MSW/Valve.hpp>
+#include <opm/input/eclipse/Schedule/MSW/WellSegments.hpp>
+#include <opm/input/eclipse/Schedule/Network/Balance.hpp>
+#include <opm/input/eclipse/Schedule/Network/Node.hpp>
+#include <opm/input/eclipse/Schedule/OilVaporizationProperties.hpp>
+#include <opm/input/eclipse/Schedule/RPTConfig.hpp>
+#include <opm/input/eclipse/Schedule/ScheduleGrid.hpp>
+#include <opm/input/eclipse/Schedule/SummaryState.hpp>
+#include <opm/input/eclipse/Schedule/Tuning.hpp>
+#include <opm/input/eclipse/Schedule/UDQ/UDQActive.hpp>
+#include <opm/input/eclipse/Schedule/UDQ/UDQConfig.hpp>
+#include <opm/input/eclipse/Schedule/Well/WList.hpp>
+#include <opm/input/eclipse/Schedule/Well/WListManager.hpp>
+#include <opm/input/eclipse/Schedule/Well/WellBrineProperties.hpp>
+#include <opm/input/eclipse/Schedule/Well/WellConnections.hpp>
+#include <opm/input/eclipse/Schedule/Well/WellFoamProperties.hpp>
+#include <opm/input/eclipse/Schedule/Well/WellMICPProperties.hpp>
+#include <opm/input/eclipse/Schedule/Well/WellPolymerProperties.hpp>
+#include <opm/input/eclipse/Schedule/SummaryState.hpp>
+
+#include <opm/input/eclipse/Units/Dimension.hpp>
+#include <opm/input/eclipse/Units/UnitSystem.hpp>
+#include <opm/input/eclipse/Units/Units.hpp>
+
+#include <opm/input/eclipse/Parser/ErrorGuard.hpp>
+#include <opm/input/eclipse/Parser/ParseContext.hpp>
+#include <opm/input/eclipse/Parser/ParserKeywords/A.hpp>
+#include <opm/input/eclipse/Parser/ParserKeywords/B.hpp>
+#include <opm/input/eclipse/Parser/ParserKeywords/C.hpp>
+#include <opm/input/eclipse/Parser/ParserKeywords/E.hpp>
+#include <opm/input/eclipse/Parser/ParserKeywords/L.hpp>
+#include <opm/input/eclipse/Parser/ParserKeywords/P.hpp>
+#include <opm/input/eclipse/Parser/ParserKeywords/R.hpp>
+#include <opm/input/eclipse/Parser/ParserKeywords/S.hpp>
+#include <opm/input/eclipse/Parser/ParserKeywords/W.hpp>
+
+#include <opm/input/eclipse/Deck/Deck.hpp>
+#include <opm/input/eclipse/Deck/DeckItem.hpp>
+#include <opm/input/eclipse/Deck/DeckKeyword.hpp>
+#include <opm/input/eclipse/Deck/DeckRecord.hpp>
+#include <opm/input/eclipse/Deck/DeckSection.hpp>
+
+#include "Well/injection.hpp"
+#include "MSW/Compsegs.hpp"
 
 #include <algorithm>
 #include <ctime>
@@ -29,70 +97,9 @@
 #include <utility>
 #include <vector>
 
+#include <fmt/chrono.h>
 #include <fmt/format.h>
 
-#include <opm/common/OpmLog/LogUtil.hpp>
-#include <opm/common/OpmLog/OpmLog.hpp>
-#include <opm/common/utility/numeric/cmp.hpp>
-#include <opm/common/utility/String.hpp>
-#include <opm/common/utility/shmatch.hpp>
-#include <opm/common/utility/OpmInputError.hpp>
-
-#include <opm/io/eclipse/rst/state.hpp>
-
-#include <opm/input/eclipse/Python/Python.hpp>
-#include <opm/input/eclipse/Deck/DeckItem.hpp>
-#include <opm/input/eclipse/Deck/DeckKeyword.hpp>
-#include <opm/input/eclipse/Deck/DeckRecord.hpp>
-#include <opm/input/eclipse/Deck/DeckSection.hpp>
-#include <opm/input/eclipse/Parser/ErrorGuard.hpp>
-#include <opm/input/eclipse/Parser/ParseContext.hpp>
-#include <opm/input/eclipse/Parser/ParserKeywords/A.hpp>
-#include <opm/input/eclipse/Parser/ParserKeywords/B.hpp>
-#include <opm/input/eclipse/Parser/ParserKeywords/C.hpp>
-#include <opm/input/eclipse/Parser/ParserKeywords/E.hpp>
-#include <opm/input/eclipse/Parser/ParserKeywords/L.hpp>
-#include <opm/input/eclipse/Parser/ParserKeywords/P.hpp>
-#include <opm/input/eclipse/Parser/ParserKeywords/R.hpp>
-#include <opm/input/eclipse/Parser/ParserKeywords/S.hpp>
-#include <opm/input/eclipse/Parser/ParserKeywords/W.hpp>
-
-#include <opm/input/eclipse/EclipseState/TracerConfig.hpp>
-#include <opm/input/eclipse/EclipseState/EclipseState.hpp>
-#include <opm/input/eclipse/Schedule/Action/State.hpp>
-#include <opm/input/eclipse/Schedule/Action/ActionX.hpp>
-#include <opm/input/eclipse/Schedule/Action/ActionResult.hpp>
-#include <opm/input/eclipse/Schedule/MSW/SICD.hpp>
-#include <opm/input/eclipse/Schedule/MSW/Valve.hpp>
-#include <opm/input/eclipse/Schedule/MSW/WellSegments.hpp>
-#include <opm/input/eclipse/Schedule/Group/GConSump.hpp>
-#include <opm/input/eclipse/Schedule/Group/GConSale.hpp>
-
-#include <opm/input/eclipse/Schedule/OilVaporizationProperties.hpp>
-#include <opm/input/eclipse/Schedule/UDQ/UDQConfig.hpp>
-#include <opm/input/eclipse/Schedule/UDQ/UDQActive.hpp>
-#include <opm/input/eclipse/Schedule/RPTConfig.hpp>
-#include <opm/input/eclipse/Schedule/Schedule.hpp>
-#include <opm/input/eclipse/Schedule/ScheduleGrid.hpp>
-#include <opm/input/eclipse/Schedule/Tuning.hpp>
-#include <opm/input/eclipse/Schedule/Network/Node.hpp>
-#include <opm/input/eclipse/Schedule/Well/WList.hpp>
-#include <opm/input/eclipse/Schedule/Well/WListManager.hpp>
-#include <opm/input/eclipse/Schedule/Well/WellFoamProperties.hpp>
-#include <opm/input/eclipse/Schedule/Well/WellInjectionProperties.hpp>
-#include <opm/input/eclipse/Schedule/Well/WellMICPProperties.hpp>
-#include <opm/input/eclipse/Schedule/Well/WellPolymerProperties.hpp>
-#include <opm/input/eclipse/Schedule/Well/WellProductionProperties.hpp>
-#include <opm/input/eclipse/Schedule/Well/WellBrineProperties.hpp>
-#include <opm/input/eclipse/Schedule/Well/WellConnections.hpp>
-#include <opm/input/eclipse/Schedule/SummaryState.hpp>
-#include <opm/input/eclipse/Units/Dimension.hpp>
-#include <opm/input/eclipse/Units/UnitSystem.hpp>
-#include <opm/input/eclipse/Units/Units.hpp>
-
-#include "Well/injection.hpp"
-#include "MSW/Compsegs.hpp"
-
 namespace {
 
     bool name_match(const std::string& pattern, const std::string& name) {
@@ -271,20 +278,23 @@ Schedule::Schedule(const Deck& deck, const EclipseState& es, const std::optional
     }
 
     /*
-      In general the serializeObject() instances are used as targets for
+      In general the serializationTestObject() instances are used as targets for
       deserialization, i.e. the serialized buffer is unpacked into this
       instance. However the Schedule object is a top level object, and the
       simulator will instantiate and manage a Schedule object to unpack into, so
       the instance created here is only for testing.
     */
-    Schedule Schedule::serializeObject()
+    Schedule Schedule::serializationTestObject()
     {
         Schedule result;
 
-        result.m_static = ScheduleStatic::serializeObject();
-        result.snapshots = { ScheduleState::serializeObject() };
-        result.m_sched_deck = ScheduleDeck::serializeObject();
-        result.restart_output = WriteRestartFileEvents::serializeObject();
+        result.m_static = ScheduleStatic::serializationTestObject();
+        result.m_sched_deck = ScheduleDeck::serializationTestObject();
+        result.action_wgnames = Action::WGNames::serializationTestObject();
+        result.exit_status = EXIT_FAILURE;
+        result.snapshots = { ScheduleState::serializationTestObject() };
+        result.restart_output = WriteRestartFileEvents::serializationTestObject();
+        result.completed_cells = CompletedCells::serializationTestObject();
 
         return result;
     }
@@ -293,11 +303,11 @@ Schedule::Schedule(const Deck& deck, const EclipseState& es, const std::optional
         return this->posixStartTime( );
     }
 
-    time_t Schedule::posixStartTime() const {
+    std::time_t Schedule::posixStartTime() const {
         return std::chrono::system_clock::to_time_t(this->m_sched_deck[0].start_time());
     }
 
-    time_t Schedule::posixEndTime() const {
+    std::time_t Schedule::posixEndTime() const {
         // This should indeed access the start_time() property of the last
         // snapshot.
         return std::chrono::system_clock::to_time_t(this->snapshots.back().start_time());
@@ -312,8 +322,10 @@ Schedule::Schedule(const Deck& deck, const EclipseState& es, const std::optional
                                  const ScheduleGrid& grid,
                                  const std::vector<std::string>& matching_wells,
                                  bool actionx_mode,
-                                 SimulatorUpdate * sim_update,
-                                 const std::unordered_map<std::string, double> * target_wellpi) {
+                                 SimulatorUpdate* sim_update,
+                                 const std::unordered_map<std::string, double>* target_wellpi,
+                                 std::unordered_map<std::string, double>* wpimult_global_factor)
+    {
 
         static const std::unordered_set<std::string> require_grid = {
             "COMPDAT",
@@ -321,7 +333,8 @@ Schedule::Schedule(const Deck& deck, const EclipseState& es, const std::optional
         };
 
 
-        HandlerContext handlerContext { block, keyword, grid, currentStep, matching_wells, actionx_mode, parseContext, errors, sim_update, target_wellpi};
+        HandlerContext handlerContext { block, keyword, grid, currentStep, matching_wells, actionx_mode, parseContext, errors, sim_update, target_wellpi,
+                                       wpimult_global_factor};
         /*
           The grid and fieldProps members create problems for reiterating the
           Schedule section. We therefor single them out very clearly here.
@@ -335,73 +348,138 @@ Schedule::Schedule(const Deck& deck, const EclipseState& es, const std::optional
 
 namespace {
 
-class ScheduleLogger {
+class ScheduleLogger
+{
 public:
-    ScheduleLogger(bool restart_skip, const std::string& prefix_, const KeywordLocation& location)
-        : prefix(prefix_)
-        , current_file(location.filename)
+    enum class Stream
     {
-        if (restart_skip)
-            this->log_function = &OpmLog::note;
-        else
-            this->log_function = &OpmLog::info;
+        Info, Note, Debug,
+    };
+
+    ScheduleLogger(const Stream stream, const std::string& prefix, const KeywordLocation& location)
+        : stream_      {stream}
+        , prefix_      {prefix}
+        , current_file_{location.filename}
+    {
+        switch (this->stream_) {
+        case Stream::Info:
+            this->log_function_ = &OpmLog::info;
+            break;
+
+        case Stream::Note:
+            this->log_function_ = &OpmLog::note;
+            break;
+
+        case Stream::Debug:
+            this->log_function_ = &OpmLog::debug;
+            break;
+        }
     }
 
-    void operator()(const std::string& msg) {
-        this->log_function(this->prefix + msg);
+    void operator()(const std::string& msg)
+    {
+        this->log_function_(this->format_message(msg));
     }
 
-    void info(const std::string& msg) {
-        OpmLog::info(this->prefix + msg);
+    void operator()(const std::vector<std::string>& msg_list)
+    {
+        std::for_each(msg_list.begin(), msg_list.end(),
+                      [this](const std::string& record)
+                      {
+                          (*this)(record);
+                      });
     }
 
-    void info(const std::vector<std::string>& msg_list) {
-        for (const auto& msg : msg_list)
-            this->info(msg);
+    void info(const std::string& msg)
+    {
+        OpmLog::info(this->format_message(msg));
     }
 
-    void complete_step(const std::string& msg) {
-        this->step_count += 1;
-        if (this->step_count == this->max_print) {
-            this->log_function(this->prefix + msg);
-            this->info(std::vector<std::string>{"Report limit reached, see PRT-file for remaining Schedule initialization.", ""});
-            this->log_function = &OpmLog::note;
-        } else {
-            this->log_function( this->prefix + msg );
-            this->log_function( this->prefix );
+    void complete_step(const std::string& msg)
+    {
+        (*this)(msg);
+
+        this->step_count_ += 1;
+        if (this->step_count_ == this->max_print_) {
+            this->redirect(&OpmLog::note, {"Report limit reached, see PRT-file for remaining Schedule initialization.", ""});
+        }
+        else {
+            // Blank line
+            (*this)("");
         }
     };
 
-    void restart() {
-        this->step_count = 0;
-        this->log_function = &OpmLog::info;
+    void restart()
+    {
+        this->step_count_ = 0;
+        this->redirect(&OpmLog::info);
     }
 
-    void location(const KeywordLocation& location) {
-        if (this->current_file == location.filename)
+    void location(const KeywordLocation& location)
+    {
+        if (this->current_file_ == location.filename) {
             return;
+        }
 
-        this->operator()( fmt::format("Reading from: {} line {}", location.filename, location.lineno) );
-        this->current_file = location.filename;
+        (*this)(fmt::format("Reading from: {} line {}", location.filename, location.lineno));
+        this->current_file_ = location.filename;
     }
 
+    static Stream select_stream(const bool log_to_debug,
+                                const bool restart_skip);
 
 private:
-    std::size_t step_count = 0;
-    std::size_t max_print  = 5;
-    std::string prefix;
-    std::string current_file;
-    void (*log_function)(const std::string&);
+    using LogFunction = void (*) (const std::string&);
+
+    Stream stream_{Stream::Info};
+    std::size_t step_count_ = 0;
+    std::size_t max_print_  = 5;
+    std::string prefix_{};
+    std::string current_file_{};
+
+    LogFunction log_function_{};
+
+    void redirect(const LogFunction               new_stream,
+                  const std::vector<std::string>& messages = {})
+    {
+        if (this->stream_ == Stream::Debug) {
+            // If we're writing to OpmLog::debug then continue to do so.
+            return;
+        }
+
+        this->log_function_ = new_stream;
+        (*this)(messages);
+    }
+
+    std::string format_message(const std::string& message) const
+    {
+        return fmt::format("{}{}", this->prefix_, message);
+    }
 };
 
+    ScheduleLogger::Stream
+    ScheduleLogger::select_stream(const bool log_to_debug,
+                                  const bool restart_skip)
+    {
+        if (log_to_debug) {
+            return ScheduleLogger::Stream::Debug;
+        }
+
+        if (restart_skip) {
+            return ScheduleLogger::Stream::Note;
+        }
+
+        return ScheduleLogger::Stream::Info;
+    }
 }
 
 void Schedule::iterateScheduleSection(std::size_t load_start, std::size_t load_end,
-                                      const ParseContext& parseContext ,
+                                      const ParseContext& parseContext,
                                       ErrorGuard& errors,
                                       const ScheduleGrid& grid,
                                       const std::unordered_map<std::string, double> * target_wellpi,
-                                      const std::string& prefix) {
+                                      const std::string& prefix,
+                                      const bool log_to_debug) {
 
         std::vector<std::pair< const DeckKeyword* , std::size_t> > rftProperties;
         std::string time_unit = this->m_static.m_unit_system.name(UnitSystem::measure::time);
@@ -429,13 +507,18 @@ void Schedule::iterateScheduleSection(std::size_t load_start, std::size_t load_e
                these keywords will be assigned to report step 0.
         */
 
-        auto restart_skip = load_start < this->m_static.rst_info.report_step;
-        ScheduleLogger logger(restart_skip, prefix, this->m_sched_deck.location());
+        const auto restart_skip = load_start < this->m_static.rst_info.report_step;
+        ScheduleLogger logger(ScheduleLogger::select_stream(log_to_debug, restart_skip),
+                              prefix, this->m_sched_deck.location());
         {
             const auto& location = this->m_sched_deck.location();
-            logger.info({"", "Processing dynamic information from", fmt::format("{} line {}", location.filename, location.lineno)});
-            if (restart_skip)
-                logger.info(fmt::format("This is a restarted run - skipping until report step {} at {}", this->m_static.rst_info.report_step, Schedule::formatDate(this->m_static.rst_info.time)));
+            logger({"", "Processing dynamic information from", fmt::format("{} line {}", location.filename, location.lineno)});
+            if (restart_skip && !log_to_debug) {
+                logger.info(fmt::format("This is a restarted run - skipping "
+                                        "until report step {} at {}",
+                                        this->m_static.rst_info.report_step,
+                                        Schedule::formatDate(this->m_static.rst_info.time)));
+            }
 
             logger(fmt::format("Initializing report step {}/{} at {} {} {} line {}",
                                load_start,
@@ -474,6 +557,7 @@ void Schedule::iterateScheduleSection(std::size_t load_start, std::size_t load_e
             }
             this->create_next(block);
 
+            std::unordered_map<std::string, double> wpimult_global_factor;
             while (true) {
                 if (keyword_index == block.size())
                     break;
@@ -520,15 +604,26 @@ void Schedule::iterateScheduleSection(std::size_t load_start, std::size_t load_e
                                     {},
                                     false,
                                     nullptr,
-                                    target_wellpi);
+                                    target_wellpi,
+                                    &wpimult_global_factor);
                 keyword_index++;
             }
 
+            this->applyGlobalWPIMULT(wpimult_global_factor);
             this->end_report(report_step);
 
             if (this->must_write_rst_file(report_step)) {
                 this->restart_output.addRestartOutput(report_step);
             }
+        } // for (auto report_step = load_start
+    }
+
+    void Schedule::applyGlobalWPIMULT( const std::unordered_map<std::string, double>& wpimult_global_factor) {
+        for (const auto& [well_name, factor] : wpimult_global_factor) {
+            auto well = this->snapshots.back().wells(well_name);
+            if (well.applyGlobalWPIMULT(factor)) {
+                this->snapshots.back().wells.update(std::move(well));
+            }
         }
     }
 
@@ -1256,13 +1351,24 @@ File {} line {}.)", pattern, location.keyword, location.filename, location.linen
         return std::chrono::duration_cast<std::chrono::seconds>(elapsed).count();
     }
 
-    time_t Schedule::simTime(std::size_t timeStep) const {
+    std::time_t Schedule::simTime(std::size_t timeStep) const {
         return std::chrono::system_clock::to_time_t( this->snapshots[timeStep].start_time() );
     }
 
     double Schedule::stepLength(std::size_t timeStep) const {
-        auto elapsed = this->snapshots[timeStep].end_time() - this->snapshots[timeStep].start_time();
-        return std::chrono::duration_cast<std::chrono::seconds>(elapsed).count();
+        const auto start = this->snapshots[timeStep].start_time();
+        const auto end = this->snapshots[timeStep].end_time();
+        if (start > end) {
+            throw std::invalid_argument {
+                    fmt::format(" Report step {} has start time after end time,\n"
+                                "   * Start time = {:%d-%b-%Y %H:%M:%S}\n"
+                                "   * End time   = {:%d-%b-%Y %H:%M:%S}.\n"
+                                " Possibly due to inconsistent RESTART/SKIPREST settings.",
+                                timeStep + 1,
+                                fmt::gmtime(TimeService::to_time_t(start)),
+                                fmt::gmtime(TimeService::to_time_t(end))) };
+        }
+        return std::chrono::duration_cast<std::chrono::seconds>(end - start).count();
     }
 
     void Schedule::applyKeywords(
@@ -1276,6 +1382,7 @@ File {} line {}.)", pattern, location.keyword, location.filename, location.linen
         const std::string prefix = "| "; /* logger prefix string */
         this->snapshots.resize(reportStep + 1);
         auto& input_block = this->m_sched_deck[reportStep];
+        std::unordered_map<std::string, double> wpimult_global_factor;
         for (auto keyword : keywords) {
             input_block.push_back(*keyword);
             this->handleKeyword(reportStep,
@@ -1287,8 +1394,10 @@ File {} line {}.)", pattern, location.keyword, location.filename, location.linen
                                 matching_wells,
                                 /*actionx_mode=*/false,
                                 &sim_update,
-                                &target_wellpi);
+                                &target_wellpi,
+                                &wpimult_global_factor);
         }
+        this->applyGlobalWPIMULT(wpimult_global_factor);
         this->end_report(reportStep);
         if (reportStep < this->m_sched_deck.size() - 1) {
             iterateScheduleSection(
@@ -1310,14 +1419,15 @@ File {} line {}.)", pattern, location.keyword, location.filename, location.linen
         SimulatorUpdate sim_update;
         ScheduleGrid grid(this->completed_cells);
 
-        OpmLog::info("/----------------------------------------------------------------------");
-        OpmLog::info(fmt::format("{0}Action {1} evaluated to true. Will add action keywords and\n{0}rerun Schedule section.\n{0}", prefix, action.name()));
+        OpmLog::debug("/----------------------------------------------------------------------");
+        OpmLog::debug(fmt::format("{0}Action {1} triggered. Will add action keywords and\n{0}rerun Schedule section.\n{0}", prefix, action.name()));
         this->snapshots.resize(reportStep + 1);
         auto& input_block = this->m_sched_deck[reportStep];
+        std::unordered_map<std::string, double> wpimult_global_factor;
         for (const auto& keyword : action) {
             input_block.push_back(keyword);
             const auto& location = keyword.location();
-            OpmLog::info(fmt::format("{}Processing keyword {} from {} line {}", prefix, location.keyword, location.filename, location.lineno));
+            OpmLog::debug(fmt::format("{}Processing keyword {} from {} line {}", prefix, location.keyword, location.filename, location.lineno));
             this->handleKeyword(reportStep,
                                 input_block,
                                 keyword,
@@ -1327,8 +1437,10 @@ File {} line {}.)", pattern, location.keyword, location.filename, location.linen
                                 matching_wells,
                                 true,
                                 &sim_update,
-                                &target_wellpi);
+                                &target_wellpi,
+                                &wpimult_global_factor);
         }
+        this->applyGlobalWPIMULT(wpimult_global_factor);
         this->end_report(reportStep);
         if (!sim_update.affected_wells.empty()) {
             this->snapshots.back().events().addEvent( ScheduleEvents::ACTIONX_WELL_EVENT );
@@ -1336,9 +1448,13 @@ File {} line {}.)", pattern, location.keyword, location.filename, location.linen
                 this->snapshots.back().wellgroup_events().addEvent(well, ScheduleEvents::ACTIONX_WELL_EVENT);
         }
 
-        if (reportStep < this->m_sched_deck.size() - 1)
-            iterateScheduleSection(reportStep + 1, this->m_sched_deck.size(), parseContext, errors, grid, &target_wellpi, prefix);
-        OpmLog::info("\\----------------------------------------------------------------------");
+        if (reportStep < this->m_sched_deck.size() - 1) {
+            const auto log_to_debug = true;
+            this->iterateScheduleSection(reportStep + 1, this->m_sched_deck.size(),
+                                         parseContext, errors, grid, &target_wellpi,
+                                         prefix, log_to_debug);
+        }
+        OpmLog::debug("\\----------------------------------------------------------------------");
 
         return sim_update;
     }
@@ -1510,9 +1626,10 @@ File {} line {}.)", pattern, location.keyword, location.filename, location.linen
     bool Schedule::operator==(const Schedule& data) const {
         return this->m_static == data.m_static &&
                this->m_sched_deck == data.m_sched_deck &&
+               this->action_wgnames == data.action_wgnames &&
+               this->exit_status == data.exit_status &&
                this->snapshots == data.snapshots &&
                this->restart_output == data.restart_output &&
-               this->action_wgnames == data.action_wgnames &&
                this->completed_cells == data.completed_cells;
      }
 
@@ -1675,26 +1792,27 @@ namespace {
             for (const auto& [control, value, wgname, ig_phase] : uda_records) {
                 if (UDQ::well_control(control)) {
                     auto& well = this->snapshots.back().wells.get(wgname);
-                    if (UDQ::injection_control(control)) {
+
+                    if (UDQ::is_well_injection_control(control, well.isInjector())) {
                         auto injection_properties = std::make_shared<Well::WellInjectionProperties>(well.getInjectionProperties());
                         injection_properties->update_uda(udq_config, udq_active, control, value);
                         well.updateInjection(std::move(injection_properties));
                     }
 
-                    if (UDQ::production_control(control)) {
+                    if (UDQ::is_well_production_control(control, well.isProducer())) {
                         auto production_properties = std::make_shared<Well::WellProductionProperties>(well.getProductionProperties());
                         production_properties->update_uda(udq_config, udq_active, control, value);
                         well.updateProduction(std::move(production_properties));
                     }
                 } else {
                     auto& group = this->snapshots.back().groups.get(wgname);
-                    if (UDQ::injection_control(control)) {
+                    if (UDQ::is_group_injection_control(control)) {
                         auto injection_properties = group.injectionProperties(ig_phase.value());
                         injection_properties.update_uda(udq_config, udq_active, control, value);
                         group.updateInjection(injection_properties);
                     }
 
-                    if (UDQ::production_control(control)) {
+                    if (UDQ::is_group_production_control(control)) {
                         auto production_properties = group.productionProperties();
                         production_properties.update_uda(udq_config, udq_active, control, value);
                         group.updateProduction(production_properties);
@@ -1710,8 +1828,10 @@ namespace {
                 actions.add( Action::ActionX(rst_action) );
             this->snapshots.back().actions.update( std::move(actions) );
         }
+
         this->snapshots.back().wtest_config.update( WellTestConfig{rst_state, report_step});
 
+        this->snapshots.back().network_balance.update(Network::Balance { rst_state.netbalan });
 
         if (!rst_state.wlists.empty())
             this->snapshots.back().wlist_manager.update( WListManager(rst_state) );
@@ -2050,7 +2170,7 @@ void Schedule::create_first(const time_point& start_time, const std::optional<ti
     sched_state.guide_rate.update( GuideRateConfig() );
     sched_state.rft_config.update( RFTConfig() );
     sched_state.rst_config.update( RSTConfig::first( this->m_static.rst_config ) );
-    sched_state.network_balance.update( Network::Balance(runspec.networkDimensions().active(), sched_state.tuning()) );
+    sched_state.network_balance.update(Network::Balance{ runspec.networkDimensions().active() });
     sched_state.update_sumthin(this->m_static.sumthin);
     sched_state.rptonly(this->m_static.rptonly);
     //sched_state.update_date( start_time );
