@@ -158,7 +158,7 @@ VFPProdTable::VFPProdTable(int table_num,
 }
 
 
-VFPProdTable VFPProdTable::serializeObject()
+VFPProdTable VFPProdTable::serializationTestObject()
 {
     VFPProdTable result;
     result.m_table_num = 1;
@@ -173,7 +173,7 @@ VFPProdTable VFPProdTable::serializeObject()
     result.m_gfr_data = {8.0};
     result.m_alq_data = {9.0, 10.0, 11.0};
     result.m_data = {12.0, 13.0, 14.0, 15.0, 16.0};
-    result.m_location = KeywordLocation::serializeObject();
+    result.m_location = KeywordLocation::serializationTestObject();
 
     return result;
 }
@@ -399,11 +399,11 @@ void VFPProdTable::check() {
 
     if (error_count > 0) {
         const auto& location = this->m_location;
-        OpmLog::warning(fmt::format("VFPPROD table {0} has {1} nonmonotonic points of BHP(THP)\n"
+        OpmLog::warning(fmt::format("VFPPROD table {0} has {1} non-monotonic points of BHP(THP)\n"
                                     "In {2} line {3}\n"
                                     "This may cause convergence issues due to switching between BHP and THP control.\n",
-                                    error_count,
                                     m_table_num,
+                                    error_count,
                                     location.filename,
                                     location.lineno));
     }
