@@ -17,121 +17,122 @@
   along with OPM.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include <fmt/format.h>
+#include <opm/input/eclipse/Schedule/Well/Well.hpp>
+
+#include <opm/io/eclipse/rst/well.hpp>
+#include <opm/output/eclipse/VectorItems/well.hpp>
+
+#include <opm/input/eclipse/EclipseState/Grid/EclipseGrid.hpp>
+#include <opm/input/eclipse/EclipseState/Runspec.hpp>
+#include <opm/input/eclipse/EclipseState/TracerConfig.hpp>
+
+#include <opm/input/eclipse/Schedule/ScheduleGrid.hpp>
+#include <opm/input/eclipse/Schedule/UDQ/UDQActive.hpp>
+
+#include <opm/common/utility/shmatch.hpp>
+
+#include <opm/input/eclipse/Parser/ParserKeywords/W.hpp>
 
 #include <opm/input/eclipse/Deck/DeckRecord.hpp>
 #include <opm/input/eclipse/Deck/DeckKeyword.hpp>
-#include <opm/io/eclipse/rst/well.hpp>
-#include <opm/output/eclipse/VectorItems/well.hpp>
-#include <opm/input/eclipse/Parser/ParserKeywords/W.hpp>
-#include <opm/input/eclipse/EclipseState/Runspec.hpp>
-#include <opm/input/eclipse/Schedule/Well/Well.hpp>
-#include <opm/input/eclipse/Schedule/UDQ/UDQActive.hpp>
-#include <opm/input/eclipse/Schedule/Well/WellInjectionProperties.hpp>
-#include <opm/input/eclipse/Schedule/Well/WellProductionProperties.hpp>
-#include <opm/input/eclipse/EclipseState/Grid/EclipseGrid.hpp>
-#include <opm/input/eclipse/Schedule/ScheduleGrid.hpp>
-#include <opm/input/eclipse/EclipseState/TracerConfig.hpp>
-#include <opm/common/utility/shmatch.hpp>
 
 #include "../MSW/Compsegs.hpp"
 
 #include <cmath>
+#include <cstddef>
+#include <memory>
+#include <optional>
 #include <ostream>
 #include <stdexcept>
+#include <string>
 #include <utility>
+#include <vector>
 
-namespace Opm {
+#include <fmt/format.h>
 
 namespace {
 
-    bool defaulted(const DeckRecord& rec, const std::string& s) {
-        const auto& item = rec.getItem( s );
-        if (item.defaultApplied(0))
-            return true;
-
-        if (item.get<int>(0) == 0)
-            return true;
-
-        return false;
+    bool defaulted(const Opm::DeckRecord& rec, const std::string& s)
+    {
+        const auto& item = rec.getItem(s);
+        return item.defaultApplied(0) || (item.get<int>(0) == 0);
     }
 
-
-    int limit(const DeckRecord& rec, const std::string&s , int shift) {
-        const auto& item = rec.getItem( s );
+    int limit(const Opm::DeckRecord& rec, const std::string& s, const int shift)
+    {
+        const auto& item = rec.getItem(s);
         return shift + item.get<int>(0);
     }
 
-    bool match_le(int value, const DeckRecord& rec, const std::string& s, int shift = 0) {
-        if (defaulted(rec,s))
-            return true;
-
-        return (value <= limit(rec,s,shift));
+    bool match_le(const int              value,
+                  const Opm::DeckRecord& rec,
+                  const std::string&     s,
+                  const int              shift = 0)
+    {
+        return defaulted(rec, s) || (value <= limit(rec, s, shift));
     }
 
-    bool match_ge(int value, const DeckRecord& rec, const std::string& s, int shift = 0) {
-        if (defaulted(rec,s))
-            return true;
-
-        return (value >= limit(rec,s,shift));
+    bool match_ge(const int              value,
+                  const Opm::DeckRecord& rec,
+                  const std::string&     s,
+                  const int              shift = 0)
+    {
+        return defaulted(rec, s) || (value >= limit(rec, s, shift));
     }
 
-
-    bool match_eq(int value, const DeckRecord& rec, const std::string& s, int shift = 0) {
-        if (defaulted(rec,s))
-            return true;
-
-        return (limit(rec,s,shift) == value);
+    bool match_eq(const int              value,
+                  const Opm::DeckRecord& rec,
+                  const std::string&     s,
+                  const int              shift = 0)
+    {
+        return defaulted(rec, s)|| (limit(rec, s, shift) == value);
     }
 
-}
-
-namespace {
-
-Connection::Order order_from_int(int int_value) {
-    switch(int_value) {
-    case 0:
-        return Connection::Order::TRACK;
-    case 1:
-        return Connection::Order::DEPTH;
-    case 2:
-        return Connection::Order::INPUT;
+Opm::Connection::Order order_from_int(const int int_value)
+{
+    switch (int_value) {
+    case 0: return Opm::Connection::Order::TRACK;
+    case 1: return Opm::Connection::Order::DEPTH;
+    case 2: return Opm::Connection::Order::INPUT;
     default:
-        throw std::invalid_argument(fmt::format("Invalid integer value: {} encountered when determining connection ordering", int_value));
+        throw std::invalid_argument {
+            fmt::format("Invalid integer value: {} encountered "
+                        "when determining connection ordering", int_value)
+        };
     }
 }
 
-Well::Status status_from_int(int int_value) {
-    using Value = RestartIO::Helpers::VectorItems::IWell::Value::Status;
+Opm::Well::Status status_from_int(const int int_value)
+{
+    using Value = Opm::RestartIO::Helpers::VectorItems::IWell::Value::Status;
 
     switch (int_value) {
-        case Value::Shut:
-            return Well::Status::SHUT;
-        case Value::Stop:
-            return Well::Status::STOP;
-        case Value::Open:
-            return Well::Status::OPEN;
-        case Value::Auto:
-            return Well::Status::AUTO;
+    case Value::Shut: return Opm::Well::Status::SHUT;
+    case Value::Stop: return Opm::Well::Status::STOP;
+    case Value::Open: return Opm::Well::Status::OPEN;
+    case Value::Auto: return Opm::Well::Status::AUTO;
     default:
-        throw std::logic_error(fmt::format("integer value: {} could not be converted to a valid state", int_value));
+        throw std::logic_error {
+            fmt::format("integer value: {} could not be "
+                        "converted to a valid well status.", int_value)
+        };
     }
 }
 
-Well::ProducerCMode producer_cmode_from_int(const int pmode)
+Opm::Well::ProducerCMode producer_cmode_from_int(const int pmode)
 {
     using CModeVal = ::Opm::RestartIO::Helpers::VectorItems::
         IWell::Value::WellCtrlMode;
 
     switch (pmode) {
-    case CModeVal::Group:    return Well::ProducerCMode::GRUP;
-    case CModeVal::OilRate:  return Well::ProducerCMode::ORAT;
-    case CModeVal::WatRate:  return Well::ProducerCMode::WRAT;
-    case CModeVal::GasRate:  return Well::ProducerCMode::GRAT;
-    case CModeVal::LiqRate:  return Well::ProducerCMode::LRAT;
-    case CModeVal::ResVRate: return Well::ProducerCMode::RESV;
-    case CModeVal::THP:      return Well::ProducerCMode::THP;
-    case CModeVal::BHP:      return Well::ProducerCMode::BHP;
+    case CModeVal::Group:    return Opm::Well::ProducerCMode::GRUP;
+    case CModeVal::OilRate:  return Opm::Well::ProducerCMode::ORAT;
+    case CModeVal::WatRate:  return Opm::Well::ProducerCMode::WRAT;
+    case CModeVal::GasRate:  return Opm::Well::ProducerCMode::GRAT;
+    case CModeVal::LiqRate:  return Opm::Well::ProducerCMode::LRAT;
+    case CModeVal::ResVRate: return Opm::Well::ProducerCMode::RESV;
+    case CModeVal::THP:      return Opm::Well::ProducerCMode::THP;
+    case CModeVal::BHP:      return Opm::Well::ProducerCMode::BHP;
     }
 
     throw std::invalid_argument {
@@ -139,20 +140,20 @@ Well::ProducerCMode producer_cmode_from_int(const int pmode)
     };
 }
 
-Well::InjectorCMode injector_cmode_from_int(const int imode)
+Opm::Well::InjectorCMode injector_cmode_from_int(const int imode)
 {
     using CModeVal = ::Opm::RestartIO::Helpers::VectorItems::
         IWell::Value::WellCtrlMode;
 
     switch (imode) {
-    case CModeVal::Group:    return Well::InjectorCMode::GRUP;
+    case CModeVal::Group:    return Opm::Well::InjectorCMode::GRUP;
     case CModeVal::OilRate:  [[fallthrough]];
     case CModeVal::WatRate:  [[fallthrough]];
     case CModeVal::GasRate:  [[fallthrough]];
-    case CModeVal::LiqRate:  return Well::InjectorCMode::RATE;
-    case CModeVal::ResVRate: return Well::InjectorCMode::RESV;
-    case CModeVal::THP:      return Well::InjectorCMode::THP;
-    case CModeVal::BHP:      return Well::InjectorCMode::BHP;
+    case CModeVal::LiqRate:  return Opm::Well::InjectorCMode::RATE;
+    case CModeVal::ResVRate: return Opm::Well::InjectorCMode::RESV;
+    case CModeVal::THP:      return Opm::Well::InjectorCMode::THP;
+    case CModeVal::BHP:      return Opm::Well::InjectorCMode::BHP;
     }
 
     throw std::invalid_argument {
@@ -160,13 +161,107 @@ Well::InjectorCMode injector_cmode_from_int(const int imode)
     };
 }
 
-constexpr Well::ProducerCMode def_whistctl_cmode = Well::ProducerCMode::CMODE_UNDEFINED;
-const static Well::WellGuideRate def_guide_rate = {true, -1, Well::GuideRateTarget::UNDEFINED, ParserKeywords::WGRUPCON::SCALING_FACTOR::defaultValue};
+bool haveEconomicLimits(const Opm::RestartIO::RstWell& rst_well)
+{
+    namespace Limits = Opm::RestartIO::Helpers::VectorItems::
+        IWell::Value::EconLimit;
+
+    auto is_finite = [](const float x)
+    {
+        const auto infty = 1.0e+20f;
+
+        return std::abs(x) < infty;
+    };
+
+    auto is_nonzero = [](const float x)
+    {
+        return std::abs(x) > 0.0f;
+    };
+
+    return (rst_well.econ_workover_procedure   != Limits::WOProcedure::None)
+        || (rst_well.econ_workover_procedure_2 != Limits::WOProcedure::None)
+        || (rst_well.econ_limit_end_run        == Limits::EndRun::Yes)
+        || (rst_well.econ_limit_quantity       != Limits::Rate)
+        || is_nonzero(rst_well.econ_limit_min_oil)
+        || is_nonzero(rst_well.econ_limit_min_gas)
+        || is_nonzero(rst_well.econ_limit_min_liq)
+        || is_finite (rst_well.econ_limit_max_wct)
+        || is_finite (rst_well.econ_limit_max_gor)
+        || is_finite (rst_well.econ_limit_max_wgr)
+        || is_finite (rst_well.econ_limit_max_wct_2);
+}
+
+std::shared_ptr<Opm::WellEconProductionLimits>
+economicLimits(const Opm::RestartIO::RstWell& rst_well)
+{
+    return haveEconomicLimits(rst_well)
+        ? std::make_shared<Opm::WellEconProductionLimits>(rst_well)
+        : std::make_shared<Opm::WellEconProductionLimits>();
+}
+
+Opm::Well::GuideRateTarget guideRatePhase(const int gr_phase)
+{
+    namespace WGrupCon = Opm::RestartIO::Helpers::VectorItems::
+        IWell::Value::WGrupCon;
+
+    using Target = Opm::Well::GuideRateTarget;
+
+    switch (gr_phase) {
+    case WGrupCon::GRPhase::Defaulted:            return Target::UNDEFINED;
+    case WGrupCon::GRPhase::Oil:                  return Target::OIL;
+    case WGrupCon::GRPhase::Water:                return Target::WAT;
+    case WGrupCon::GRPhase::Gas:                  return Target::GAS;
+    case WGrupCon::GRPhase::Liquid:               return Target::LIQ;
+    case WGrupCon::GRPhase::SurfaceInjectionRate: return Target::RAT;
+    case WGrupCon::GRPhase::ReservoirVolumeRate:  return Target::RES;
+    }
+
+    throw std::invalid_argument {
+        fmt::format("Cannot convert integer value {} "
+                    "to guiderate phase target", gr_phase)
+    };
+}
+
+bool isGroupControllable(const int gr_controllable_flag)
+{
+    return gr_controllable_flag != Opm::RestartIO::Helpers::VectorItems::
+        IWell::Value::WGrupCon::Controllable::No;
+}
+
+double guideRateValue(const float gr_value)
+{
+    return (std::abs(gr_value) < 1.0e+20f)
+        ? gr_value
+        : Opm::ParserKeywords::WGRUPCON::GUIDE_RATE::defaultValue;
+}
+
+Opm::Well::WellGuideRate guideRate(const Opm::RestartIO::RstWell& rst_well)
+{
+    return {
+        isGroupControllable(rst_well.group_controllable_flag),
+        guideRateValue(rst_well.grupcon_gr_value),
+        guideRatePhase(rst_well.grupcon_gr_phase),
+        rst_well.grupcon_gr_scaling
+    };
+}
+
+std::shared_ptr<Opm::WVFPEXP>
+explicitTHPOptions(const Opm::RestartIO::RstWell& rst_well)
+{
+    auto options = std::make_shared<Opm::WVFPEXP>();
+    options->update(rst_well);
+
+    return options;
+}
+
+constexpr Opm::Well::ProducerCMode def_whistctl_cmode = Opm::Well::ProducerCMode::CMODE_UNDEFINED;
 const static bool def_automatic_shutin = true;
 constexpr double def_solvent_fraction = 0;
 
 }
 
+namespace Opm {
+
 Well::Well(const RestartIO::RstWell& rst_well,
            int report_step,
            const TracerConfig& tracer_config,
@@ -185,11 +280,11 @@ Well::Well(const RestartIO::RstWell& rst_well,
     unit_system(unit_system_arg),
     udq_undefined(udq_undefined_arg),
     wtype(rst_well.wtype),
-    guide_rate(def_guide_rate),
+    guide_rate(guideRate(rst_well)),
     efficiency_factor(rst_well.efficiency_factor),
     solvent_fraction(def_solvent_fraction),
     prediction_mode(rst_well.hist_requested_control == 0),
-    econ_limits(std::make_shared<WellEconProductionLimits>()),
+    econ_limits(economicLimits(rst_well)),
     foam_properties(std::make_shared<WellFoamProperties>()),
     polymer_properties(std::make_shared<WellPolymerProperties>()),
     micp_properties(std::make_shared<WellMICPProperties>()),
@@ -198,7 +293,7 @@ Well::Well(const RestartIO::RstWell& rst_well,
     connections(std::make_shared<WellConnections>(order_from_int(rst_well.completion_ordering), headI, headJ)),
     production(std::make_shared<WellProductionProperties>(unit_system_arg, wname)),
     injection(std::make_shared<WellInjectionProperties>(unit_system_arg, wname)),
-    wvfpexp(std::make_shared<WVFPEXP>()),
+    wvfpexp(explicitTHPOptions(rst_well)),
     status(status_from_int(rst_well.well_status))
 {
     if (this->wtype.producer()) {
@@ -383,7 +478,7 @@ Well::Well(const std::string& wname_arg,
     this->updateProduction(p);
 }
 
-Well Well::serializeObject()
+Well Well::serializationTestObject()
 {
     Well result;
     result.wname = "test1";
@@ -393,7 +488,7 @@ Well Well::serializeObject()
     result.headI = 3;
     result.headJ = 4;
     result.ref_depth = 5;
-    result.unit_system = UnitSystem::serializeObject();
+    result.unit_system = UnitSystem::serializationTestObject();
     result.udq_undefined = 6.0;
     result.status = Status::AUTO;
     result.drainage_radius = 7.0;
@@ -402,21 +497,21 @@ Well Well::serializeObject()
     result.pvt_table = 77;
     result.gas_inflow = GasInflowEquation::GPP;
     result.wtype = WellType(Phase::WATER);
-    result.guide_rate = WellGuideRate::serializeObject();
+    result.guide_rate = WellGuideRate::serializationTestObject();
     result.efficiency_factor = 8.0;
     result.solvent_fraction = 9.0;
     result.prediction_mode = false;
-    result.econ_limits = std::make_shared<Opm::WellEconProductionLimits>(Opm::WellEconProductionLimits::serializeObject());
-    result.foam_properties = std::make_shared<WellFoamProperties>(WellFoamProperties::serializeObject());
-    result.polymer_properties =  std::make_shared<WellPolymerProperties>(WellPolymerProperties::serializeObject());
-    result.micp_properties =  std::make_shared<WellMICPProperties>(WellMICPProperties::serializeObject());
-    result.brine_properties = std::make_shared<WellBrineProperties>(WellBrineProperties::serializeObject());
-    result.tracer_properties = std::make_shared<WellTracerProperties>(WellTracerProperties::serializeObject());
-    result.connections = std::make_shared<WellConnections>(WellConnections::serializeObject());
-    result.production = std::make_shared<Well::WellProductionProperties>(Well::WellProductionProperties::serializeObject());
-    result.injection = std::make_shared<Well::WellInjectionProperties>(Well::WellInjectionProperties::serializeObject());
-    result.segments = std::make_shared<WellSegments>(WellSegments::serializeObject());
-    result.wvfpexp = std::make_shared<WVFPEXP>(WVFPEXP::serializeObject());
+    result.econ_limits = std::make_shared<Opm::WellEconProductionLimits>(Opm::WellEconProductionLimits::serializationTestObject());
+    result.foam_properties = std::make_shared<WellFoamProperties>(WellFoamProperties::serializationTestObject());
+    result.polymer_properties =  std::make_shared<WellPolymerProperties>(WellPolymerProperties::serializationTestObject());
+    result.micp_properties =  std::make_shared<WellMICPProperties>(WellMICPProperties::serializationTestObject());
+    result.brine_properties = std::make_shared<WellBrineProperties>(WellBrineProperties::serializationTestObject());
+    result.tracer_properties = std::make_shared<WellTracerProperties>(WellTracerProperties::serializationTestObject());
+    result.connections = std::make_shared<WellConnections>(WellConnections::serializationTestObject());
+    result.production = std::make_shared<Well::WellProductionProperties>(Well::WellProductionProperties::serializationTestObject());
+    result.injection = std::make_shared<Well::WellInjectionProperties>(Well::WellInjectionProperties::serializationTestObject());
+    result.segments = std::make_shared<WellSegments>(WellSegments::serializationTestObject());
+    result.wvfpexp = std::make_shared<WVFPEXP>(WVFPEXP::serializationTestObject());
     result.m_pavg = PAvg();
 
     return result;
@@ -804,26 +899,38 @@ double Well::getGuideRate() const {
     return this->guide_rate.guide_rate;
 }
 
-Well::GuideRateTarget Well::getGuideRatePhase() const {
-    if (this->wtype.producer())
-        return this->guide_rate.guide_phase;
+Well::GuideRateTarget Well::getGuideRatePhase() const
+{
+    const auto target = this->getRawGuideRatePhase();
 
-    if (this->guide_rate.guide_phase == GuideRateTarget::RAT) {
-        switch (this->getPreferredPhase()) {
-        case Phase::OIL:
-            return GuideRateTarget::OIL;
-        case Phase::GAS:
-            return GuideRateTarget::GAS;
-        case Phase::WATER:
-            return GuideRateTarget::WAT;
-        default:
-            throw std::logic_error("Can not convert well preferred phase to GuideRate target phase");
-        }
+    if (this->isInjector() && (target == GuideRateTarget::RAT)) {
+        return this->preferredPhaseAsGuideRatePhase();
     }
 
+    return target;
+}
+
+Well::GuideRateTarget Well::getRawGuideRatePhase() const
+{
     return this->guide_rate.guide_phase;
 }
 
+Well::GuideRateTarget Well::preferredPhaseAsGuideRatePhase() const
+{
+    switch (this->getPreferredPhase()) {
+    case Phase::OIL:   return GuideRateTarget::OIL;
+    case Phase::GAS:   return GuideRateTarget::GAS;
+    case Phase::WATER: return GuideRateTarget::WAT;
+
+    default:
+        throw std::logic_error {
+            fmt::format("Unable to convert well preferred "
+                        "phase {} to GuideRate target phase",
+                        static_cast<int>(this->getPreferredPhase()))
+        };
+    }
+}
+
 double Well::getGuideRateScalingFactor() const {
     return this->guide_rate.scale_factor;
 }
@@ -1143,6 +1250,18 @@ bool Well::handleWPIMULT(const DeckRecord& record) {
 }
 
 
+bool Opm::Well::applyGlobalWPIMULT(const double scaling_factor)
+{
+    auto new_connections = std::make_shared<WellConnections>(this->connections->ordering(), this->headI, this->headJ);
+    for (auto c : *this->connections) {
+        c.scaleWellPi(scaling_factor);
+        new_connections->add(c);
+    }
+
+    return this->updateConnections(std::move(new_connections), false);
+}
+
+
 void Well::updateSegments(std::shared_ptr<WellSegments> segments_arg) {
     this->segments = std::move(segments_arg);
     this->updateRefDepth( this->segments->depthTopSegment() );
@@ -1696,8 +1815,7 @@ PAvgCalculator Well::pavg_calculator(const EclipseGrid& grid, const std::vector<
     return PAvgCalculator(this->name(), this->getWPaveRefDepth(), grid, porv, this->getConnections(), this->m_pavg);
 }
 
-
-}
+} // namespace Opm
 
 int Opm::Well::eclipseControlMode(const Well::InjectorCMode imode,
                              const InjectorType        itype)
