@@ -89,6 +89,8 @@ namespace {
         0.0,
         0.0,
         0.0,
+        0.0,
+        0.0,
     };
 
     static const double to_metric[] = {
@@ -110,6 +112,7 @@ namespace {
         1 / ( Metric::GasSurfaceVolume / Metric::Time ),
         1 / ( Metric::ReservoirVolume / Metric::Time ),
         1 / ( Metric::GeomVolume / Metric::Time ),
+        1 / Metric::Length,     // Pipeflow velocity: Length / Second.
         1 / Metric::Transmissibility,
         1 / (Metric::Permeability * Metric::Length),
         1 / Metric::Mass,
@@ -131,6 +134,7 @@ namespace {
         1 / (Metric::Pressure / Metric::Density / Opm::unit::square(Metric::GeomVolume / Metric::Time)),
         1 / Metric::PolymerDensity,
         1 / Metric::Salinity,
+        1 / (1 / Metric::Time),
     };
 
     static const double from_metric[] = {
@@ -152,6 +156,7 @@ namespace {
         Metric::GasSurfaceVolume / Metric::Time,
         Metric::ReservoirVolume / Metric::Time,
         Metric::GeomVolume / Metric::Time,
+        Metric::Length,         // Pipeflow velocity: Length / Second.
         Metric::Transmissibility,
         Metric::Permeability * Metric::Length,
         Metric::Mass,
@@ -173,6 +178,7 @@ namespace {
         Metric::Pressure / Metric::Density / Opm::unit::square(Metric::GeomVolume / Metric::Time),
         Metric::PolymerDensity,
         Metric::Salinity,
+        1 / Metric::Time,
     };
 
     static constexpr const char* metric_names[static_cast<int>(UnitSystem::measure::_count)] = {
@@ -194,6 +200,7 @@ namespace {
         "SM3/DAY",
         "RM3/DAY",
         "SM3/DAY",  // Should possibly be RM3/DAY
+        "M/SEC",    // pipeflow velocity
         "CPR3/DAY/BARS",
         "MDM",
         "KG",
@@ -215,6 +222,7 @@ namespace {
         "BARS/(KG/SM3)/(RM3/DAY)2", /* AICD strength parameter */
         "KG / SM3", /*polymer density */
         "KG / SM3", /*salinity */
+        "SM3/SM3/DAY",
     };
 
     static_assert(numElems(from_metric_offset) == static_cast<std::size_t>(UnitSystem::measure::_count),
@@ -277,6 +285,8 @@ namespace {
         0.0,
         0.0,
         0.0,
+        0.0,
+        0.0,
     };
 
     static const double to_field[] = {
@@ -298,6 +308,7 @@ namespace {
         1 / ( Field::GasSurfaceVolume / Field::Time ),
         1 / ( Field::ReservoirVolume / Field::Time ),
         1 / ( Field::GeomVolume / Field::Time ),
+        1 / Field::Length,     // Pipeflow velocity: Length / Second.
         1 / Field::Transmissibility,
         1 / (Field::Permeability * Field::Length),
         1 / Field::Mass,
@@ -319,6 +330,7 @@ namespace {
         1 / (Field::Pressure / Field::Density / Opm::unit::square(Field::GeomVolume / Field::Time)),
         1 / Field::PolymerDensity,
         1 / Field::Salinity,
+        1 / (Field::GasSurfaceVolume / Field::LiquidSurfaceVolume / Field::Time),
     };
 
     static const double from_field[] = {
@@ -340,6 +352,7 @@ namespace {
          Field::GasSurfaceVolume / Field::Time,
          Field::ReservoirVolume / Field::Time,
          Field::GeomVolume / Field::Time,
+         Field::Length,         // Pipeflow velocity: Length / Second.
          Field::Transmissibility,
          Field::Permeability * Field::Length,
          Field::Mass,
@@ -361,6 +374,7 @@ namespace {
          Field::Pressure / Field::Density / Opm::unit::square(Field::GeomVolume / Field::Time),
          Field::PolymerDensity,
          Field::Salinity,
+         Field::GasSurfaceVolume / Field::LiquidSurfaceVolume / Field::Time,
     };
 
     static constexpr const char* field_names[static_cast<int>(UnitSystem::measure::_count)] = {
@@ -382,6 +396,7 @@ namespace {
         "MSCF/DAY",
         "RB/DAY",
         "FT3/DAY",   // Should possibly be RFT3/DAY
+        "FT/SEC",    // Pipeflow velocity
         "CPRB/DAY/PSI",
         "MDFT",
         "LB",
@@ -403,6 +418,7 @@ namespace {
         "PSI/(LB/FT3)/(RFT3/DAY)2", /* AICD strength parameter */
         "LB/STB", /*polymer density */
         "LB/STB", /*salinity */
+        "MSCF/STB/DAY",
     };
 
     static_assert(numElems(from_field_offset) == static_cast<std::size_t>(UnitSystem::measure::_count),
@@ -465,6 +481,8 @@ namespace {
         0.0,
         0.0,
         0.0,
+        0.0,
+        0.0,
     };
 
     static const double to_lab[] = {
@@ -486,6 +504,7 @@ namespace {
         1 / ( Lab::GasSurfaceVolume / Lab::Time ),
         1 / ( Lab::ReservoirVolume / Lab::Time ),
         1 / ( Lab::GeomVolume / Lab::Time ),
+        1 / Lab::Length,        // Pipeflow velocity: Length / Second
         1 / Lab::Transmissibility,
         1 / (Lab::Permeability * Lab::Length),
         1 / Lab::Mass,
@@ -507,6 +526,7 @@ namespace {
         1 / (Lab::Pressure / Lab::Density / Opm::unit::square(Lab::GeomVolume / Lab::Time)),
         1 / Lab::PolymerDensity,
         1 / Lab::Salinity,
+        1 / (Lab::GasDissolutionFactor / Lab::Time),
     };
 
     static const double from_lab[] = {
@@ -528,6 +548,7 @@ namespace {
         Lab::GasSurfaceVolume / Lab::Time,
         Lab::ReservoirVolume / Lab::Time,
         Lab::GeomVolume / Lab::Time,
+        Lab::Length,            // Pipeflow velocity: Length / Second
         Lab::Transmissibility,
         Lab::Permeability * Lab::Length,
         Lab::Mass,
@@ -549,6 +570,7 @@ namespace {
         Lab::Pressure / Lab::Density / Opm::unit::square(Lab::GeomVolume / Lab::Time),
         Lab::PolymerDensity,
         Lab::Salinity,
+        Lab::GasDissolutionFactor / Lab::Time,
     };
 
     static constexpr const char* lab_names[static_cast<int>(UnitSystem::measure::_count)] = {
@@ -570,6 +592,7 @@ namespace {
         "SCC/HR",
         "RCC/HR",
         "SCC/HR",   // Should possibly be RCC/HR
+        "CM/SEC",   // Pipeflow velocity
         "CPRCC/HR/ATM",
         "MDCC",
         "G",
@@ -591,6 +614,7 @@ namespace {
         "ATM/(G/SCC)/(RCC/H)2", /* AICD strength parameter */
         "G/SCC", /*polymer density */
         "G/SCC", /*salinity */
+        "SCC/SCC/HR",
     };
 
     static_assert(numElems(from_lab_offset) == static_cast<std::size_t>(UnitSystem::measure::_count),
@@ -653,6 +677,8 @@ namespace {
         0.0,
         0.0,
         0.0,
+        0.0,
+        0.0,
     };
 
     static const double to_pvt_m[] = {
@@ -674,6 +700,7 @@ namespace {
         1 / ( PVT_M::GasSurfaceVolume / PVT_M::Time ),
         1 / ( PVT_M::ReservoirVolume / PVT_M::Time ),
         1 / ( PVT_M::GeomVolume / PVT_M::Time ),
+        1 / PVT_M::Length,      // Pipeflow velocity: Length / Second
         1 / PVT_M::Transmissibility,
         1 / (PVT_M::Permeability * PVT_M::Length),
         1 / PVT_M::Mass,
@@ -695,6 +722,7 @@ namespace {
         1 / (PVT_M::Pressure / PVT_M::Density  / Opm::unit::square(PVT_M::GeomVolume / PVT_M::Time)),
         1 / PVT_M::PolymerDensity,
         1 / PVT_M::Salinity,
+        1 / (PVT_M::GasSurfaceVolume / PVT_M::LiquidSurfaceVolume /PVT_M::Time),
     };
 
     static const double from_pvt_m[] = {
@@ -716,6 +744,7 @@ namespace {
         PVT_M::GasSurfaceVolume / PVT_M::Time,
         PVT_M::ReservoirVolume / PVT_M::Time,
         PVT_M::GeomVolume / PVT_M::Time,
+        PVT_M::Length,          // Pipeflow velocity: Length / Second
         PVT_M::Transmissibility,
         PVT_M::Permeability * PVT_M::Length,
         PVT_M::Mass,
@@ -737,6 +766,7 @@ namespace {
         PVT_M::Pressure / PVT_M::Density  / Opm::unit::square(PVT_M::GeomVolume / PVT_M::Time),
         PVT_M::PolymerDensity,
         PVT_M::Salinity,
+        PVT_M::GasSurfaceVolume / PVT_M::LiquidSurfaceVolume /PVT_M::Time,
     };
 
     static constexpr const char* pvt_m_names[static_cast<int>(UnitSystem::measure::_count)] = {
@@ -758,6 +788,7 @@ namespace {
         "SM3/DAY",
         "RM3/DAY",
         "SM3/DAY",     // Should possibly be SM3/DAY
+        "M/SEC",       // Pipeflow velocity
         "CPR3/DAY/ATM",
         "MDM",
         "KG",
@@ -779,6 +810,7 @@ namespace {
         "ATM/(KG/SM3)/(RM3/DAY)2", /* AICD strength parameter */
         "KG/SM3", /*polymer density */
         "KG/SM3", /*salinity */
+        "SM3/SM3/DAY",
     };
 
     static_assert(numElems(from_pvt_m_offset) == static_cast<std::size_t>(UnitSystem::measure::_count),
@@ -841,6 +873,8 @@ namespace {
         0.0,
         0.0,
         0.0,
+        0.0,
+        0.0,
     };
 
     static const double to_input[] = {
@@ -883,6 +917,8 @@ namespace {
         1,
         1,
         1,
+        1,
+        1,
     };
 
     static const double from_input[] = {
@@ -925,6 +961,8 @@ namespace {
         1,
         1,
         1,
+        1,
+        1,
     };
 
     static constexpr const char* input_names[static_cast<int>(UnitSystem::measure::_count)] = {
@@ -946,6 +984,7 @@ namespace {
         "SM3/DAY",
         "RM3/DAY",
         "SM3/DAY",
+        "M/SEC",                // Pipeflow velocity.
         "CPR3/DAY/BARS",
         "MDM",
         "KG",
@@ -967,6 +1006,7 @@ namespace {
         "BARS/(KG/SM3)/(RM3/DAY)2", /* AICD strength parameter */
         "KG/SM3", /*polymer density */
         "KG/SM3", /*salinity */
+        "SM3/SM3/DAY",
     };
 
     static_assert(numElems(from_input_offset) == static_cast<std::size_t>(UnitSystem::measure::_count),
@@ -994,7 +1034,7 @@ namespace {
         init();
     }
 
-    UnitSystem UnitSystem::serializeObject()
+    UnitSystem UnitSystem::serializationTestObject()
     {
         return UnitSystem(UnitType::UNIT_TYPE_METRIC);
     }
@@ -1308,11 +1348,11 @@ namespace {
     }
 
 
-    Dimension UnitSystem::uda_dim(UDAControl control) const {
+    Dimension UnitSystem::uda_dim(const UDAControl control) const {
         switch (control) {
-        case UDAControl::WCONPROD_ORAT:
-        case UDAControl::WCONPROD_WRAT:
-        case UDAControl::WCONPROD_LRAT:
+        case UDAControl::WCONPROD_ORAT:  case UDAControl::WELTARG_ORAT:
+        case UDAControl::WCONPROD_WRAT:  case UDAControl::WELTARG_WRAT:
+        case UDAControl::WCONPROD_LRAT:  case UDAControl::WELTARG_LRAT:
         case UDAControl::GCONPROD_OIL_TARGET:
         case UDAControl::GCONPROD_WATER_TARGET:
         case UDAControl::GCONPROD_LIQUID_TARGET:
@@ -1320,16 +1360,17 @@ namespace {
 
         case UDAControl::WCONINJE_RESV:
         case UDAControl::WCONPROD_RESV:
+        case UDAControl::WELTARG_RESV:
         case UDAControl::GCONINJE_RESV_MAX_RATE:
             return this->getDimension(UnitSystem::measure::geometric_volume_rate);
 
         case UDAControl::WCONPROD_GRAT:
+        case UDAControl::WELTARG_GRAT:
             return this->getDimension(UnitSystem::measure::gas_surface_rate);
 
-        case UDAControl::WCONPROD_BHP:
-        case UDAControl::WCONPROD_THP:
-        case UDAControl::WCONINJE_THP:
-        case UDAControl::WCONINJE_BHP:
+        case UDAControl::WCONPROD_BHP:  case UDAControl::WCONPROD_THP:
+        case UDAControl::WCONINJE_BHP:  case UDAControl::WCONINJE_THP:
+        case UDAControl::WELTARG_BHP:   case UDAControl::WELTARG_THP:
             return this->getDimension(UnitSystem::measure::pressure);
 
         case UDAControl::GCONINJE_TARGET_REINJ_FRACTION:
@@ -1339,7 +1380,9 @@ namespace {
             return this->getDimension(UnitSystem::measure::identity);
 
         default:
-            throw std::logic_error("No dimension");
+            throw std::logic_error {
+                "No dimension for UDA control '" + UDQ::controlName(control) + '\''
+            };
         }
     }
 
@@ -1522,4 +1565,4 @@ namespace {
         };
     }
 
-}
+} // namespace Opm
