@@ -25,7 +25,6 @@
 
 #include <opm/output/eclipse/VectorItems/aquifer.hpp>
 
-#include <opm/input/eclipse/EclipseState/EclipseState.hpp>
 #include <opm/input/eclipse/EclipseState/Aquifer/Aquancon.hpp>
 #include <opm/input/eclipse/EclipseState/Aquifer/AquiferConfig.hpp>
 #include <opm/input/eclipse/EclipseState/Aquifer/AquiferCT.hpp>
@@ -102,6 +101,10 @@ namespace {
                                        ConnectionCallBack&&      connectionOp)
     {
         for (const auto& [aquiferID, connections] : aqConfig.connections().data()) {
+            if ( !aqConfig.hasAnalyticalAquifer(int(aquiferID)) ) {
+                continue;
+            }
+
             const auto tot_influx =
                 std::accumulate(connections.begin(), connections.end(), 0.0,
                     [](const double t, const Opm::Aquancon::AquancCell& connection) -> double
@@ -216,6 +219,11 @@ namespace {
             using FDValue = VI::IAnalyticAquiferConn::Value::FaceDirection;
 
             switch (faceDir) {
+            case Opm::FaceDir::DirEnum::Unknown:
+                // Invalid.  Must not write aquifer data if we get here.
+                // Fall through to 'throw' below.
+                break;
+
             case Opm::FaceDir::DirEnum::XMinus: return FDValue::IMinus;
             case Opm::FaceDir::DirEnum::XPlus:  return FDValue::IPlus;
             case Opm::FaceDir::DirEnum::YMinus: return FDValue::JMinus;
