@@ -85,7 +85,7 @@ AQUTAB
 
 SOLUTION
 AQUCT
-   1 2000.0 1.5 100 .3 3.0e-5 330 10 360.0 1 2 /
+   1 2000.0 1.5 100 .3 3.0e-5 330 10 360.0 1 2 1* 20/
 /
 )");
 }
@@ -149,6 +149,7 @@ BOOST_AUTO_TEST_CASE(AquiferCTTest)
             BOOST_CHECK_EQUAL(it.inftableID, 2);
             BOOST_CHECK_MESSAGE(it.initial_pressure.has_value(), "Initial pressure must be defined in CT aquifer");
             BOOST_CHECK_CLOSE(it.initial_pressure.value(), 1.5e5, 1e-6);
+            BOOST_CHECK_CLOSE(it.initial_temperature.value(), 20 + 273.15, 1e-6);
         }
     }
 
@@ -159,6 +160,7 @@ BOOST_AUTO_TEST_CASE(AquiferCTTest)
             BOOST_CHECK_CLOSE(it.porosity, 0.3, 1.0e-8);
             BOOST_CHECK_EQUAL(it.inftableID, 2);
             BOOST_CHECK_MESSAGE(! it.initial_pressure.has_value(), "Initial pressure must NOT be defined in CT aquifer");
+            BOOST_CHECK_MESSAGE(! it.initial_temperature.has_value(), "Initial temperature is NOT supposed to be defined in these aquifers");
         }
 
         auto data = aquiferct.data();
@@ -277,14 +279,14 @@ BOOST_AUTO_TEST_CASE(AquanconTest_DEFAULT_INFLUX)
     const auto& grid = makeGrid();
     Aquancon aqcon(grid, deck1);
 
-    const auto& cells_aq1 = aqcon[1];
+    const auto& cells_aq1 = aqcon.getConnections(1);
     /*
       The cells I = 0..2 are connected to aquifer 1; cell I==0 is inactive and
       not counted here ==> a total of 2 cells are connected to aquifer 1.
     */
     BOOST_CHECK_EQUAL(cells_aq1.size(), 2U);
 
-    const auto& cells_aq2 = aqcon[2];
+    const auto& cells_aq2 = aqcon.getConnections(2);
     BOOST_CHECK_EQUAL(cells_aq2.size(), 1U);
     BOOST_CHECK(aqcon.active());
 
@@ -352,8 +354,8 @@ BOOST_AUTO_TEST_CASE(AquanconTest_ALLOW_AQUIFER_INSIDE_OR_NOT)
     const Aquancon aq2(data);
 
     BOOST_CHECK(aqucon == aq2);
-    auto cells1 = aqucon[1];
-    auto cells2 = aqucon[2];
+    const auto cells1 = aqucon.getConnections(1);
+    const auto cells2 = aqucon.getConnections(2);
     BOOST_CHECK_EQUAL(cells1.size() , 2U);
     BOOST_CHECK_EQUAL(cells2.size() , 1U);
 }
@@ -395,7 +397,7 @@ AQUTAB
 
 SOLUTION
 AQUFETP
-1  70000.0  4.0e3 2.0e9 1.0e-5	500 1 0	0 /
+1  70000.0  4.0e3 2.0e9 1.0e-5	500 1 0	20 /
 /
 )");
 }
@@ -471,7 +473,7 @@ AQUTAB
 
 SOLUTION
 AQUFETP
-1  70000.0  1* 2.0e9 1.0e-5	500 1 0	0 /
+1  70000.0  1* 2.0e9 1.0e-5	500 1 0	1* /
 /
 )");
 }
@@ -492,6 +494,7 @@ BOOST_AUTO_TEST_CASE(AquifetpTest)
         BOOST_CHECK_CLOSE(it.initial_watvolume, 2.0e9, 1.0e-8);
         BOOST_CHECK_CLOSE(it.prod_index, 500/86400e5, 1.0e-8);
         BOOST_CHECK_MESSAGE(it.initial_pressure.has_value(), "Fetkovich aquifer must have initial pressure value");
+        BOOST_CHECK_CLOSE(it.initial_temperature.value(), 20 + 273.15, 1.0e-8);
     }
     const auto& data = aquifetp.data();
     Aquifetp aq2(data);
@@ -508,6 +511,7 @@ BOOST_AUTO_TEST_CASE(AquifetpTest)
         BOOST_CHECK_CLOSE(it.initial_watvolume, 2.0e9, 1.0e-8);
         BOOST_CHECK_CLOSE(it.prod_index, 500/86400e5, 1.0e-8);
         BOOST_CHECK_MESSAGE(!it.initial_pressure.has_value(), "Fetkovich aquifer mut NOT have initial pressure value when defaulted");
+        BOOST_CHECK_MESSAGE(!it.initial_temperature.has_value(), "Initial temperature is NOT supposed to be defined in these aquifers");
     }
 }
 
@@ -639,6 +643,14 @@ END
     BOOST_CHECK_MESSAGE(  aquConfig.hasAquifer(7), "Configuration object must have Aquifer ID 7");
     BOOST_CHECK_MESSAGE(! aquConfig.hasAquifer(8), "Configuration object must NOT have Aquifer ID 8");
 
+    BOOST_CHECK_MESSAGE(  aquConfig.hasAnalyticalAquifer(1), "Configuration object must have Analytical Aquifer ID 1");
+    BOOST_CHECK_MESSAGE(  aquConfig.hasAnalyticalAquifer(2), "Configuration object must have Analytical Aquifer ID 2");
+    BOOST_CHECK_MESSAGE(  aquConfig.hasAnalyticalAquifer(3), "Configuration object must have Analytical Aquifer ID 3");
+    BOOST_CHECK_MESSAGE(  !aquConfig.hasAnalyticalAquifer(4), "Configuration object must NOT have Analytical Aquifer ID 4");
+    BOOST_CHECK_MESSAGE(  !aquConfig.hasAnalyticalAquifer(5), "Configuration object must NOT have Analytical Aquifer ID 5");
+    BOOST_CHECK_MESSAGE(  !aquConfig.hasAnalyticalAquifer(6), "Configuration object must NOT have Analytical Aquifer ID 6");
+    BOOST_CHECK_MESSAGE(  !aquConfig.hasAnalyticalAquifer(7), "Configuration object must NOT have Analytical Aquifer ID 7");
+
     {
         const auto expect = std::vector<int>{ 1, 2, 3 };
         const auto analytic = analyticAquiferIDs(aquConfig);
@@ -912,7 +924,7 @@ AQUANCON
 )";
        const auto& [aquconn, _] = load_aquifer(aq);
        (void)_;
-       auto cell1 = aquconn[1][0];
+       auto cell1 = aquconn.getConnections(1)[0];
        BOOST_CHECK_EQUAL(cell1.influx_coeff, 2.0);
    }
 
@@ -925,7 +937,7 @@ AQUANCON
 )";
        const auto& [aquconn, grid] = load_aquifer(aq);
        const auto& dims = grid.getCellDims(0,14,2);
-       auto cell1 = aquconn[1][0];
+       auto cell1 = aquconn.getConnections(1)[0];
        BOOST_CHECK_EQUAL(cell1.influx_coeff, 2.0 * dims[0]*dims[2]);
    }
 
@@ -937,7 +949,7 @@ AQUANCON
 )";
        const auto& [aquconn, grid] = load_aquifer(aq);
        const auto& dims = grid.getCellDims(0,14,2);
-       auto cell1 = aquconn[1][0];
+       auto cell1 = aquconn.getConnections(1)[0];
        BOOST_CHECK_EQUAL(cell1.influx_coeff, 3.0 * dims[1]*dims[2]);
    }
 
@@ -950,7 +962,7 @@ AQUANCON
 )";
        const auto& [aquconn, grid] = load_aquifer(aq);
        const auto& dims = grid.getCellDims(0,14,2);
-       auto cell1 = aquconn[1][0];
+       auto cell1 = aquconn.getConnections(1)[0];
        BOOST_CHECK_EQUAL(cell1.influx_coeff, 4 * ( 100 + 3.0 * dims[1]*dims[2]));
    }
 
@@ -963,7 +975,7 @@ AQUANCON
 /
 )";
        const auto& [aquconn, grid] = load_aquifer(aq);
-       auto cell1 = aquconn[1][0];
+       auto cell1 = aquconn.getConnections(1)[0];
        BOOST_CHECK_EQUAL(cell1.influx_coeff, 2*(77 + 3*(0 + 4*100)));
    }
 }
