@@ -30,6 +30,7 @@
 
 #include <boost/test/unit_test.hpp>
 #include <boost/test/tools/floating_point_comparison.hpp>
+
 #include <opm/common/utility/OpmInputError.hpp>
 
 #include <opm/input/eclipse/Parser/Parser.hpp>
@@ -40,6 +41,7 @@
 #include <opm/input/eclipse/Deck/DeckSection.hpp>
 #include <opm/input/eclipse/Deck/Deck.hpp>
 #include <opm/input/eclipse/Deck/DeckKeyword.hpp>
+#include <opm/input/eclipse/EclipseState/EclipseState.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/TableManager.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/FieldPropsManager.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/EclipseGrid.hpp>
@@ -47,7 +49,7 @@
 #include <opm/input/eclipse/EclipseState/Runspec.hpp>
 #include <opm/input/eclipse/Schedule/Schedule.hpp>
 
-#include "src/opm/input/eclipse/EclipseState/Grid/FieldProps.hpp"
+#include <opm/input/eclipse/EclipseState/Grid/FieldProps.hpp>
 
 using namespace Opm;
 
@@ -133,6 +135,58 @@ PERMX
 }
 
 
+
+BOOST_AUTO_TEST_CASE(CreateFieldPropsForActnum) {
+    std::string deck_string = R"(
+GRID
+
+BOX
+  1 3 1 3 1 1 /
+
+ACTNUM
+   9*0 /
+
+BOX
+  1 3 1 2 1 1 /
+
+ACTNUM
+   6*1 /
+
+ENDBOX
+
+-- Re-enable (3,3,1)
+EQUALS
+    ACTNUM 1 3 3 3 3 1 1 /
+/
+)";
+
+    Deck deck = Parser{}.parseString(deck_string);
+    EclipseGrid grid{GridDims{3, 3, 1}};
+    FieldProps fp(deck, grid);
+    std::vector<int> expected_actnum = { 1, 1, 1,
+                                         1, 1, 1,
+                                         0, 0, 1  };
+    auto actnum = fp.actnumRaw();
+    BOOST_CHECK_EQUAL_COLLECTIONS(actnum.begin(), actnum.end(), expected_actnum.begin(), expected_actnum.end());
+}
+
+
+BOOST_AUTO_TEST_CASE(CreateFieldPropsForActnumButNoActnumInDeck) {
+    std::string deck_string = R"(
+GRID
+)";
+
+    Deck deck = Parser{}.parseString(deck_string);
+    EclipseGrid grid{GridDims{3, 3, 1}};
+    FieldProps fp(deck, grid);
+    std::vector<int> expected_actnum = { 1, 1, 1,
+                                         1, 1, 1,
+                                         1, 1, 1  };
+    auto actnum = fp.actnumRaw();
+    BOOST_CHECK_EQUAL_COLLECTIONS(actnum.begin(), actnum.end(), expected_actnum.begin(), expected_actnum.end());
+}
+
+
 BOOST_AUTO_TEST_CASE(INVALID_COPY) {
     std::string deck_string = R"(
 GRID
@@ -1987,11 +2041,7 @@ OPERATE
 )";
 
 
-
-
-
-
-    BOOST_CHECK_THROW(make_fp(invalid_copy), std::logic_error);
+    BOOST_CHECK_THROW(make_fp(invalid_copy), OpmInputError);
     BOOST_CHECK_THROW(make_fp(invalid_region), std::logic_error);
     BOOST_CHECK_THROW(make_fp(invalid_operate), std::logic_error);
 }
@@ -2083,12 +2133,6 @@ MINVALUE
             }
         }
     }
-    auto buffer = fpm.serialize_tran();
-    fpm.deserialize_tran(buffer);
-
-    BOOST_CHECK(fpm.tran_active("TRANX"));
-    BOOST_CHECK(fpm.tran_active("TRANY"));
-    BOOST_CHECK(fpm.tran_active("TRANZ"));
 }
 
 BOOST_AUTO_TEST_CASE(TRAN_KEYS) {
