@@ -434,6 +434,55 @@ WELLDIMS
     BOOST_CHECK_EQUAL(wd.maxGroupsInField(), 0);  // WELLDIMS(3), defaulted
 }
 
+BOOST_AUTO_TEST_CASE(WELLDIMS_MaxWList_Dflt)
+{
+    const auto input = std::string { R"(
+RUNSPEC
+
+WELLDIMS
+/
+)" };
+
+    const auto wd = Welldims {
+        Parser{}.parseString(input)
+    };
+
+    BOOST_CHECK_EQUAL(wd.maxWellListsPrWell(), 1); // WELLDIMS(11), defaulted
+}
+
+BOOST_AUTO_TEST_CASE(WELLDIMS_MaxWList_Assigned)
+{
+    const auto input = std::string { R"(
+RUNSPEC
+
+WELLDIMS
+10* 5 /
+)" };
+
+    const auto wd = Welldims {
+        Parser{}.parseString(input)
+    };
+
+    BOOST_CHECK_EQUAL(wd.maxWellListsPrWell(), 5); // WELLDIMS(11), assigned
+}
+
+BOOST_AUTO_TEST_CASE(WELLDIMS_MaxWList_Zero)
+{
+    const auto input = std::string { R"(
+RUNSPEC
+
+WELLDIMS
+10* 0 /
+)" };
+
+    const auto wd = Welldims {
+        Parser{}.parseString(input)
+    };
+
+    // WELLDIMS(11) assigned but reset to at least 1.
+    BOOST_CHECK_EQUAL(wd.maxWellListsPrWell(), 1);
+}
+
 BOOST_AUTO_TEST_CASE(WSEGDIMS_NotSpecified)
 {
     const auto input = std::string {
