@@ -38,10 +38,16 @@
 #include <opm/input/eclipse/EclipseState/Tables/Tabdims.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/PlyadsTable.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/PlymaxTable.hpp>
+#include <opm/input/eclipse/EclipseState/Tables/FlatTable.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/FoamadsTable.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/FoammobTable.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/PbvdTable.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/PdvdTable.hpp>
+#include <opm/input/eclipse/EclipseState/Tables/PvdgTable.hpp>
+#include <opm/input/eclipse/EclipseState/Tables/PvdoTable.hpp>
+#include <opm/input/eclipse/EclipseState/Tables/PvtgTable.hpp>
+#include <opm/input/eclipse/EclipseState/Tables/PvtoTable.hpp>
+#include <opm/input/eclipse/EclipseState/Tables/PvtxTable.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/DenT.hpp>
 
 #include <opm/input/eclipse/Schedule/VFPProdTable.hpp>
@@ -271,8 +277,8 @@ SWOF
 END
 )");
 
-    Opm::SwofTable swof1Table(deck["SWOF"].back().getRecord(0).getItem(0), false);
-    Opm::SwofTable swof2Table(deck["SWOF"].back().getRecord(1).getItem(0), false);
+    Opm::SwofTable swof1Table(deck["SWOF"].back().getRecord(0).getItem(0), false, 0);
+    Opm::SwofTable swof2Table(deck["SWOF"].back().getRecord(1).getItem(0), false, 1);
 
     BOOST_CHECK_EQUAL(swof1Table.numRows(), 2U);
     BOOST_CHECK_EQUAL(swof2Table.numRows(), 3U);
@@ -313,7 +319,7 @@ BOOST_AUTO_TEST_CASE(PbvdTable_Tests) {
     Opm::Parser parser;
     auto deck = parser.parseString(deckData);
 
-    Opm::PbvdTable pbvdTable1(deck["PBVD"].back().getRecord(0).getItem(0));
+    Opm::PbvdTable pbvdTable1(deck["PBVD"].back().getRecord(0).getItem(0), 0);
 
     BOOST_CHECK_EQUAL(pbvdTable1.numRows(), 2U);
     BOOST_CHECK_EQUAL(pbvdTable1.numColumns(), 2U);
@@ -322,7 +328,7 @@ BOOST_AUTO_TEST_CASE(PbvdTable_Tests) {
     BOOST_CHECK_EQUAL(pbvdTable1.getPbubColumn().front(), 100000); // 1 barsa
 
     // depth must be increasing down the column.
-    BOOST_CHECK_THROW(Opm::PbvdTable pbvdTable2(deck["PBVD"].back().getRecord(1).getItem(0)), std::invalid_argument);
+    BOOST_CHECK_THROW(Opm::PbvdTable pbvdTable2(deck["PBVD"].back().getRecord(1).getItem(0), 1), std::invalid_argument);
 }
 
 
@@ -341,7 +347,7 @@ BOOST_AUTO_TEST_CASE(PdvdTable_Tests) {
     Opm::Parser parser;
     auto deck = parser.parseString(deckData);
 
-    Opm::PdvdTable pdvdTable1(deck["PDVD"].back().getRecord(0).getItem(0));
+    Opm::PdvdTable pdvdTable1(deck["PDVD"].back().getRecord(0).getItem(0), 0);
 
     BOOST_CHECK_EQUAL(pdvdTable1.numRows(), 2U);
     BOOST_CHECK_EQUAL(pdvdTable1.numColumns(), 2U);
@@ -350,7 +356,7 @@ BOOST_AUTO_TEST_CASE(PdvdTable_Tests) {
     BOOST_CHECK_EQUAL(pdvdTable1.getPdewColumn().front(), 100000); // 1 barsa
 
     // depth must be increasing down the column.
-    BOOST_CHECK_THROW(Opm::PdvdTable pdvdTable2(deck["PDVD"].back().getRecord(1).getItem(0)), std::invalid_argument);
+    BOOST_CHECK_THROW(Opm::PdvdTable pdvdTable2(deck["PDVD"].back().getRecord(1).getItem(0), 1), std::invalid_argument);
 }
 
 BOOST_AUTO_TEST_CASE(SgwfnTable_Tests) {
@@ -369,8 +375,8 @@ BOOST_AUTO_TEST_CASE(SgwfnTable_Tests) {
     auto deck = parser.parseString(deckData);
 
 
-    Opm::SgwfnTable sgwfn1Table(deck["SGWFN"].back().getRecord(0).getItem(0));
-    Opm::SgwfnTable sgwfn2Table(deck["SGWFN"].back().getRecord(1).getItem(0));
+    Opm::SgwfnTable sgwfn1Table(deck["SGWFN"].back().getRecord(0).getItem(0), 0);
+    Opm::SgwfnTable sgwfn2Table(deck["SGWFN"].back().getRecord(1).getItem(0), 1);
 
     BOOST_CHECK_EQUAL(sgwfn1Table.numRows(), 2U);
     BOOST_CHECK_EQUAL(sgwfn2Table.numRows(), 3U);
@@ -414,8 +420,8 @@ SGOF
 END
 )");
 
-    Opm::SgofTable sgof1Table(deck["SGOF"].back().getRecord(0).getItem(0), false);
-    Opm::SgofTable sgof2Table(deck["SGOF"].back().getRecord(1).getItem(0), false);
+    Opm::SgofTable sgof1Table(deck["SGOF"].back().getRecord(0).getItem(0), false, 0);
+    Opm::SgofTable sgof2Table(deck["SGOF"].back().getRecord(1).getItem(0), false, 1);
 
     BOOST_CHECK_EQUAL(sgof1Table.numRows(), 2U);
     BOOST_CHECK_EQUAL(sgof2Table.numRows(), 3U);
@@ -460,7 +466,7 @@ BOOST_AUTO_TEST_CASE(PlyadsTable_Tests) {
         Opm::Parser parser;
         auto deck = parser.parseString(correctDeckData);
         const auto& plyadsKeyword = deck["PLYADS"].back();
-        Opm::PlyadsTable plyadsTable(plyadsKeyword.getRecord(0).getItem(0));
+        Opm::PlyadsTable plyadsTable(plyadsKeyword.getRecord(0).getItem(0), 0);
 
 
         BOOST_CHECK_CLOSE(plyadsTable.getPolymerConcentrationColumn().front(), 0.0, 1e-6);
@@ -490,7 +496,7 @@ BOOST_AUTO_TEST_CASE(PlyadsTable_Tests) {
         auto deck = parser.parseString(incorrectDeckData);
         const auto& plyadsKeyword = deck["PLYADS"].back();
 
-        BOOST_CHECK_THROW(Opm::PlyadsTable(plyadsKeyword.getRecord(0).getItem(0)), std::invalid_argument);
+        BOOST_CHECK_THROW(Opm::PlyadsTable(plyadsKeyword.getRecord(0).getItem(0), 0), std::invalid_argument);
     }
 
     {
@@ -513,7 +519,7 @@ BOOST_AUTO_TEST_CASE(PlyadsTable_Tests) {
         auto deck = parser.parseString(incorrectDeckData);
         const auto& plyadsKeyword = deck["PLYADS"].back();
 
-        BOOST_CHECK_THROW(Opm::PlyadsTable(plyadsKeyword.getRecord(0).getItem(0)), std::invalid_argument);
+        BOOST_CHECK_THROW(Opm::PlyadsTable(plyadsKeyword.getRecord(0).getItem(0), 0), std::invalid_argument);
     }
 }
 
@@ -536,7 +542,7 @@ BOOST_AUTO_TEST_CASE(FoamadsTable_Tests) {
         Opm::Parser parser;
         auto deck = parser.parseString(correctDeckData);
         const auto& foamadsKeyword = deck["FOAMADS"].back();
-        Opm::FoamadsTable foamadsTable(foamadsKeyword.getRecord(0).getItem(0));
+        Opm::FoamadsTable foamadsTable(foamadsKeyword.getRecord(0).getItem(0), 0);
 
 
         BOOST_CHECK_CLOSE(foamadsTable.getFoamConcentrationColumn().front(), 0.0, 1e-6);
@@ -566,7 +572,7 @@ BOOST_AUTO_TEST_CASE(FoamadsTable_Tests) {
         auto deck = parser.parseString(incorrectDeckData);
         const auto& foamadsKeyword = deck["FOAMADS"].back();
 
-        BOOST_CHECK_THROW(Opm::FoamadsTable(foamadsKeyword.getRecord(0).getItem(0)), std::invalid_argument);
+        BOOST_CHECK_THROW(Opm::FoamadsTable(foamadsKeyword.getRecord(0).getItem(0), 0), std::invalid_argument);
     }
 
     {
@@ -589,7 +595,7 @@ BOOST_AUTO_TEST_CASE(FoamadsTable_Tests) {
         auto deck = parser.parseString(incorrectDeckData);
         const auto& foamadsKeyword = deck["FOAMADS"].back();
 
-        BOOST_CHECK_THROW(Opm::FoamadsTable(foamadsKeyword.getRecord(0).getItem(0)), std::invalid_argument);
+        BOOST_CHECK_THROW(Opm::FoamadsTable(foamadsKeyword.getRecord(0).getItem(0), 0), std::invalid_argument);
     }
 }
 
@@ -606,7 +612,7 @@ BOOST_AUTO_TEST_CASE(FoammobTable_Tests) {
         Opm::Parser parser;
         auto deck = parser.parseString(correctDeckData);
         const auto& foammobKeyword = deck["FOAMMOB"].back();
-        Opm::FoammobTable foammobTable(foammobKeyword.getRecord(0).getItem(0));
+        Opm::FoammobTable foammobTable(foammobKeyword.getRecord(0).getItem(0), 0);
 
 
         BOOST_CHECK_CLOSE(foammobTable.getFoamConcentrationColumn().front(), 0.0, 1e-6);
@@ -630,7 +636,7 @@ BOOST_AUTO_TEST_CASE(FoammobTable_Tests) {
         auto deck = parser.parseString(incorrectDeckData);
         const auto& foammobKeyword = deck["FOAMMOB"].back();
 
-        BOOST_CHECK_THROW(Opm::FoammobTable(foammobKeyword.getRecord(0).getItem(0)), std::invalid_argument);
+        BOOST_CHECK_THROW(Opm::FoammobTable(foammobKeyword.getRecord(0).getItem(0), 0), std::invalid_argument);
     }
 
     {
@@ -647,12 +653,639 @@ BOOST_AUTO_TEST_CASE(FoammobTable_Tests) {
         auto deck = parser.parseString(incorrectDeckData);
         const auto& foammobKeyword = deck["FOAMMOB"].back();
 
-        BOOST_CHECK_THROW(Opm::FoammobTable(foammobKeyword.getRecord(0).getItem(0)), std::invalid_argument);
+        BOOST_CHECK_THROW(Opm::FoammobTable(foammobKeyword.getRecord(0).getItem(0), 0), std::invalid_argument);
     }
 }
 
+BOOST_AUTO_TEST_CASE(PvdoTable_Tests) {
+    // PVDO tables from opm-tests/model6/0_BASE_MODEL6.DATA .
+    const auto deck = Opm::Parser{}.parseString(R"(RUNSPEC
+OIL
+WATER
+TABDIMS
+1 2 /
+PROPS
+DENSITY
+   924.1      1026.0      1.03446 /
+   924.1      1026.0      1.03446 /
+PVDO
+ 23.0  1.10770  52.630
+ 27.5  1.08610  53.660
+ 32.1  1.06460  54.730
+ 50.0  1.06350  58.940
+/
+/ -- Copied from table 1
+END
+)");
 
+    const auto tmgr = Opm::TableManager { deck };
+    const auto& pvdo = tmgr.getPvdoTables();
+    BOOST_REQUIRE_EQUAL(pvdo.size(), std::size_t{2});
 
+    {
+        const auto& t1 = pvdo.getTable<PvdoTable>(0);
+
+        const auto& p = t1.getPressureColumn();
+        BOOST_REQUIRE_EQUAL(p.size(), std::size_t{4});
+        BOOST_CHECK_CLOSE(p[0], 2.3e6, 1.0e-8);
+        BOOST_CHECK_CLOSE(p[1], 2.75e6, 1.0e-8);
+        BOOST_CHECK_CLOSE(p[2], 3.21e6, 1.0e-8);
+        BOOST_CHECK_CLOSE(p[3], 5.0e6, 1.0e-8);
+
+        const auto& B = t1.getFormationFactorColumn();
+        BOOST_REQUIRE_EQUAL(B.size(), std::size_t{4});
+        BOOST_CHECK_CLOSE(B[0], 1.10770, 1.0e-8);
+        BOOST_CHECK_CLOSE(B[1], 1.08610, 1.0e-8);
+        BOOST_CHECK_CLOSE(B[2], 1.06460, 1.0e-8);
+        BOOST_CHECK_CLOSE(B[3], 1.06350, 1.0e-8);
+
+        const auto& mu = t1.getViscosityColumn();
+        BOOST_REQUIRE_EQUAL(mu.size(), std::size_t{4});
+        BOOST_CHECK_CLOSE(mu[0], 52.630e-3, 1.0e-8);
+        BOOST_CHECK_CLOSE(mu[1], 53.660e-3, 1.0e-8);
+        BOOST_CHECK_CLOSE(mu[2], 54.730e-3, 1.0e-8);
+        BOOST_CHECK_CLOSE(mu[3], 58.940e-3, 1.0e-8);
+    }
+
+    {
+        const auto& t2 = pvdo.getTable<PvdoTable>(1);
+
+        const auto& p = t2.getPressureColumn();
+        BOOST_REQUIRE_EQUAL(p.size(), std::size_t{4});
+        BOOST_CHECK_CLOSE(p[0], 2.3e6, 1.0e-8);
+        BOOST_CHECK_CLOSE(p[1], 2.75e6, 1.0e-8);
+        BOOST_CHECK_CLOSE(p[2], 3.21e6, 1.0e-8);
+        BOOST_CHECK_CLOSE(p[3], 5.0e6, 1.0e-8);
+
+        const auto& B = t2.getFormationFactorColumn();
+        BOOST_REQUIRE_EQUAL(B.size(), std::size_t{4});
+        BOOST_CHECK_CLOSE(B[0], 1.10770, 1.0e-8);
+        BOOST_CHECK_CLOSE(B[1], 1.08610, 1.0e-8);
+        BOOST_CHECK_CLOSE(B[2], 1.06460, 1.0e-8);
+        BOOST_CHECK_CLOSE(B[3], 1.06350, 1.0e-8);
+
+        const auto& mu = t2.getViscosityColumn();
+        BOOST_REQUIRE_EQUAL(mu.size(), std::size_t{4});
+        BOOST_CHECK_CLOSE(mu[0], 52.630e-3, 1.0e-8);
+        BOOST_CHECK_CLOSE(mu[1], 53.660e-3, 1.0e-8);
+        BOOST_CHECK_CLOSE(mu[2], 54.730e-3, 1.0e-8);
+        BOOST_CHECK_CLOSE(mu[3], 58.940e-3, 1.0e-8);
+    }
+}
+
+BOOST_AUTO_TEST_CASE(PvdgTable_Tests) {
+    // PVT tables from opm-tests/model5/include/pvt_live_oil_dgas.ecl .
+    const auto deck = Opm::Parser{}.parseString(R"(RUNSPEC
+OIL
+WATER
+TABDIMS
+1 2 /
+PROPS
+DENSITY
+   924.1      1026.0      1.03446 /
+   924.1      1026.0      1.03446 /
+PVDG
+-- Table number: 1
+    10.0000    0.266161     0.0108
+    15.0000    0.127259     0.0116
+    25.0000    0.062022     0.0123 /
+/ -- Copied from table 1
+END
+)");
+
+    const auto tmgr = Opm::TableManager { deck };
+    const auto& pvdg = tmgr.getPvdgTables();
+    BOOST_REQUIRE_EQUAL(pvdg.size(), std::size_t{2});
+
+    {
+        const auto& t1 = pvdg.getTable<PvdgTable>(0);
+
+        const auto& p = t1.getPressureColumn();
+        BOOST_REQUIRE_EQUAL(p.size(), std::size_t{3});
+        BOOST_CHECK_CLOSE(p[0], 1.0e6, 1.0e-8);
+        BOOST_CHECK_CLOSE(p[1], 1.5e6, 1.0e-8);
+        BOOST_CHECK_CLOSE(p[2], 2.5e6, 1.0e-8);
+
+        const auto& B = t1.getFormationFactorColumn();
+        BOOST_REQUIRE_EQUAL(B.size(), std::size_t{3});
+        BOOST_CHECK_CLOSE(B[0], 0.266161, 1.0e-8);
+        BOOST_CHECK_CLOSE(B[1], 0.127259, 1.0e-8);
+        BOOST_CHECK_CLOSE(B[2], 0.062022, 1.0e-8);
+
+        const auto& mu = t1.getViscosityColumn();
+        BOOST_REQUIRE_EQUAL(mu.size(), std::size_t{3});
+        BOOST_CHECK_CLOSE(mu[0], 0.0108e-3, 1.0e-8);
+        BOOST_CHECK_CLOSE(mu[1], 0.0116e-3, 1.0e-8);
+        BOOST_CHECK_CLOSE(mu[2], 0.0123e-3, 1.0e-8);
+    }
+
+    {
+        const auto& t2 = pvdg.getTable<PvdgTable>(1);
+
+        const auto& p = t2.getPressureColumn();
+        BOOST_REQUIRE_EQUAL(p.size(), std::size_t{3});
+        BOOST_CHECK_CLOSE(p[0], 1.0e6, 1.0e-8);
+        BOOST_CHECK_CLOSE(p[1], 1.5e6, 1.0e-8);
+        BOOST_CHECK_CLOSE(p[2], 2.5e6, 1.0e-8);
+
+        const auto& B = t2.getFormationFactorColumn();
+        BOOST_REQUIRE_EQUAL(B.size(), std::size_t{3});
+        BOOST_CHECK_CLOSE(B[0], 0.266161, 1.0e-8);
+        BOOST_CHECK_CLOSE(B[1], 0.127259, 1.0e-8);
+        BOOST_CHECK_CLOSE(B[2], 0.062022, 1.0e-8);
+
+        const auto& mu = t2.getViscosityColumn();
+        BOOST_REQUIRE_EQUAL(mu.size(), std::size_t{3});
+        BOOST_CHECK_CLOSE(mu[0], 0.0108e-3, 1.0e-8);
+        BOOST_CHECK_CLOSE(mu[1], 0.0116e-3, 1.0e-8);
+        BOOST_CHECK_CLOSE(mu[2], 0.0123e-3, 1.0e-8);
+    }
+}
+
+BOOST_AUTO_TEST_CASE(PvtoTable_Tests) {
+    // PVT tables from opm-tests/model5/include/pvt_live_oil_dgas.ecl .
+    const auto deck = Opm::Parser{}.parseString(R"(RUNSPEC
+OIL
+GAS
+TABDIMS
+1 2 /
+PROPS
+DENSITY
+   924.1      1026.0      1.03446 /
+   924.1      1026.0      1.03446 /
+PVTO
+-- Table number: 1
+     3.9140      10.000   1.102358     2.8625
+                 15.000   1.101766     2.9007
+                 25.000   1.100611     2.9695 /
+
+     7.0500      15.000   1.112540     2.6589
+                 25.000   1.111313     2.7221
+                 45.000   1.108952     2.8374 /
+/
+/ -- Copied from region 1
+END
+)");
+
+    const auto tmgr = Opm::TableManager { deck };
+    const auto& pvto = tmgr.getPvtoTables();
+    BOOST_REQUIRE_EQUAL(pvto.size(), std::size_t{2});
+
+    {
+        const auto& t1 = pvto[0];
+
+        BOOST_REQUIRE_EQUAL(t1.size(), std::size_t{2});
+
+        const auto& satTbl = t1.getSaturatedTable();
+        {
+            BOOST_REQUIRE_EQUAL(satTbl.numRows(), std::size_t{2});
+            BOOST_REQUIRE_EQUAL(satTbl.numColumns(), std::size_t{4});
+
+            const auto& rs = satTbl.getColumn(0);
+            BOOST_CHECK_CLOSE(rs[0], 3.914, 1.0e-8);
+            BOOST_CHECK_CLOSE(rs[1], 7.05, 1.0e-8);
+
+            const auto& p = satTbl.getColumn(1);
+            BOOST_CHECK_CLOSE(p[0], 1.0e6, 1.0e-8);
+            BOOST_CHECK_CLOSE(p[1], 1.5e6, 1.0e-8);
+
+            const auto& B = satTbl.getColumn(2);
+            BOOST_CHECK_CLOSE(B[0], 1.102358, 1.0e-8);
+            BOOST_CHECK_CLOSE(B[1], 1.11254, 1.0e-8);
+
+            const auto& mu = satTbl.getColumn(3);
+            BOOST_CHECK_CLOSE(mu[0], 2.8625e-3, 1.0e-8);
+            BOOST_CHECK_CLOSE(mu[1], 2.6589e-3, 1.0e-8);
+        }
+
+        {
+            const auto& u1 = t1.getUnderSaturatedTable(0);
+            BOOST_REQUIRE_EQUAL(u1.numRows(), std::size_t{3});
+            BOOST_REQUIRE_EQUAL(u1.numColumns(), std::size_t{3});
+
+            const auto& p = u1.getColumn(0);
+            BOOST_REQUIRE_EQUAL(p.size(), std::size_t{3});
+            BOOST_CHECK_CLOSE(p[0], 1.0e6, 1.0e-8);
+            BOOST_CHECK_CLOSE(p[1], 1.5e6, 1.0e-8);
+            BOOST_CHECK_CLOSE(p[2], 2.5e6, 1.0e-8);
+
+            const auto& B = u1.getColumn(1);
+            BOOST_REQUIRE_EQUAL(B.size(), std::size_t{3});
+            BOOST_CHECK_CLOSE(B[0], 1.102358, 1.0e-8);
+            BOOST_CHECK_CLOSE(B[1], 1.101766, 1.0e-8);
+            BOOST_CHECK_CLOSE(B[2], 1.100611, 1.0e-8);
+
+            const auto& mu = u1.getColumn(2);
+            BOOST_REQUIRE_EQUAL(mu.size(), std::size_t{3});
+            BOOST_CHECK_CLOSE(mu[0], 2.8625e-3, 1.0e-8);
+            BOOST_CHECK_CLOSE(mu[1], 2.9007e-3, 1.0e-8);
+            BOOST_CHECK_CLOSE(mu[2], 2.9695e-3, 1.0e-8);
+        }
+
+        {
+            const auto& u2 = t1.getUnderSaturatedTable(1);
+            BOOST_REQUIRE_EQUAL(u2.numRows(), std::size_t{3});
+            BOOST_REQUIRE_EQUAL(u2.numColumns(), std::size_t{3});
+
+            const auto& p = u2.getColumn(0);
+            BOOST_REQUIRE_EQUAL(p.size(), std::size_t{3});
+            BOOST_CHECK_CLOSE(p[0], 1.5e6, 1.0e-8);
+            BOOST_CHECK_CLOSE(p[1], 2.5e6, 1.0e-8);
+            BOOST_CHECK_CLOSE(p[2], 4.5e6, 1.0e-8);
+
+            const auto& B = u2.getColumn(1);
+            BOOST_REQUIRE_EQUAL(B.size(), std::size_t{3});
+            BOOST_CHECK_CLOSE(B[0], 1.112540, 1.0e-8);
+            BOOST_CHECK_CLOSE(B[1], 1.111313, 1.0e-8);
+            BOOST_CHECK_CLOSE(B[2], 1.108952, 1.0e-8);
+
+            const auto& mu = u2.getColumn(2);
+            BOOST_REQUIRE_EQUAL(mu.size(), std::size_t{3});
+            BOOST_CHECK_CLOSE(mu[0], 2.6589e-3, 1.0e-8);
+            BOOST_CHECK_CLOSE(mu[1], 2.7221e-3, 1.0e-8);
+            BOOST_CHECK_CLOSE(mu[2], 2.8374e-3, 1.0e-8);
+        }
+    }
+
+    {
+        const auto& t2 = pvto[1];
+
+        BOOST_REQUIRE_EQUAL(t2.size(), std::size_t{2});
+
+        const auto& satTbl = t2.getSaturatedTable();
+        {
+            BOOST_REQUIRE_EQUAL(satTbl.numRows(), std::size_t{2});
+            BOOST_REQUIRE_EQUAL(satTbl.numColumns(), std::size_t{4});
+
+            const auto& rs = satTbl.getColumn(0);
+            BOOST_CHECK_CLOSE(rs[0], 3.914, 1.0e-8);
+            BOOST_CHECK_CLOSE(rs[1], 7.05, 1.0e-8);
+
+            const auto& p = satTbl.getColumn(1);
+            BOOST_CHECK_CLOSE(p[0], 1.0e6, 1.0e-8);
+            BOOST_CHECK_CLOSE(p[1], 1.5e6, 1.0e-8);
+
+            const auto& B = satTbl.getColumn(2);
+            BOOST_CHECK_CLOSE(B[0], 1.102358, 1.0e-8);
+            BOOST_CHECK_CLOSE(B[1], 1.11254, 1.0e-8);
+
+            const auto& mu = satTbl.getColumn(3);
+            BOOST_CHECK_CLOSE(mu[0], 2.8625e-3, 1.0e-8);
+            BOOST_CHECK_CLOSE(mu[1], 2.6589e-3, 1.0e-8);
+        }
+
+        {
+            const auto& u1 = t2.getUnderSaturatedTable(0);
+            BOOST_REQUIRE_EQUAL(u1.numRows(), std::size_t{3});
+            BOOST_REQUIRE_EQUAL(u1.numColumns(), std::size_t{3});
+
+            const auto& p = u1.getColumn(0);
+            BOOST_REQUIRE_EQUAL(p.size(), std::size_t{3});
+            BOOST_CHECK_CLOSE(p[0], 1.0e6, 1.0e-8);
+            BOOST_CHECK_CLOSE(p[1], 1.5e6, 1.0e-8);
+            BOOST_CHECK_CLOSE(p[2], 2.5e6, 1.0e-8);
+
+            const auto& B = u1.getColumn(1);
+            BOOST_REQUIRE_EQUAL(B.size(), std::size_t{3});
+            BOOST_CHECK_CLOSE(B[0], 1.102358, 1.0e-8);
+            BOOST_CHECK_CLOSE(B[1], 1.101766, 1.0e-8);
+            BOOST_CHECK_CLOSE(B[2], 1.100611, 1.0e-8);
+
+            const auto& mu = u1.getColumn(2);
+            BOOST_REQUIRE_EQUAL(mu.size(), std::size_t{3});
+            BOOST_CHECK_CLOSE(mu[0], 2.8625e-3, 1.0e-8);
+            BOOST_CHECK_CLOSE(mu[1], 2.9007e-3, 1.0e-8);
+            BOOST_CHECK_CLOSE(mu[2], 2.9695e-3, 1.0e-8);
+        }
+
+        {
+            const auto& u2 = t2.getUnderSaturatedTable(1);
+            BOOST_REQUIRE_EQUAL(u2.numRows(), std::size_t{3});
+            BOOST_REQUIRE_EQUAL(u2.numColumns(), std::size_t{3});
+
+            const auto& p = u2.getColumn(0);
+            BOOST_REQUIRE_EQUAL(p.size(), std::size_t{3});
+            BOOST_CHECK_CLOSE(p[0], 1.5e6, 1.0e-8);
+            BOOST_CHECK_CLOSE(p[1], 2.5e6, 1.0e-8);
+            BOOST_CHECK_CLOSE(p[2], 4.5e6, 1.0e-8);
+
+            const auto& B = u2.getColumn(1);
+            BOOST_REQUIRE_EQUAL(B.size(), std::size_t{3});
+            BOOST_CHECK_CLOSE(B[0], 1.112540, 1.0e-8);
+            BOOST_CHECK_CLOSE(B[1], 1.111313, 1.0e-8);
+            BOOST_CHECK_CLOSE(B[2], 1.108952, 1.0e-8);
+
+            const auto& mu = u2.getColumn(2);
+            BOOST_REQUIRE_EQUAL(mu.size(), std::size_t{3});
+            BOOST_CHECK_CLOSE(mu[0], 2.6589e-3, 1.0e-8);
+            BOOST_CHECK_CLOSE(mu[1], 2.7221e-3, 1.0e-8);
+            BOOST_CHECK_CLOSE(mu[2], 2.8374e-3, 1.0e-8);
+        }
+    }
+}
+
+BOOST_AUTO_TEST_CASE(PvtgTable_Tests) {
+    // PVT tables from opm-tests/norne/INCLUDE/PVT/PVT-WET-GAS.INC .
+    const auto deck = Opm::Parser{}.parseString(R"(RUNSPEC
+OIL
+GAS
+TABDIMS
+1 2 /
+PROPS
+DENSITY
+   924.1      1026.0      1.03446 /
+   924.1      1026.0      1.03446 /
+PVTG
+-- Table number: 1
+     50.00    0.00000497   0.024958     0.01441
+              0.00000248   0.024958     0.01440
+              0.00000000   0.024958     0.01440 /
+
+     70.00    0.00000521   0.017639     0.01491
+              0.00000261   0.017641     0.01490
+              0.00000000   0.017643     0.01490 /
+/
+/ -- Copied from region 1
+END
+)");
+
+    const auto tmgr = Opm::TableManager { deck };
+    const auto& pvtg = tmgr.getPvtgTables();
+    BOOST_REQUIRE_EQUAL(pvtg.size(), std::size_t{2});
+
+    {
+        const auto& t1 = pvtg[0];
+
+        BOOST_REQUIRE_EQUAL(t1.size(), std::size_t{2});
+
+        const auto& satTbl = t1.getSaturatedTable();
+        {
+            BOOST_REQUIRE_EQUAL(satTbl.numRows(), std::size_t{2});
+            BOOST_REQUIRE_EQUAL(satTbl.numColumns(), std::size_t{4});
+
+            const auto& p = satTbl.getColumn(0);
+            BOOST_CHECK_CLOSE(p[0], 5.0e6, 1.0e-8);
+            BOOST_CHECK_CLOSE(p[1], 7.0e6, 1.0e-8);
+
+            const auto& rv = satTbl.getColumn(1);
+            BOOST_CHECK_CLOSE(rv[0], 0.00000497, 1.0e-8);
+            BOOST_CHECK_CLOSE(rv[1], 0.00000521, 1.0e-8);
+
+            const auto& B = satTbl.getColumn(2);
+            BOOST_CHECK_CLOSE(B[0], 0.024958, 1.0e-8);
+            BOOST_CHECK_CLOSE(B[1], 0.017639, 1.0e-8);
+
+            const auto& mu = satTbl.getColumn(3);
+            BOOST_CHECK_CLOSE(mu[0], 0.01441e-3, 1.0e-8);
+            BOOST_CHECK_CLOSE(mu[1], 0.01491e-3, 1.0e-8);
+        }
+
+        {
+            const auto& u1 = t1.getUnderSaturatedTable(0);
+            BOOST_REQUIRE_EQUAL(u1.numRows(), std::size_t{3});
+            BOOST_REQUIRE_EQUAL(u1.numColumns(), std::size_t{3});
+
+            const auto& rv = u1.getColumn(0);
+            BOOST_REQUIRE_EQUAL(rv.size(), std::size_t{3});
+            BOOST_CHECK_CLOSE(rv[0], 0.00000497, 1.0e-8);
+            BOOST_CHECK_CLOSE(rv[1], 0.00000248, 1.0e-8);
+            BOOST_CHECK_CLOSE(rv[2], 0.00000000, 1.0e-8);
+
+            const auto& B = u1.getColumn(1);
+            BOOST_REQUIRE_EQUAL(B.size(), std::size_t{3});
+            BOOST_CHECK_CLOSE(B[0], 0.024958, 1.0e-8);
+            BOOST_CHECK_CLOSE(B[1], 0.024958, 1.0e-8);
+            BOOST_CHECK_CLOSE(B[2], 0.024958, 1.0e-8);
+
+            const auto& mu = u1.getColumn(2);
+            BOOST_REQUIRE_EQUAL(mu.size(), std::size_t{3});
+            BOOST_CHECK_CLOSE(mu[0], 0.01441e-3, 1.0e-8);
+            BOOST_CHECK_CLOSE(mu[1], 0.01440e-3, 1.0e-8);
+            BOOST_CHECK_CLOSE(mu[2], 0.01440e-3, 1.0e-8);
+        }
+
+        {
+            const auto& u2 = t1.getUnderSaturatedTable(1);
+            BOOST_REQUIRE_EQUAL(u2.numRows(), std::size_t{3});
+            BOOST_REQUIRE_EQUAL(u2.numColumns(), std::size_t{3});
+
+            const auto& rv = u2.getColumn(0);
+            BOOST_REQUIRE_EQUAL(rv.size(), std::size_t{3});
+            BOOST_CHECK_CLOSE(rv[0], 0.00000521, 1.0e-8);
+            BOOST_CHECK_CLOSE(rv[1], 0.00000261, 1.0e-8);
+            BOOST_CHECK_CLOSE(rv[2], 0.00000000, 1.0e-8);
+
+            const auto& B = u2.getColumn(1);
+            BOOST_REQUIRE_EQUAL(B.size(), std::size_t{3});
+            BOOST_CHECK_CLOSE(B[0], 0.017639, 1.0e-8);
+            BOOST_CHECK_CLOSE(B[1], 0.017641, 1.0e-8);
+            BOOST_CHECK_CLOSE(B[2], 0.017643, 1.0e-8);
+
+            const auto& mu = u2.getColumn(2);
+            BOOST_REQUIRE_EQUAL(mu.size(), std::size_t{3});
+            BOOST_CHECK_CLOSE(mu[0], 0.01491e-3, 1.0e-8);
+            BOOST_CHECK_CLOSE(mu[1], 0.01490e-3, 1.0e-8);
+            BOOST_CHECK_CLOSE(mu[2], 0.01490e-3, 1.0e-8);
+        }
+    }
+
+    {
+        const auto& t2 = pvtg[1];
+
+        BOOST_REQUIRE_EQUAL(t2.size(), std::size_t{2});
+
+        const auto& satTbl = t2.getSaturatedTable();
+        {
+            BOOST_REQUIRE_EQUAL(satTbl.numRows(), std::size_t{2});
+            BOOST_REQUIRE_EQUAL(satTbl.numColumns(), std::size_t{4});
+
+            const auto& p = satTbl.getColumn(0);
+            BOOST_CHECK_CLOSE(p[0], 5.0e6, 1.0e-8);
+            BOOST_CHECK_CLOSE(p[1], 7.0e6, 1.0e-8);
+
+            const auto& rv = satTbl.getColumn(1);
+            BOOST_CHECK_CLOSE(rv[0], 0.00000497, 1.0e-8);
+            BOOST_CHECK_CLOSE(rv[1], 0.00000521, 1.0e-8);
+
+            const auto& B = satTbl.getColumn(2);
+            BOOST_CHECK_CLOSE(B[0], 0.024958, 1.0e-8);
+            BOOST_CHECK_CLOSE(B[1], 0.017639, 1.0e-8);
+
+            const auto& mu = satTbl.getColumn(3);
+            BOOST_CHECK_CLOSE(mu[0], 0.01441e-3, 1.0e-8);
+            BOOST_CHECK_CLOSE(mu[1], 0.01491e-3, 1.0e-8);
+        }
+
+        {
+            const auto& u1 = t2.getUnderSaturatedTable(0);
+            BOOST_REQUIRE_EQUAL(u1.numRows(), std::size_t{3});
+            BOOST_REQUIRE_EQUAL(u1.numColumns(), std::size_t{3});
+
+            const auto& rv = u1.getColumn(0);
+            BOOST_REQUIRE_EQUAL(rv.size(), std::size_t{3});
+            BOOST_CHECK_CLOSE(rv[0], 0.00000497, 1.0e-8);
+            BOOST_CHECK_CLOSE(rv[1], 0.00000248, 1.0e-8);
+            BOOST_CHECK_CLOSE(rv[2], 0.00000000, 1.0e-8);
+
+            const auto& B = u1.getColumn(1);
+            BOOST_REQUIRE_EQUAL(B.size(), std::size_t{3});
+            BOOST_CHECK_CLOSE(B[0], 0.024958, 1.0e-8);
+            BOOST_CHECK_CLOSE(B[1], 0.024958, 1.0e-8);
+            BOOST_CHECK_CLOSE(B[2], 0.024958, 1.0e-8);
+
+            const auto& mu = u1.getColumn(2);
+            BOOST_REQUIRE_EQUAL(mu.size(), std::size_t{3});
+            BOOST_CHECK_CLOSE(mu[0], 0.01441e-3, 1.0e-8);
+            BOOST_CHECK_CLOSE(mu[1], 0.01440e-3, 1.0e-8);
+            BOOST_CHECK_CLOSE(mu[2], 0.01440e-3, 1.0e-8);
+        }
+
+        {
+            const auto& u2 = t2.getUnderSaturatedTable(1);
+            BOOST_REQUIRE_EQUAL(u2.numRows(), std::size_t{3});
+            BOOST_REQUIRE_EQUAL(u2.numColumns(), std::size_t{3});
+
+            const auto& rv = u2.getColumn(0);
+            BOOST_REQUIRE_EQUAL(rv.size(), std::size_t{3});
+            BOOST_CHECK_CLOSE(rv[0], 0.00000521, 1.0e-8);
+            BOOST_CHECK_CLOSE(rv[1], 0.00000261, 1.0e-8);
+            BOOST_CHECK_CLOSE(rv[2], 0.00000000, 1.0e-8);
+
+            const auto& B = u2.getColumn(1);
+            BOOST_REQUIRE_EQUAL(B.size(), std::size_t{3});
+            BOOST_CHECK_CLOSE(B[0], 0.017639, 1.0e-8);
+            BOOST_CHECK_CLOSE(B[1], 0.017641, 1.0e-8);
+            BOOST_CHECK_CLOSE(B[2], 0.017643, 1.0e-8);
+
+            const auto& mu = u2.getColumn(2);
+            BOOST_REQUIRE_EQUAL(mu.size(), std::size_t{3});
+            BOOST_CHECK_CLOSE(mu[0], 0.01491e-3, 1.0e-8);
+            BOOST_CHECK_CLOSE(mu[1], 0.01490e-3, 1.0e-8);
+            BOOST_CHECK_CLOSE(mu[2], 0.01490e-3, 1.0e-8);
+        }
+    }
+}
+
+BOOST_AUTO_TEST_CASE(PvtwTable_Tests) {
+    // PVT tables from opm-tests/model5/include/pvt_live_oil_dgas.ecl .
+    const auto deck = Opm::Parser{}.parseString(R"(RUNSPEC
+OIL
+WATER
+TABDIMS
+1 2 /
+PROPS
+DENSITY
+   924.1      1026.0      1.03446 /
+   924.1      1026.0      1.03446 /
+PVTW
+   79.0  1.02643  0.37876E-04  0.39831  0.74714E-04 /
+/
+END
+)");
+
+    const auto tmgr = Opm::TableManager { deck };
+    const auto& pvtw = tmgr.getPvtwTable();
+    BOOST_REQUIRE_EQUAL(pvtw.size(), std::size_t{2});
+
+    {
+        const auto& t1 = pvtw[0];
+        BOOST_CHECK_CLOSE(t1.reference_pressure, 7.9e6, 1.0e-8);
+        BOOST_CHECK_CLOSE(t1.volume_factor, 1.02643, 1.0e-8);
+        BOOST_CHECK_CLOSE(t1.compressibility, 0.37876e-9, 1.0e-8);
+        BOOST_CHECK_CLOSE(t1.viscosity, 0.39831e-3, 1.0e-8);
+        BOOST_CHECK_CLOSE(t1.viscosibility, 0.74714e-9, 1.0e-9);
+    }
+
+    {
+        const auto& t2 = pvtw[1];
+        BOOST_CHECK_CLOSE(t2.reference_pressure, 7.9e6, 1.0e-8);
+        BOOST_CHECK_CLOSE(t2.volume_factor, 1.02643, 1.0e-8);
+        BOOST_CHECK_CLOSE(t2.compressibility, 0.37876e-9, 1.0e-8);
+        BOOST_CHECK_CLOSE(t2.viscosity, 0.39831e-3, 1.0e-8);
+        BOOST_CHECK_CLOSE(t2.viscosibility, 0.74714e-9, 1.0e-9);
+    }
+
+    const auto& dens = tmgr.getDensityTable();
+    BOOST_REQUIRE_EQUAL(dens.size(), std::size_t{2});
+
+    {
+        const auto& t1 = dens[0];
+        BOOST_CHECK_CLOSE(t1.oil, 924.1, 1.0e-8);
+        BOOST_CHECK_CLOSE(t1.gas, 1.03446, 1.0e-8);
+        BOOST_CHECK_CLOSE(t1.water, 1026.0, 1.0e-8);
+    }
+
+    {
+        const auto& t2 = dens[1];
+        BOOST_CHECK_CLOSE(t2.oil, 924.1, 1.0e-8);
+        BOOST_CHECK_CLOSE(t2.gas, 1.03446, 1.0e-8);
+        BOOST_CHECK_CLOSE(t2.water, 1026.0, 1.0e-8);
+    }
+}
+
+BOOST_AUTO_TEST_CASE(DensityTable_Tests) {
+    // PVT tables from opm-tests/model5/include/pvt_live_oil_dgas.ecl .
+    const auto deck = Opm::Parser{}.parseString(R"(RUNSPEC
+OIL
+WATER
+TABDIMS
+1 2 /
+PROPS
+DENSITY
+   924.1  1026.0  1.03446 /
+/ -- Copied from region 1
+END
+)");
+
+    const auto tmgr = Opm::TableManager { deck };
+    const auto& dens = tmgr.getDensityTable();
+    BOOST_REQUIRE_EQUAL(dens.size(), std::size_t{2});
+
+    {
+        const auto& t1 = dens[0];
+        BOOST_CHECK_CLOSE(t1.oil, 924.1, 1.0e-8);
+        BOOST_CHECK_CLOSE(t1.gas, 1.03446, 1.0e-8);
+        BOOST_CHECK_CLOSE(t1.water, 1026.0, 1.0e-8);
+    }
+
+    {
+        const auto& t2 = dens[1];
+        BOOST_CHECK_CLOSE(t2.oil, 924.1, 1.0e-8);
+        BOOST_CHECK_CLOSE(t2.gas, 1.03446, 1.0e-8);
+        BOOST_CHECK_CLOSE(t2.water, 1026.0, 1.0e-8);
+    }
+}
+
+BOOST_AUTO_TEST_CASE(GravityTable_Tests) {
+    const auto deck = Opm::Parser{}.parseString(R"(RUNSPEC
+OIL
+WATER
+TABDIMS
+1 2 /
+GRAVITY
+  12.34 1.2 1.21 /
+/ -- Copied from region 1
+END
+)");
+
+    const auto tmgr = Opm::TableManager { deck };
+    const auto& dens = tmgr.getDensityTable();
+    BOOST_REQUIRE_EQUAL(dens.size(), std::size_t{2});
+
+    {
+        const auto& t1 = dens[0];
+        BOOST_CHECK_CLOSE(  983.731924360, t1.oil  , 1.0e-8 );
+        BOOST_CHECK_CLOSE( 1200.0        , t1.water, 1.0e-8 );
+        BOOST_CHECK_CLOSE(    1.4762     , t1.gas  , 1.0e-8 );
+    }
+
+    {
+        const auto& t2 = dens[1];
+        BOOST_CHECK_CLOSE(  983.731924360, t2.oil  , 1.0e-8 );
+        BOOST_CHECK_CLOSE( 1200.0        , t2.water, 1.0e-8 );
+        BOOST_CHECK_CLOSE(    1.4762     , t2.gas  , 1.0e-8 );
+    }
+}
 
 /**
  * Tests "happy path" for a VFPPROD table, i.e., when everything goes well
