@@ -26,33 +26,38 @@
 #include <opm/output/eclipse/AggregateConnectionData.hpp>
 #include <opm/output/eclipse/AggregateGroupData.hpp>
 
-#include <opm/input/eclipse/Schedule/Action/State.hpp>
-#include <opm/input/eclipse/Schedule/SummaryState.hpp>
 #include <opm/output/eclipse/VectorItems/intehead.hpp>
 #include <opm/output/eclipse/VectorItems/well.hpp>
 #include <opm/output/eclipse/WriteRestartHelpers.hpp>
-#include <opm/common/utility/TimeService.hpp>
-
-#include <opm/io/eclipse/rst/well.hpp>
-#include <opm/io/eclipse/rst/header.hpp>
-#include <opm/input/eclipse/Python/Python.hpp>
 
 #include <opm/output/data/Wells.hpp>
 
-#include <opm/input/eclipse/Deck/Deck.hpp>
-#include <opm/input/eclipse/Parser/Parser.hpp>
-#include <opm/input/eclipse/EclipseState/EclipseState.hpp>
-#include <opm/input/eclipse/Schedule/Schedule.hpp>
-#include <opm/input/eclipse/Schedule/Action/State.hpp>
-#include <opm/input/eclipse/Schedule/Well/WellTestConfig.hpp>
-#include <opm/input/eclipse/Schedule/Well/WellTestState.hpp>
-#include <opm/io/eclipse/rst/state.hpp>
 #include <opm/io/eclipse/ERst.hpp>
 #include <opm/io/eclipse/RestartFileView.hpp>
 #include <opm/io/eclipse/OutputStream.hpp>
 
+#include <opm/io/eclipse/rst/well.hpp>
+#include <opm/io/eclipse/rst/header.hpp>
+#include <opm/io/eclipse/rst/state.hpp>
+
+#include <opm/input/eclipse/EclipseState/EclipseState.hpp>
+
+#include <opm/input/eclipse/Python/Python.hpp>
+
+#include <opm/input/eclipse/Schedule/Action/State.hpp>
+#include <opm/input/eclipse/Schedule/Schedule.hpp>
+#include <opm/input/eclipse/Schedule/SummaryState.hpp>
+#include <opm/input/eclipse/Schedule/Well/WellTestConfig.hpp>
+#include <opm/input/eclipse/Schedule/Well/WellTestState.hpp>
+
+#include <opm/input/eclipse/Parser/Parser.hpp>
+#include <opm/input/eclipse/Deck/Deck.hpp>
+
+#include <opm/common/utility/TimeService.hpp>
+
 #include <exception>
 #include <stdexcept>
+#include <string>
 #include <utility>
 #include <vector>
 
@@ -401,11 +406,250 @@ TSTEP            -- 9
         return Opm::Parser{}.parseString(input);
     }
 
-    Opm::Deck msw_sim(std::string fname) {
+    Opm::Deck wecon_etc_sim()
+    {
+        // Mostly copy of tests/FIRST_SIM.DATA
+        const auto input = std::string {
+            R"~(
+RUNSPEC
+OIL
+GAS
+WATER
+DISGAS
+UNIFOUT
+UNIFIN
+DIMENS
+ 10 10 10 /
+WELLDIMS
+ 6  20  1  6  /
+TABDIMS
+ 1  1  15  15  2  15  /
+FIELD
+EQLDIMS
+ 1  /
+
+GRID
+DXV
+10*100. /
+DYV
+10*100. /
+DZV
+10*100. /
+TOPS
+100*7000. /
+
+PORO
+1000*0.2 /
+
+PERMX
+1000*100. /
+
+PERMY
+1000*100. /
+
+PERMZ
+1000*10. /
+
+
+PROPS       ==========================================================
+
+-- WATER RELATIVE PERMEABILITY AND CAPILLARY PRESSURE ARE TABULATED AS
+-- A FUNCTION OF WATER SATURATION.
+--
+--  SWAT   KRW   PCOW
+SWFN
+
+    0.12  0       0
+    1.0   0.00001 0  /
+
+-- SIMILARLY FOR GAS
+--
+--  SGAS   KRG   PCOG
+SGFN
+
+    0     0       0
+    0.02  0       0
+    0.05  0.005   0
+    0.12  0.025   0
+    0.2   0.075   0
+    0.25  0.125   0
+    0.3   0.19    0
+    0.4   0.41    0
+    0.45  0.6     0
+    0.5   0.72    0
+    0.6   0.87    0
+    0.7   0.94    0
+    0.85  0.98    0
+    1.0   1.0     0
+/
+
+-- OIL RELATIVE PERMEABILITY IS TABULATED AGAINST OIL SATURATION
+-- FOR OIL-WATER AND OIL-GAS-CONNATE WATER CASES
+--
+--  SOIL     KROW     KROG
+SOF3
+
+    0        0        0
+    0.18     0        0
+    0.28     0.0001   0.0001
+    0.38     0.001    0.001
+    0.43     0.01     0.01
+    0.48     0.021    0.021
+    0.58     0.09     0.09
+    0.63     0.2      0.2
+    0.68     0.35     0.35
+    0.76     0.7      0.7
+    0.83     0.98     0.98
+    0.86     0.997    0.997
+    0.879    1        1
+    0.88     1        1    /
+
+
+-- PVT PROPERTIES OF WATER
+--
+--    REF. PRES. REF. FVF  COMPRESSIBILITY  REF VISCOSITY  VISCOSIBILITY
+PVTW
+       4014.7     1.029        3.13D-6           0.31            0 /
+
+-- ROCK COMPRESSIBILITY
+--
+--    REF. PRES   COMPRESSIBILITY
+ROCK
+        14.7          3.0D-6          /
+
+-- SURFACE DENSITIES OF RESERVOIR FLUIDS
+--
+--        OIL   WATER   GAS
+DENSITY
+         49.1   64.79  0.06054  /
+
+-- PVT PROPERTIES OF DRY GAS (NO VAPOURISED OIL)
+-- WE WOULD USE PVTG TO SPECIFY THE PROPERTIES OF WET GAS
+--
+--   PGAS   BGAS   VISGAS
+PVDG
+     14.7 166.666   0.008
+    264.7  12.093   0.0096
+    514.7   6.274   0.0112
+   1014.7   3.197   0.014
+   2014.7   1.614   0.0189
+   2514.7   1.294   0.0208
+   3014.7   1.080   0.0228
+   4014.7   0.811   0.0268
+   5014.7   0.649   0.0309
+   9014.7   0.386   0.047   /
+
+-- PVT PROPERTIES OF LIVE OIL (WITH DISSOLVED GAS)
+-- WE WOULD USE PVDO TO SPECIFY THE PROPERTIES OF DEAD OIL
+--
+-- FOR EACH VALUE OF RS THE SATURATION PRESSURE, FVF AND VISCOSITY
+-- ARE SPECIFIED. FOR RS=1.27 AND 1.618, THE FVF AND VISCOSITY OF
+-- UNDERSATURATED OIL ARE DEFINED AS A FUNCTION OF PRESSURE. DATA
+-- FOR UNDERSATURATED OIL MAY BE SUPPLIED FOR ANY RS, BUT MUST BE
+-- SUPPLIED FOR THE HIGHEST RS (1.618).
+--
+--   RS      POIL  FVFO  VISO
+PVTO
+    0.001    14.7 1.062  1.04    /
+    0.0905  264.7 1.15   0.975   /
+    0.18    514.7 1.207  0.91    /
+    0.371  1014.7 1.295  0.83    /
+    0.636  2014.7 1.435  0.695   /
+    0.775  2514.7 1.5    0.641   /
+    0.93   3014.7 1.565  0.594   /
+    1.270  4014.7 1.695  0.51
+           5014.7 1.671  0.549
+           9014.7 1.579  0.74    /
+    1.618  5014.7 1.827  0.449
+           9014.7 1.726  0.605   /
+/
+
+
+REGIONS    ===========================================================
+
+
+FIPNUM
+
+  1000*1
+/
+
+EQLNUM
+
+  1000*1
+/
+
+
+SOLUTION    ============================================================
+
+EQUIL
+7020.00 2700.00 7990.00  .00000 7200.00  .00000     0      0       5 /
+
+RSVD       2 TABLES    3 NODES IN EACH           FIELD   12:00 17 AUG 83
+   7000.0  1.0000
+   7990.0  1.0000
+/
+
+SCHEDULE
+RPTRST
+BASIC=1
+/
+DATES             -- 1
+ 10  OKT 2008 /
+/
+WELSPECS
+      'OP_1'       'OP'   9   9 1*     'OIL' 1*      1*  1*   1*  1*   1*  1*  /
+      'OP_2'       'OP'   9   9 1*     'OIL' 1*      1*  1*   1*  1*   1*  1*  /
+/
+COMPDAT
+      'OP_1'  9  9   1   1 'OPEN' 1*   32.948   0.311  3047.839 1*  1*  'X'  22.100 /
+      'OP_2'  9  9   2   2 'OPEN' 1*   46.825   0.311  4332.346 1*  1*  'X'  22.123 /
+      'OP_1'  9  9   3   3 'OPEN' 1*   32.948   0.311  3047.839 1*  1*  'X'  22.100 /
+/
+
+WTEST
+ 'OP_1' 1  PGD  3 2 /
+/
+
+WCONPROD
+      'OP_1' 'OPEN' 'ORAT' 20000  4* 1000 /
+/
+WCONINJE
+      'OP_2' 'GAS' 'OPEN' 'RATE' 100 200 400 /
+/
+
+DATES             -- 2
+ 20  JAN 2011 /
+/
+WELSPECS
+      'OP_3'       'OP'   9   9 1*     'OIL' 1*      1*  1*   1*  1*   1*  1*  /
+/
+COMPDAT
+      'OP_3'  9  9   1   1 'OPEN' 1*   32.948   0.311  3047.839 1*  1*  'X'  22.100 /
+/
+
+WECON
+ 'OP_1'   1.234 12.345 0.87 210.98 1*  WELL   NO /
+ 'OP_3'   1* 1* 1* 0.0 0.0 +CON  YES 1* POTN  0.56 PLUG 1* 10.23 /
+/
+
+WGRUPCON
+  OP_1 YES 123.456 GAS 0.75 /
+  OP_3 NO 100.0 /
+/
+
+TSTEP            -- 3
+10 /
+)~" };
+
+        return Opm::Parser{}.parseString(input);
+    }
+
+    Opm::Deck msw_sim(const std::string& fname)
+    {
         return Opm::Parser{}.parseFile(fname);
     }
 
-   Opm::SummaryState sim_state()
+    Opm::SummaryState sim_state()
     {
         auto state = Opm::SummaryState{Opm::TimeService::now()};
 
@@ -579,14 +823,12 @@ struct SimulationCase
     explicit SimulationCase(const Opm::Deck& deck)
         : es   { deck }
         , grid { deck }
-        , python{ std::make_shared<Opm::Python>()}
-        , sched{ deck, es, python }
+        , sched{ deck, es, std::make_shared<Opm::Python>() }
     {}
 
     // Order requirement: 'es' must be declared/initialised before 'sched'.
     Opm::EclipseState es;
     Opm::EclipseGrid  grid;
-    std::shared_ptr<Opm::Python> python;
     Opm::Schedule     sched;
 };
 
@@ -606,11 +848,15 @@ BOOST_AUTO_TEST_CASE (Constructor)
     BOOST_CHECK_EQUAL(awd.getZWell().size(), ih.nwells * ih.nzwelz);
 }
 
+// --------------------------------------------------------------------
+
 BOOST_AUTO_TEST_CASE (Declared_Well_Data)
 {
     const auto simCase = SimulationCase{first_sim()};
+
     Opm::Action::State action_state;
     Opm::WellTestState wtest_state;
+
     // Report Step 1: 2008-10-10 --> 2011-01-20
     const auto rptStep = std::size_t{1};
 
@@ -624,11 +870,18 @@ BOOST_AUTO_TEST_CASE (Declared_Well_Data)
     auto awd = Opm::RestartIO::Helpers::AggregateWellData{ih.value};
 
     wtest_state.close_well("OP_1", Opm::WellTestConfig::Reason::PHYSICAL, 0);
-    auto tw = wtest_state.test_wells(simCase.sched[rptStep].wtest_config(), 86400 * 10);
-    BOOST_CHECK(tw == std::vector<std::string>{"OP_1"});
+    {
+        auto tw = wtest_state.test_wells(simCase.sched[rptStep].wtest_config(), 86400 * 10);
+        BOOST_CHECK(tw == std::vector<std::string>{"OP_1"});
+    }
 
     awd.captureDeclaredWellData(simCase.sched,
-                                simCase.es.tracer(), rptStep, action_state, wtest_state, smry, ih.value);
+                                simCase.es.tracer(),
+                                rptStep,
+                                action_state,
+                                wtest_state,
+                                smry,
+                                ih.value);
 
     // IWEL (OP_1)
     {
@@ -651,7 +904,6 @@ BOOST_AUTO_TEST_CASE (Declared_Well_Data)
         BOOST_CHECK_EQUAL(iwell[start + Ix::CompOrd], 0); // Track ordering (default)
 
         BOOST_CHECK_EQUAL(iwell[start + Ix::item18], -100); // M2 Magic
-        BOOST_CHECK_EQUAL(iwell[start + Ix::item25], -  1); // M2 Magic
         BOOST_CHECK_EQUAL(iwell[start + Ix::item48], -  1); // M2 Magic
         BOOST_CHECK_EQUAL(iwell[start + Ix::item32],    7); // M2 Magic
     }
@@ -674,7 +926,6 @@ BOOST_AUTO_TEST_CASE (Declared_Well_Data)
         BOOST_CHECK_EQUAL(iwell[start + Ix::CompOrd], 0); // Track ordering (default)
 
         BOOST_CHECK_EQUAL(iwell[start + Ix::item18], -100); // M2 Magic
-        BOOST_CHECK_EQUAL(iwell[start + Ix::item25], -  1); // M2 Magic
         BOOST_CHECK_EQUAL(iwell[start + Ix::item48], -  1); // M2 Magic
         BOOST_CHECK_EQUAL(iwell[start + Ix::item32],    7); // M2 Magic
     }
@@ -766,15 +1017,19 @@ BOOST_AUTO_TEST_CASE (Declared_Well_Data)
 
         BOOST_CHECK_EQUAL(zwell[i1 + Ix::WellName].c_str(), "OP_2    ");
     }
+
     {
         WorkArea work;
         std::string outputDir = "./";
         std::string baseName = "TEST";
         {
-            Opm::EclIO::OutputStream::Restart rstFile {Opm::EclIO::OutputStream::ResultSet {outputDir, baseName},
-                                                       rptStep,
-                                                       Opm::EclIO::OutputStream::Formatted {false},
-                                                       Opm::EclIO::OutputStream::Unified {true}};
+            Opm::EclIO::OutputStream::Restart rstFile {
+                Opm::EclIO::OutputStream::ResultSet {outputDir, baseName},
+                rptStep,
+                Opm::EclIO::OutputStream::Formatted {false},
+                Opm::EclIO::OutputStream::Unified   {true}
+            };
+
             const double secs_elapsed = 100;
             const double next_step_size = 10;
 
@@ -782,8 +1037,10 @@ BOOST_AUTO_TEST_CASE (Declared_Well_Data)
                 createInteHead(simCase.es, simCase.es.getInputGrid(), simCase.sched, secs_elapsed,
                                rptStep, rptStep, rptStep);
 
-            const auto dh = Opm::RestartIO::Helpers::createDoubHead(simCase.es, simCase.sched, rptStep, rptStep+1,
-                                                                    secs_elapsed, next_step_size);
+            const auto dh =
+                Opm::RestartIO::Helpers::createDoubHead(simCase.es, simCase.sched,
+                                                        rptStep, rptStep+1,
+                                                        secs_elapsed, next_step_size);
 
             const auto& lh = Opm::RestartIO::Helpers::createLogiHead(simCase.es);
 
@@ -797,6 +1054,7 @@ BOOST_AUTO_TEST_CASE (Declared_Well_Data)
                 rstFile.write("XGRP", group_aggregator.getXGroup());
                 rstFile.write("ZGRP", group_aggregator.getZGroup());
             }
+
             rstFile.write("IWEL", awd.getIWell());
             rstFile.write("SWEL", awd.getSWell());
             rstFile.write("XWEL", awd.getXWell());
@@ -804,7 +1062,10 @@ BOOST_AUTO_TEST_CASE (Declared_Well_Data)
             {
                 auto conn_aggregator = Opm::RestartIO::Helpers::AggregateConnectionData(IH);
                 auto xw = Opm::data::Wells {};
-                conn_aggregator.captureDeclaredConnData(simCase.sched, simCase.es.getInputGrid(), simCase.es.getUnits(), xw, sim_state(), rptStep);
+                conn_aggregator.captureDeclaredConnData(simCase.sched, simCase.es.getInputGrid(),
+                                                        simCase.es.getUnits(), xw,
+                                                        sim_state(), rptStep);
+
                 rstFile.write("ICON", conn_aggregator.getIConn());
                 rstFile.write("SCON", conn_aggregator.getSConn());
                 rstFile.write("XCON", conn_aggregator.getXConn());
@@ -826,10 +1087,6 @@ BOOST_AUTO_TEST_CASE (Declared_Well_Data)
         }
     }
 
-
-
-
-
     // SWEL (OP_6)
     // Report Step 8: 2014-10-18 --> 2014-10-28
     const auto rptStep_8 = std::size_t{8};
@@ -843,7 +1100,12 @@ BOOST_AUTO_TEST_CASE (Declared_Well_Data)
     //smry = sim_state();
     awd = Opm::RestartIO::Helpers::AggregateWellData{ih_8.value};
     awd.captureDeclaredWellData(simCase.sched,
-                                simCase.es.tracer(), rptStep_8, action_state, wtest_state, smry, ih_8.value);
+                                simCase.es.tracer(),
+                                rptStep_8,
+                                action_state,
+                                wtest_state,
+                                smry,
+                                ih_8.value);
     {
         using Ix = ::Opm::RestartIO::Helpers::VectorItems::SWell::index;
 
@@ -862,24 +1124,237 @@ BOOST_AUTO_TEST_CASE (Declared_Well_Data)
     }
 }
 
+// --------------------------------------------------------------------
+
+BOOST_AUTO_TEST_CASE (WECON)
+{
+    const auto simCase = SimulationCase{wecon_etc_sim()};
+
+    const auto action_state = Opm::Action::State{};
+    const auto wtest_state = Opm::WellTestState{};
+
+    // Report Step 1: 2008-10-10 --> 2011-01-20
+    const auto rptStep = std::size_t{2};
+
+    const auto ih = MockIH {
+        static_cast<int>(simCase.sched.getWells(rptStep).size())
+    };
+
+    BOOST_CHECK_EQUAL(ih.nwells, MockIH::Sz{3});
+
+    const auto smry = sim_state();
+    auto awd = Opm::RestartIO::Helpers::AggregateWellData{ih.value};
+
+    awd.captureDeclaredWellData(simCase.sched,
+                                simCase.es.tracer(),
+                                rptStep,
+                                action_state,
+                                wtest_state,
+                                smry,
+                                ih.value);
+
+    // IWEL (OP_1)
+    {
+        using Ix = ::Opm::RestartIO::Helpers::VectorItems::IWell::index;
+        namespace EconValue = Opm::RestartIO::Helpers::VectorItems::IWell::Value::EconLimit;
+
+        const auto start = 0*ih.niwelz;
+
+        const auto& iwell = awd.getIWell();
+        BOOST_CHECK_EQUAL(iwell[start + Ix::EconWorkoverProcedure],
+                          EconValue::WOProcedure::StopOrShut);
+
+        BOOST_CHECK_EQUAL(iwell[start + Ix::EconLimitEndRun],
+                          EconValue::EndRun::No);
+
+        BOOST_CHECK_EQUAL(iwell[start + Ix::EconLimitQuantity],
+                          EconValue::Quantity::Rate);
+
+        BOOST_CHECK_EQUAL(iwell[start + Ix::EconWorkoverProcedure_2],
+                          EconValue::WOProcedure::StopOrShut);
+    }
+
+    // IWEL (OP_3)
+    {
+        using Ix = ::Opm::RestartIO::Helpers::VectorItems::IWell::index;
+        namespace EconValue = Opm::RestartIO::Helpers::VectorItems::IWell::Value::EconLimit;
+
+        const auto start = 2*ih.niwelz;
+
+        const auto& iwell = awd.getIWell();
+        BOOST_CHECK_EQUAL(iwell[start + Ix::EconWorkoverProcedure],
+                          EconValue::WOProcedure::ConAndBelow);
+
+        BOOST_CHECK_EQUAL(iwell[start + Ix::EconLimitEndRun],
+                          EconValue::EndRun::Yes);
+
+        BOOST_CHECK_EQUAL(iwell[start + Ix::EconLimitQuantity],
+                          EconValue::Quantity::Potential);
+
+        BOOST_CHECK_EQUAL(iwell[start + Ix::EconWorkoverProcedure_2],
+                          EconValue::WOProcedure::Plug);
+    }
+
+    // SWEL (OP_1)
+    {
+        using Ix = ::Opm::RestartIO::Helpers::VectorItems::SWell::index;
+
+        const auto i0 = 0*ih.nswelz;
+
+        const auto& swell = awd.getSWell();
+        BOOST_CHECK_CLOSE(swell[i0 + Ix::EconLimitMinOil], 1.234f, 1.0e-7f);
+        BOOST_CHECK_CLOSE(swell[i0 + Ix::EconLimitMinGas], 12.345f, 1.0e-7f);
+        BOOST_CHECK_CLOSE(swell[i0 + Ix::EconLimitMaxWct], 0.87f, 1.0e-7f);
+        BOOST_CHECK_CLOSE(swell[i0 + Ix::EconLimitMaxGor], 210.98f, 1.0e-7f);
+        BOOST_CHECK_CLOSE(swell[i0 + Ix::EconLimitMaxWct_2], 0.0f, 1.0e-7f);
+        BOOST_CHECK_CLOSE(swell[i0 + Ix::EconLimitMinLiq], 0.0f, 1.0e-7f);
+    }
+
+    // SWEL (OP_3)
+    {
+        using Ix = ::Opm::RestartIO::Helpers::VectorItems::SWell::index;
+
+        const auto i2 = 2*ih.nswelz;
+
+        const auto& swell = awd.getSWell();
+        BOOST_CHECK_CLOSE(swell[i2 + Ix::EconLimitMinOil], 0.0f, 1.0e-7f);
+        BOOST_CHECK_CLOSE(swell[i2 + Ix::EconLimitMinGas], 0.0f, 1.0e-7f);
+        BOOST_CHECK_CLOSE(swell[i2 + Ix::EconLimitMaxWct], 1.0e+20f, 1.0e-7f);
+        BOOST_CHECK_CLOSE(swell[i2 + Ix::EconLimitMaxGor], 1.0e+20f, 1.0e-7f);
+        BOOST_CHECK_CLOSE(swell[i2 + Ix::EconLimitMaxWct_2], 0.56f, 1.0e-7f);
+        BOOST_CHECK_CLOSE(swell[i2 + Ix::EconLimitMinLiq], 10.23f, 1.0e-7f);
+    }
+}
+
+// --------------------------------------------------------------------
+
+BOOST_AUTO_TEST_CASE (WGRUPCON)
+{
+    const auto simCase = SimulationCase{wecon_etc_sim()};
+
+    const auto action_state = Opm::Action::State{};
+    const auto wtest_state = Opm::WellTestState{};
+
+    // Report Step 1: 2008-10-10 --> 2011-01-20
+    const auto rptStep = std::size_t{2};
+
+    const auto ih = MockIH {
+        static_cast<int>(simCase.sched.getWells(rptStep).size())
+    };
+
+    BOOST_CHECK_EQUAL(ih.nwells, MockIH::Sz{3});
+
+    const auto smry = sim_state();
+    auto awd = Opm::RestartIO::Helpers::AggregateWellData{ih.value};
+
+    awd.captureDeclaredWellData(simCase.sched,
+                                simCase.es.tracer(),
+                                rptStep,
+                                action_state,
+                                wtest_state,
+                                smry,
+                                ih.value);
+
+    // IWEL (OP_1)
+    {
+        using Ix = ::Opm::RestartIO::Helpers::VectorItems::IWell::index;
+        namespace GrupConValue = Opm::RestartIO::Helpers::VectorItems::IWell::Value::WGrupCon;
+
+        const auto start = 0*ih.niwelz;
+
+        const auto& iwell = awd.getIWell();
+        BOOST_CHECK_EQUAL(iwell[start + Ix::WGrupConControllable],
+                          GrupConValue::Controllable::Yes);
+
+        BOOST_CHECK_EQUAL(iwell[start + Ix::WGrupConGRPhase],
+                          GrupConValue::GRPhase::Gas);
+    }
+
+    // IWEL (OP_3)
+    {
+        using Ix = ::Opm::RestartIO::Helpers::VectorItems::IWell::index;
+        namespace GrupConValue = Opm::RestartIO::Helpers::VectorItems::IWell::Value::WGrupCon;
+
+        const auto start = 1*ih.niwelz;
+
+        const auto& iwell = awd.getIWell();
+        BOOST_CHECK_EQUAL(iwell[start + Ix::WGrupConControllable],
+                          GrupConValue::Controllable::Yes);
+
+        BOOST_CHECK_EQUAL(iwell[start + Ix::WGrupConGRPhase],
+                          GrupConValue::GRPhase::Defaulted);
+    }
+
+    // IWEL (OP_3)
+    {
+        using Ix = ::Opm::RestartIO::Helpers::VectorItems::IWell::index;
+        namespace GrupConValue = Opm::RestartIO::Helpers::VectorItems::IWell::Value::WGrupCon;
+
+        const auto start = 2*ih.niwelz;
+
+        const auto& iwell = awd.getIWell();
+        BOOST_CHECK_EQUAL(iwell[start + Ix::WGrupConControllable],
+                          GrupConValue::Controllable::No);
+
+        BOOST_CHECK_EQUAL(iwell[start + Ix::WGrupConGRPhase],
+                          GrupConValue::GRPhase::Defaulted);
+    }
+
+    // SWEL (OP_1)
+    {
+        using Ix = ::Opm::RestartIO::Helpers::VectorItems::SWell::index;
+
+        const auto i0 = 0*ih.nswelz;
+
+        const auto& swell = awd.getSWell();
+        BOOST_CHECK_CLOSE(swell[i0 + Ix::WGrupConGuideRate], 123.456f, 1.0e-7f);
+        BOOST_CHECK_CLOSE(swell[i0 + Ix::WGrupConGRScaling], 0.75f, 1.0e-7f);
+    }
+
+    // SWEL (OP_2)
+    {
+        using Ix = ::Opm::RestartIO::Helpers::VectorItems::SWell::index;
+
+        const auto i1 = 1*ih.nswelz;
+
+        const auto& swell = awd.getSWell();
+        BOOST_CHECK_CLOSE(swell[i1 + Ix::WGrupConGuideRate], -1.0e+20f, 1.0e-7f);
+        BOOST_CHECK_CLOSE(swell[i1 + Ix::WGrupConGRScaling], 1.0f, 1.0e-7f);
+    }
+
+    // SWEL (OP_3)
+    {
+        using Ix = ::Opm::RestartIO::Helpers::VectorItems::SWell::index;
+
+        const auto i2 = 2*ih.nswelz;
+
+        const auto& swell = awd.getSWell();
+        BOOST_CHECK_CLOSE(swell[i2 + Ix::WGrupConGuideRate], 100.0f, 1.0e-7f);
+        BOOST_CHECK_CLOSE(swell[i2 + Ix::WGrupConGRScaling], 1.0f, 1.0e-7f);
+    }
+}
+
+// --------------------------------------------------------------------
 
 BOOST_AUTO_TEST_CASE (Declared_Well_Data_MSW_well_data)
 {
     const auto simCase = SimulationCase{msw_sim("0A4_GRCTRL_LRAT_LRAT_GGR_BASE_MODEL2_MSW_ALL.DATA")};
-    Opm::EclipseState es    = simCase.es;
-    Opm::Schedule     sched = simCase.sched;
-    Opm::Action::State action_state;
-    Opm::WellTestState wtest_state;
     const auto rptStep = std::size_t{1};
 
     const auto ih = MockIH {
         static_cast<int>(simCase.sched.getWells(rptStep).size())
     };
+
     const auto smry = sim_state();
 
     auto awd = Opm::RestartIO::Helpers::AggregateWellData{ih.value};
     awd.captureDeclaredWellData(simCase.sched,
-                                simCase.es.tracer(), rptStep, action_state, wtest_state, smry, ih.value);
+                                simCase.es.tracer(),
+                                rptStep,
+                                Opm::Action::State{},
+                                Opm::WellTestState{},
+                                smry,
+                                ih.value);
 
     // IWEL (PROD1)
     {
@@ -919,8 +1394,6 @@ BOOST_AUTO_TEST_CASE (Declared_Well_Data_MSW_well_data)
         BOOST_CHECK_EQUAL(iwell[start + Ix::MSW_PlossMod], 2); // PROD3 - H-- => 2,
         BOOST_CHECK_EQUAL(iwell[start + Ix::MSW_MulPhaseMod] , 1); // PROD3 - HO => 0
     }
-
-
 }
 
 // --------------------------------------------------------------------
@@ -1257,49 +1730,61 @@ BOOST_AUTO_TEST_CASE (Dynamic_Well_Data_Step2)
     }
 }
 
+// --------------------------------------------------------------------
 
-BOOST_AUTO_TEST_CASE(WELL_POD) {
+BOOST_AUTO_TEST_CASE(WELL_POD)
+{
     const auto simCase = SimulationCase{first_sim()};
     const auto& units = simCase.es.getUnits();
+
     // Report Step 2: 2011-01-20 --> 2013-06-15
     const auto rptStep = std::size_t{2};
     const auto sim_step = rptStep - 1;
-    Opm::SummaryState sumState(Opm::TimeService::now());
-    Opm::WellTestState wtest_state;
-    const auto xw   = well_rates_1();
-    Opm::Action::State action_state;
 
-    const auto ih = Opm::RestartIO::Helpers::createInteHead(simCase.es,
-                                                            simCase.grid,
-                                                            simCase.sched,
-                                                            0,
-                                                            sim_step,
-                                                            sim_step,
-                                                            sim_step);
+    const auto sumState = Opm::SummaryState { Opm::TimeService::now() };
+
+    const auto xw = well_rates_1();
+    const auto ih =
+        Opm::RestartIO::Helpers::createInteHead(simCase.es,
+                                                simCase.grid,
+                                                simCase.sched,
+                                                0,
+                                                sim_step,
+                                                sim_step,
+                                                sim_step);
 
     auto wellData = Opm::RestartIO::Helpers::AggregateWellData(ih);
-    wellData.captureDeclaredWellData(simCase.sched, simCase.es.tracer(), sim_step, action_state, wtest_state, sumState, ih);
-    wellData.captureDynamicWellData(simCase.sched, simCase.es.tracer(), sim_step, xw , sumState);
+    wellData.captureDeclaredWellData(simCase.sched, simCase.es.tracer(), sim_step,
+                                     Opm::Action::State{}, Opm::WellTestState{},
+                                     sumState, ih);
+
+    wellData.captureDynamicWellData(simCase.sched, simCase.es.tracer(),
+                                    sim_step, xw, sumState);
 
     auto connectionData = Opm::RestartIO::Helpers::AggregateConnectionData(ih);
-    connectionData.captureDeclaredConnData(simCase.sched, simCase.grid, units, xw , sumState, sim_step);
+    connectionData.captureDeclaredConnData(simCase.sched, simCase.grid, units,
+                                           xw, sumState, sim_step);
 
     const auto& iwel = wellData.getIWell();
     const auto& swel = wellData.getSWell();
     const auto& xwel = wellData.getXWell();
-    const auto& zwel8 = wellData.getZWell();
 
     const auto& icon = connectionData.getIConn();
     const auto& scon = connectionData.getSConn();
     const auto& xcon = connectionData.getXConn();
 
-    Opm::RestartIO::RstHeader header(simCase.es.runspec(), units, ih, std::vector<bool>(100), std::vector<double>(1000));
-    std::vector<Opm::RestartIO::RstWell> wells;
-    std::vector<std::string> zwel;
-    for (const auto& s8: zwel8)
-        zwel.push_back(s8.c_str());
+    const auto header = Opm::RestartIO::RstHeader {
+        simCase.es.runspec(), units, ih, std::vector<bool>(100), std::vector<double>(1000)
+    };
 
-    for (auto iw = 0; iw < header.num_wells; iw++) {
+    std::vector<Opm::RestartIO::RstWell> wells;
+
+    auto zwel = std::vector<std::string>{};
+    for (const auto& s8 : wellData.getZWell()) {
+        zwel.push_back(s8.c_str());
+    }
+
+    for (auto iw = 0; iw < header.num_wells; ++iw) {
         std::size_t zwel_offset = header.nzwelz * iw;
         std::size_t iwel_offset = header.niwelz * iw;
         std::size_t swel_offset = header.nswelz * iw;
@@ -1318,7 +1803,6 @@ BOOST_AUTO_TEST_CASE(WELL_POD) {
                            icon.data() + icon_offset,
                            scon.data() + scon_offset,
                            xcon.data() + xcon_offset);
-
     }
 
     // Well OP2
