@@ -214,7 +214,28 @@ BOOST_AUTO_TEST_CASE(TestExtESmry_1) {
         BOOST_REQUIRE_CLOSE (smryVect[i], bpr_10103_ref[i], 0.01);
 
     ExtESmry esmry2("SPE1CASE1.ESMRY");
-    esmry2.loadData();
+    auto fopr2a = esmry2.get("FOPR");
+    esmry2.loadData({"FOPR"});
+    auto fopr2b = esmry2.get("FOPR");
+
+    BOOST_CHECK_EQUAL(fopr2a.size(), fopr2b.size());
+
+    ExtESmry esmry3("SPE1CASE1.ESMRY");
+    auto fopr3a = esmry3.get("FOPR");
+    esmry3.loadData();
+    auto fopr3b = esmry3.get("FOPR");
+
+    BOOST_CHECK_EQUAL(fopr3a.size(), fopr3b.size());
+
+    ExtESmry esmry4("SPE1CASE1.ESMRY");
+
+    // vector list holding two elements of FOPR, should not be loaded twice
+    std::vector<std::string> vect_list{"FOPR", "WBHP:PROD", "FOPR", "FGOR"};
+    esmry4.loadData(vect_list);
+    auto fopr4a = esmry4.get("FOPR");
+    auto fgor4a = esmry4.get("FGOR");
+
+    BOOST_CHECK_EQUAL(fopr4a.size(), fgor4a.size());
 }
 
 BOOST_AUTO_TEST_CASE(TestExtESmry_2) {
@@ -373,5 +394,24 @@ BOOST_AUTO_TEST_CASE(TestESmry_3) {
     for (unsigned int i=0;i< smryVect.size();i++)
         BOOST_REQUIRE_CLOSE (smryVect[i], bpr_10103_ref[i], 0.01);
 
-}
+    auto fopt = esmry1.get("FOPT");
 
+    // fopt vector not present in base run, should get zeros as not
+    // for time step 0, 1, ... 62
+
+    for (size_t n = 0; n < 63; n++)
+        BOOST_CHECK_EQUAL(fopt[n], 0.0);
+
+    std::vector<float> fopt_rst_ref = { 3.19319e+07, 3.2234e+07, 3.25642e+07, 3.28804e+07, 3.32039e+07, 3.35138e+07,
+            3.38309e+07, 3.41447e+07, 3.44453e+07, 3.47521e+07, 3.50441e+07, 3.53404e+07, 3.56317e+07,
+            3.58914e+07, 3.61752e+07, 3.6447e+07, 3.67249e+07, 3.6991e+07, 3.7263e+07, 3.75319e+07, 3.77893e+07,
+            3.80521e+07, 3.83033e+07, 3.85594e+07, 3.88122e+07, 3.9038e+07, 3.92847e+07, 3.95201e+07, 3.97599e+07,
+            3.99889e+07, 4.02226e+07, 4.04534e+07, 4.06741e+07, 4.08993e+07, 4.11148e+07, 4.13349e+07, 4.15525e+07,
+            4.17471e+07, 4.19604e+07, 4.21647e+07, 4.23735e+07, 4.25736e+07, 4.27781e+07, 4.29805e+07, 4.31744e+07,
+            4.33726e+07, 4.35625e+07, 4.37566e+07, 4.39487e+07, 4.41205e+07, 4.43087e+07, 4.4489e+07, 4.46735e+07,
+            4.48504e+07, 4.50315e+07, 4.52109e+07, 4.53828e+07, 4.55587e+07, 4.57272e+07, 4.58995e+07 };
+
+
+    for (size_t n = 63; n < fopt.size(); n++)
+        BOOST_REQUIRE_CLOSE(fopt[n], fopt_rst_ref[n-63], 0.01);
+}
