@@ -31,6 +31,7 @@
 #include <opm/output/data/Groups.hpp>
 #include <opm/input/eclipse/Python/Python.hpp>
 
+#include <opm/input/eclipse/EclipseState/EclipseState.hpp>
 #include <opm/input/eclipse/EclipseState/Tables/Eqldims.hpp>
 #include <opm/input/eclipse/Deck/Deck.hpp>
 #include <opm/input/eclipse/EclipseState/Grid/EclipseGrid.hpp>
@@ -454,8 +455,6 @@ void compare( const RestartValue& fst,
         for( ; first != fst.solution.data( key).end(); ++first, ++second )
             BOOST_CHECK_CLOSE( *first, *second, tol );
     }
-
-    BOOST_CHECK_EQUAL( fst.wells, snd.wells );
 }
 
 
@@ -595,9 +594,6 @@ void compare_equal( const RestartValue& fst,
         for( ; first != fst.solution.data( key ).end(); ++first, ++second )
           BOOST_CHECK_EQUAL( *first, *second);
     }
-
-    BOOST_CHECK_EQUAL( fst.wells, snd.wells );
-    //BOOST_CHECK_EQUAL( fst.extra, snd.extra );
 }
 
 BOOST_AUTO_TEST_CASE(EclipseReadWriteWellStateData_double) {
